@@ -3,25 +3,26 @@ package account
 import (
 	"errors"
 
-	"git.sr.ht/~rjarry/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 )
 
 type CheckMail struct{}
 
 func init() {
-	register(CheckMail{})
+	commands.Register(CheckMail{})
+}
+
+func (CheckMail) Context() commands.CommandContext {
+	return commands.ACCOUNT
 }
 
 func (CheckMail) Aliases() []string {
 	return []string{"check-mail"}
 }
 
-func (CheckMail) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (CheckMail) Execute(aerc *widgets.Aerc, args []string) error {
-	acct := aerc.SelectedAccount()
+func (CheckMail) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
 	}
