@@ -3,39 +3,40 @@ package account
 import (
 	"errors"
 
-	"git.sr.ht/~rjarry/aerc/lib/statusline"
-	"git.sr.ht/~rjarry/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
+	"git.sr.ht/~rjarry/aerc/lib/state"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type Connection struct{}
 
 func init() {
-	register(Connection{})
+	commands.Register(Connection{})
+}
+
+func (Connection) Context() commands.CommandContext {
+	return commands.ACCOUNT
 }
 
 func (Connection) Aliases() []string {
 	return []string{"connect", "disconnect"}
 }
 
-func (Connection) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (Connection) Execute(aerc *widgets.Aerc, args []string) error {
-	acct := aerc.SelectedAccount()
+func (c Connection) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
 	}
 	cb := func(msg types.WorkerMessage) {
-		acct.SetStatus(statusline.ConnectionActivity(""))
+		acct.SetStatus(state.ConnectionActivity(""))
 	}
 	if args[0] == "connect" {
 		acct.Worker().PostAction(&types.Connect{}, cb)
-		acct.SetStatus(statusline.ConnectionActivity("Connecting..."))
+		acct.SetStatus(state.ConnectionActivity("Connecting..."))
 	} else {
 		acct.Worker().PostAction(&types.Disconnect{}, cb)
-		acct.SetStatus(statusline.ConnectionActivity("Disconnecting..."))
+		acct.SetStatus(state.ConnectionActivity("Disconnecting..."))
 	}
 	return nil
 }
