@@ -2,30 +2,27 @@ package account
 
 import (
 	"errors"
-	"fmt"
 
-	"git.sr.ht/~rjarry/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 )
 
 type ExpandCollapseFolder struct{}
 
 func init() {
-	register(ExpandCollapseFolder{})
+	commands.Register(ExpandCollapseFolder{})
+}
+
+func (ExpandCollapseFolder) Context() commands.CommandContext {
+	return commands.ACCOUNT
 }
 
 func (ExpandCollapseFolder) Aliases() []string {
 	return []string{"expand-folder", "collapse-folder"}
 }
 
-func (ExpandCollapseFolder) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (ExpandCollapseFolder) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) > 1 {
-		return expandCollapseFolderUsage(args[0])
-	}
-	acct := aerc.SelectedAccount()
+func (ExpandCollapseFolder) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
 	}
@@ -36,7 +33,3 @@ func (ExpandCollapseFolder) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 	return nil
 }
-
-func expandCollapseFolderUsage(cmd string) error {
-	return fmt.Errorf("Usage: %s", cmd)
-}
