@@ -6,39 +6,40 @@ import (
 	"fmt"
 	"io"
 	"os"
-	"path/filepath"
 	"sync/atomic"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	mboxer "git.sr.ht/~rjarry/aerc/worker/mbox"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
-type ImportMbox struct{}
+type ImportMbox struct {
+	Filename string `opt:"filename" complete:"CompleteFilename"`
+}
 
 func init() {
-	register(ImportMbox{})
+	commands.Register(ImportMbox{})
+}
+
+func (ImportMbox) Context() commands.CommandContext {
+	return commands.ACCOUNT
 }
 
 func (ImportMbox) Aliases() []string {
 	return []string{"import-mbox"}
 }
 
-func (ImportMbox) Complete(aerc *widgets.Aerc, args []string) []string {
-	return commands.CompletePath(filepath.Join(args...))
+func (*ImportMbox) CompleteFilename(arg string) []string {
+	return commands.CompletePath(arg, false)
 }
 
-func (ImportMbox) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) != 2 {
-		return importFolderUsage(args[0])
-	}
-	filename := args[1]
-
-	acct := aerc.SelectedAccount()
+func (i ImportMbox) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
 	}
@@ -52,24 +53,25 @@ func (ImportMbox) Execute(aerc *widgets.Aerc, args []string) error {
 		return errors.New("No directory selected")
 	}
 
+	i.Filename = xdg.ExpandHome(i.Filename)
+
 	importFolder := func() {
 		defer log.PanicHandler()
-		statusInfo := fmt.Sprintln("Importing", filename, "to folder", folder)
-		aerc.PushStatus(statusInfo, 10*time.Second)
+		statusInfo := fmt.Sprintln("Importing", i.Filename, "to folder", folder)
+		app.PushStatus(statusInfo, 10*time.Second)
 		log.Debugf(statusInfo)
-		f, err := os.Open(filename)
+		f, err := os.Open(i.Filename)
 		if err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 			return
 		}
 		defer f.Close()
 
 		messages, err := mboxer.Read(f)
 		if err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 			return
 		}
-		worker := acct.Worker()
 
 		var appended uint32
 		for i, m := range messages {
@@ -83,27 +85,28 @@ func (ImportMbox) Execute(aerc *widgets.Aerc, args []string) error {
 					break
 				}
 				nbytes, _ := io.Copy(&buf, r)
-				worker.PostAction(&types.AppendMessage{
-					Destination: folder,
-					Flags:       models.SeenFlag,
-					Date:        time.Now(),
-					Reader:      &buf,
-					Length:      int(nbytes),
-				}, func(msg types.WorkerMessage) {
-					switch msg := msg.(type) {
-					case *types.Unsupported:
-						errMsg := fmt.Sprintf("%s: AppendMessage is unsupported", args[0])
-						log.Errorf(errMsg)
-						aerc.PushError(errMsg)
-						return
-					case *types.Error:
-						log.Errorf("AppendMessage failed: %v", msg.Error)
-						done <- false
-					case *types.Done:
-						atomic.AddUint32(&appended, 1)
-						done <- true
-					}
-				})
+				store.Append(
+					folder,
+					models.SeenFlag,
+					time.Now(),
+					&buf,
+					int(nbytes),
+					func(msg types.WorkerMessage) {
+						switch msg := msg.(type) {
+						case *types.Unsupported:
+							errMsg := fmt.Sprintf("%s: AppendMessage is unsupported", args[0])
+							log.Errorf(errMsg)
+							app.PushError(errMsg)
+							return
+						case *types.Error:
+							log.Errorf("AppendMessage failed: %v", msg.Error)
+							done <- false
+						case *types.Done:
+							atomic.AddUint32(&appended, 1)
+							done <- true
+						}
+					},
+				)
 
 				select {
 				case ok := <-done:
@@ -125,30 +128,25 @@ func (ImportMbox) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 		infoStr := fmt.Sprintf("%s: imported %d of %d successfully.", args[0], appended, len(messages))
 		log.Debugf(infoStr)
-		aerc.PushSuccess(infoStr)
+		app.PushSuccess(infoStr)
 	}
 
 	if len(store.Uids()) > 0 {
-		confirm := widgets.NewSelectorDialog(
+		confirm := app.NewSelectorDialog(
 			"Selected directory is not empty",
 			fmt.Sprintf("Import mbox file to %s anyways?", folder),
-			[]string{"No", "Yes"}, 0, aerc.SelectedAccountUiConfig(),
+			[]string{"No", "Yes"}, 0, app.SelectedAccountUiConfig(),
 			func(option string, err error) {
-				aerc.CloseDialog()
-				aerc.Invalidate()
+				app.CloseDialog()
 				if option == "Yes" {
 					go importFolder()
 				}
 			},
 		)
-		aerc.AddDialog(confirm)
+		app.AddDialog(confirm)
 	} else {
 		go importFolder()
 	}
 
 	return nil
 }
-
-func importFolderUsage(cmd string) error {
-	return fmt.Errorf("Usage: %s <filename>", cmd)
-}
