@@ -2,31 +2,28 @@ package account
 
 import (
 	"errors"
-	"fmt"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
 type NextPrevResult struct{}
 
 func init() {
-	register(NextPrevResult{})
+	commands.Register(NextPrevResult{})
+}
+
+func (NextPrevResult) Context() commands.CommandContext {
+	return commands.ACCOUNT
 }
 
 func (NextPrevResult) Aliases() []string {
 	return []string{"next-result", "prev-result"}
 }
 
-func (NextPrevResult) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (NextPrevResult) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) > 1 {
-		return nextPrevResultUsage(args[0])
-	}
-	acct := aerc.SelectedAccount()
+func (NextPrevResult) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
 	}
@@ -45,7 +42,3 @@ func (NextPrevResult) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 	return nil
 }
-
-func nextPrevResultUsage(cmd string) error {
-	return fmt.Errorf("Usage: %s [<n>[%%]]", cmd)
-}
