@@ -2,7 +2,7 @@ package compose
 
 import (
 	"bufio"
-	"errors"
+	"bytes"
 	"fmt"
 	"io"
 	"os"
@@ -10,49 +10,61 @@ import (
 	"path/filepath"
 	"strings"
 
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
-	"git.sr.ht/~rjarry/aerc/widgets"
-	"github.com/mitchellh/go-homedir"
+	"github.com/pkg/errors"
 )
 
-type Attach struct{}
+type Attach struct {
+	Menu bool   `opt:"-m"`
+	Name string `opt:"-r"`
+	Path string `opt:"path" required:"false" complete:"CompletePath"`
+	Args string `opt:"..." required:"false"`
+}
 
 func init() {
-	register(Attach{})
+	commands.Register(Attach{})
+}
+
+func (Attach) Context() commands.CommandContext {
+	return commands.COMPOSE
 }
 
 func (Attach) Aliases() []string {
 	return []string{"attach"}
 }
 
-func (Attach) Complete(aerc *widgets.Aerc, args []string) []string {
-	path := strings.Join(args, " ")
-	return commands.CompletePath(path)
+func (*Attach) CompletePath(arg string) []string {
+	return commands.CompletePath(arg, false)
 }
 
-func (a Attach) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) == 1 {
-		return fmt.Errorf("Usage: :attach <path>")
+func (a Attach) Execute(args []string) error {
+	if a.Menu && a.Name != "" {
+		return errors.New("-m and -r are mutually exclusive")
 	}
-
-	if args[1] == "-m" {
-		return a.openMenu(aerc, args[2:])
+	switch {
+	case a.Menu:
+		return a.openMenu()
+	case a.Name != "":
+		if a.Path == "" {
+			return errors.New("command is required")
+		}
+		return a.readCommand()
+	default:
+		if a.Args != "" {
+			return errors.New("only a single path is supported")
+		}
+		return a.addPath(a.Path)
 	}
-
-	return a.addPath(aerc, strings.Join(args[1:], " "))
 }
 
-func (a Attach) addPath(aerc *widgets.Aerc, path string) error {
-	path, err := homedir.Expand(path)
-	if err != nil {
-		log.Errorf("failed to expand path '%s': %v", path, err)
-		aerc.PushError(err.Error())
-		return err
-	}
-
+func (a Attach) addPath(path string) error {
+	path = xdg.ExpandHome(path)
 	attachments, err := filepath.Glob(path)
 	if err != nil && errors.Is(err, filepath.ErrBadPattern) {
 		log.Warnf("failed to parse as globbing pattern: %v", err)
@@ -72,17 +84,17 @@ func (a Attach) addPath(aerc *widgets.Aerc, path string) error {
 		}
 	}
 
-	composer, _ := aerc.SelectedTabContent().(*widgets.Composer)
+	composer, _ := app.SelectedTabContent().(*app.Composer)
 	for _, attach := range attachments {
 		log.Debugf("attaching '%s'", attach)
 
 		pathinfo, err := os.Stat(attach)
 		if err != nil {
 			log.Errorf("failed to stat file: %v", err)
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 			return err
 		} else if pathinfo.IsDir() && len(attachments) == 1 {
-			aerc.PushError("Attachment must be a file, not a directory")
+			app.PushError("Attachment must be a file, not a directory")
 			return nil
 		}
 
@@ -90,26 +102,22 @@ func (a Attach) addPath(aerc *widgets.Aerc, path string) error {
 	}
 
 	if len(attachments) == 1 {
-		aerc.PushSuccess(fmt.Sprintf("Attached %s", path))
+		app.PushSuccess(fmt.Sprintf("Attached %s", path))
 	} else {
-		aerc.PushSuccess(fmt.Sprintf("Attached %d files", len(attachments)))
+		app.PushSuccess(fmt.Sprintf("Attached %d files", len(attachments)))
 	}
 
 	return nil
 }
 
-func (a Attach) openMenu(aerc *widgets.Aerc, args []string) error {
+func (a Attach) openMenu() error {
 	filePickerCmd := config.Compose.FilePickerCmd
 	if filePickerCmd == "" {
 		return fmt.Errorf("no file-picker-cmd defined")
 	}
 
 	if strings.Contains(filePickerCmd, "%s") {
-		verb := ""
-		if len(args) > 0 {
-			verb = args[0]
-		}
-		filePickerCmd = strings.ReplaceAll(filePickerCmd, "%s", verb)
+		filePickerCmd = strings.ReplaceAll(filePickerCmd, "%s", a.Path)
 	}
 
 	picks, err := os.CreateTemp("", "aerc-filepicker-*")
@@ -117,10 +125,16 @@ func (a Attach) openMenu(aerc *widgets.Aerc, args []string) error {
 		return err
 	}
 
-	filepicker := exec.Command("sh", "-c", filePickerCmd+" >&3")
-	filepicker.ExtraFiles = append(filepicker.ExtraFiles, picks)
+	var filepicker *exec.Cmd
+	if strings.Contains(filePickerCmd, "%f") {
+		filePickerCmd = strings.ReplaceAll(filePickerCmd, "%f", picks.Name())
+		filepicker = exec.Command("sh", "-c", filePickerCmd)
+	} else {
+		filepicker = exec.Command("sh", "-c", filePickerCmd+" >&3")
+		filepicker.ExtraFiles = append(filepicker.ExtraFiles, picks)
+	}
 
-	t, err := widgets.NewTerminal(filepicker)
+	t, err := app.NewTerminal(filepicker)
 	if err != nil {
 		return err
 	}
@@ -135,7 +149,7 @@ func (a Attach) openMenu(aerc *widgets.Aerc, args []string) error {
 			}
 		}()
 
-		aerc.CloseDialog()
+		app.CloseDialog()
 
 		if err != nil {
 			log.Errorf("terminal closed with error: %v", err)
@@ -155,7 +169,7 @@ func (a Attach) openMenu(aerc *widgets.Aerc, args []string) error {
 				continue
 			}
 			log.Tracef("File picker attaches: %v", f)
-			err := a.addPath(aerc, f)
+			err := a.addPath(f)
 			if err != nil {
 				log.Errorf("attach failed for file %s: %v", f, err)
 			}
@@ -163,17 +177,37 @@ func (a Attach) openMenu(aerc *widgets.Aerc, args []string) error {
 		}
 	}
 
-	aerc.AddDialog(widgets.NewDialog(
-		ui.NewBox(t, "File Picker", "", aerc.SelectedAccountUiConfig()),
-		// start pos on screen
-		func(h int) int {
-			return h / 8
-		},
-		// dialog height
-		func(h int) int {
-			return h - 2*h/8
-		},
+	app.AddDialog(app.LargeDialog(
+		ui.NewBox(t, "File Picker", "", app.SelectedAccountUiConfig()),
 	))
 
 	return nil
 }
+
+func (a Attach) readCommand() error {
+	cmd := exec.Command("sh", "-c", a.Path+" "+a.Args)
+
+	data, err := cmd.Output()
+	if err != nil {
+		return errors.Wrap(err, "Output")
+	}
+
+	reader := bufio.NewReader(bytes.NewReader(data))
+
+	mimeType, mimeParams, err := lib.FindMimeType(a.Name, reader)
+	if err != nil {
+		return errors.Wrap(err, "FindMimeType")
+	}
+
+	mimeParams["name"] = a.Name
+
+	composer, _ := app.SelectedTabContent().(*app.Composer)
+	err = composer.AddPartAttachment(a.Name, mimeType, mimeParams, reader)
+	if err != nil {
+		return errors.Wrap(err, "AddPartAttachment")
+	}
+
+	app.PushSuccess(fmt.Sprintf("Attached %s", a.Name))
+
+	return nil
+}
