@@ -4,46 +4,60 @@ import (
 	"bytes"
 	"time"
 
-	"github.com/miolini/datacounter"
 	"github.com/pkg/errors"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
-type Postpone struct{}
+type Postpone struct {
+	Folder string `opt:"-t" complete:"CompleteFolder"`
+}
 
 func init() {
-	register(Postpone{})
+	commands.Register(Postpone{})
+}
+
+func (Postpone) Context() commands.CommandContext {
+	return commands.COMPOSE
 }
 
 func (Postpone) Aliases() []string {
 	return []string{"postpone"}
 }
 
-func (Postpone) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
+func (*Postpone) CompleteFolder(arg string) []string {
+	return commands.GetFolders(arg)
 }
 
-func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) != 1 {
-		return errors.New("Usage: postpone")
-	}
-	acct := aerc.SelectedAccount()
+func (p Postpone) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
 	}
-	tab := aerc.SelectedTab()
+	store := acct.Store()
+	if store == nil {
+		return errors.New("No message store selected")
+	}
+	tab := app.SelectedTab()
 	if tab == nil {
 		return errors.New("No tab selected")
 	}
-	composer, _ := tab.Content.(*widgets.Composer)
+	composer, _ := tab.Content.(*app.Composer)
 	config := composer.Config()
 	tabName := tab.Name
 
-	if config.Postpone == "" {
+	targetFolder := config.Postpone
+	if composer.RecalledFrom() != "" {
+		targetFolder = composer.RecalledFrom()
+	}
+	if p.Folder != "" {
+		targetFolder = p.Folder
+	}
+	if targetFolder == "" {
 		return errors.New("No Postpone location configured")
 	}
 
@@ -59,7 +73,7 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 	dirs := acct.Directories().List()
 	alreadyCreated := false
 	for _, dir := range dirs {
-		if dir == config.Postpone {
+		if dir == targetFolder {
 			alreadyCreated = true
 			break
 		}
@@ -74,46 +88,47 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 
 		errStr := <-errChan
 		if errStr != "" {
-			aerc.PushError(errStr)
+			app.PushError(errStr)
 			return
 		}
 
 		handleErr := func(err error) {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 			log.Errorf("Postponing failed: %v", err)
-			aerc.NewTab(composer, tabName)
+			app.NewTab(composer, tabName)
 		}
 
-		aerc.RemoveTab(composer, false)
-		var buf bytes.Buffer
-		ctr := datacounter.NewWriterCounter(&buf)
-		err = composer.WriteMessage(header, ctr)
+		app.RemoveTab(composer, false)
+		buf := &bytes.Buffer{}
+
+		err = composer.WriteMessage(header, buf)
 		if err != nil {
 			handleErr(errors.Wrap(err, "WriteMessage"))
 			return
 		}
-		nbytes := int(ctr.Count())
-		worker.PostAction(&types.AppendMessage{
-			Destination: config.Postpone,
-			Flags:       models.SeenFlag,
-			Date:        time.Now(),
-			Reader:      &buf,
-			Length:      int(nbytes),
-		}, func(msg types.WorkerMessage) {
-			switch msg := msg.(type) {
-			case *types.Done:
-				aerc.PushStatus("Message postponed.", 10*time.Second)
-				composer.Close()
-			case *types.Error:
-				handleErr(msg.Error)
-			}
-		})
+		store.Append(
+			targetFolder,
+			models.SeenFlag|models.DraftFlag,
+			time.Now(),
+			buf,
+			buf.Len(),
+			func(msg types.WorkerMessage) {
+				switch msg := msg.(type) {
+				case *types.Done:
+					app.PushStatus("Message postponed.", 10*time.Second)
+					composer.SetPostponed()
+					composer.Close()
+				case *types.Error:
+					handleErr(msg.Error)
+				}
+			},
+		)
 	}()
 
 	if !alreadyCreated {
 		// to synchronise the creating of the directory
 		worker.PostAction(&types.CreateDirectory{
-			Directory: config.Postpone,
+			Directory: targetFolder,
 		}, func(msg types.WorkerMessage) {
 			switch msg := msg.(type) {
 			case *types.Done:
