@@ -10,58 +10,73 @@ import (
 	"strings"
 	"time"
 
-	"git.sr.ht/~sircmpwn/getopt"
 	"github.com/emersion/go-sasl"
 	"github.com/emersion/go-smtp"
-	"github.com/google/shlex"
 	"github.com/pkg/errors"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/commands/mode"
+	"git.sr.ht/~rjarry/aerc/commands/msg"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/hooks"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"git.sr.ht/~rjarry/go-opt"
 	"github.com/emersion/go-message/mail"
 	"golang.org/x/oauth2"
 )
 
-type Send struct{}
+type Send struct {
+	Archive string `opt:"-a" action:"ParseArchive" metavar:"flat|year|month" complete:"CompleteArchive"`
+	CopyTo  string `opt:"-t" complete:"CompleteFolders"`
+}
 
 func init() {
-	register(Send{})
+	commands.Register(Send{})
+}
+
+func (Send) Context() commands.CommandContext {
+	return commands.COMPOSE
 }
 
 func (Send) Aliases() []string {
 	return []string{"send"}
 }
 
-func (Send) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
+func (*Send) CompleteArchive(arg string) []string {
+	return commands.FilterList(msg.ARCHIVE_TYPES, arg, nil)
 }
 
-func (Send) Execute(aerc *widgets.Aerc, args []string) error {
-	opts, optind, err := getopt.Getopts(args, "a:")
-	if err != nil {
-		return err
-	}
-	if optind != len(args) {
-		return errors.New("Usage: send [-a <flat|year|month>]")
-	}
-	var archive string
-	for _, opt := range opts {
-		if opt.Option == 'a' {
-			archive = opt.Value
+func (*Send) CompleteFolders(arg string) []string {
+	return commands.GetFolders(arg)
+}
+
+func (s *Send) ParseArchive(arg string) error {
+	for _, a := range msg.ARCHIVE_TYPES {
+		if a == arg {
+			s.Archive = arg
+			return nil
 		}
 	}
-	tab := aerc.SelectedTab()
+	return errors.New("unsupported archive type")
+}
+
+func (s Send) Execute(args []string) error {
+	tab := app.SelectedTab()
 	if tab == nil {
 		return errors.New("No selected tab")
 	}
-	composer, _ := tab.Content.(*widgets.Composer)
+	composer, _ := tab.Content.(*app.Composer)
 	tabName := tab.Name
+
 	config := composer.Config()
 
+	if s.CopyTo == "" {
+		s.CopyTo = config.CopyTo
+	}
+
 	outgoing, err := config.Outgoing.ConnectionString()
 	if err != nil {
 		return errors.Wrap(err, "ReadCredentials(outgoing)")
@@ -79,6 +94,9 @@ func (Send) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return errors.Wrap(err, "listRecipients")
 	}
+	if len(rcpts) == 0 {
+		return errors.New("Cannot send message with no recipients")
+	}
 
 	uri, err := url.Parse(outgoing)
 	if err != nil {
@@ -94,12 +112,14 @@ func (Send) Execute(aerc *widgets.Aerc, args []string) error {
 		domain = domain_
 	}
 	ctx := sendCtx{
-		uri:    uri,
-		scheme: scheme,
-		auth:   auth,
-		from:   config.From,
-		rcpts:  rcpts,
-		domain: domain,
+		uri:     uri,
+		scheme:  scheme,
+		auth:    auth,
+		from:    config.From,
+		rcpts:   rcpts,
+		domain:  domain,
+		archive: s.Archive,
+		copyto:  s.CopyTo,
 	}
 
 	log.Debugf("send config uri: %s", ctx.uri)
@@ -109,19 +129,24 @@ func (Send) Execute(aerc *widgets.Aerc, args []string) error {
 	log.Debugf("send config rcpts: %s", ctx.rcpts)
 	log.Debugf("send config domain: %s", ctx.domain)
 
-	warn, err := composer.ShouldWarnAttachment()
-	if err != nil || warn {
-		msg := "You may have forgotten an attachment."
-		if err != nil {
-			log.Warnf("failed to check for a forgotten attachment: %v", err)
-			msg = "Failed to check for a forgotten attachment."
+	warnSubject := composer.ShouldWarnSubject()
+	warnAttachment := composer.ShouldWarnAttachment()
+	if warnSubject || warnAttachment {
+		var msg string
+		switch {
+		case warnSubject && warnAttachment:
+			msg = "The subject is empty, and you may have forgotten an attachment."
+		case warnSubject:
+			msg = "The subject is empty."
+		default:
+			msg = "You may have forgotten an attachment."
 		}
 
-		prompt := widgets.NewPrompt(
+		prompt := app.NewPrompt(
 			msg+" Abort send? [Y/n] ",
 			func(text string) {
 				if text == "n" || text == "N" {
-					send(aerc, composer, ctx, header, tabName, archive)
+					send(composer, ctx, header, tabName)
 				}
 			}, func(cmd string) ([]string, string) {
 				if cmd == "" {
@@ -132,28 +157,27 @@ func (Send) Execute(aerc *widgets.Aerc, args []string) error {
 			},
 		)
 
-		aerc.PushPrompt(prompt)
+		app.PushPrompt(prompt)
 	} else {
-		send(aerc, composer, ctx, header, tabName, archive)
+		send(composer, ctx, header, tabName)
 	}
 
 	return nil
 }
 
-func send(aerc *widgets.Aerc, composer *widgets.Composer, ctx sendCtx,
-	header *mail.Header, tabName string, archive string,
+func send(composer *app.Composer, ctx sendCtx,
+	header *mail.Header, tabName string,
 ) {
 	// we don't want to block the UI thread while we are sending
 	// so we do everything in a goroutine and hide the composer from the user
-	aerc.RemoveTab(composer, false)
-	aerc.PushStatus("Sending...", 10*time.Second)
+	app.RemoveTab(composer, false)
+	app.PushStatus("Sending...", 10*time.Second)
 	log.Debugf("send uri: %s", ctx.uri.String())
 
 	// enter no-quit mode
 	mode.NoQuit()
 
 	var copyBuf bytes.Buffer // for the Sent folder content if CopyTo is set
-	config := composer.Config()
 
 	failCh := make(chan error)
 	// writer
@@ -169,6 +193,8 @@ func send(aerc *widgets.Aerc, composer *widgets.Composer, ctx sendCtx,
 			fallthrough
 		case "smtps":
 			sender, err = newSmtpSender(ctx)
+		case "jmap":
+			sender, err = newJmapSender(composer, header, ctx)
 		case "":
 			sender, err = newSendmailSender(ctx)
 		default:
@@ -181,7 +207,7 @@ func send(aerc *widgets.Aerc, composer *widgets.Composer, ctx sendCtx,
 
 		var writer io.Writer = sender
 
-		if config.CopyTo != "" {
+		if ctx.copyto != "" && ctx.scheme != "jmap" {
 			writer = io.MultiWriter(writer, &copyBuf)
 		}
 		err = composer.WriteMessage(header, writer)
@@ -201,27 +227,35 @@ func send(aerc *widgets.Aerc, composer *widgets.Composer, ctx sendCtx,
 
 		err := <-failCh
 		if err != nil {
-			aerc.PushError(strings.ReplaceAll(err.Error(), "\n", " "))
-			aerc.NewTab(composer, tabName)
+			app.PushError(strings.ReplaceAll(err.Error(), "\n", " "))
+			app.NewTab(composer, tabName)
 			return
 		}
-		if config.CopyTo != "" {
-			aerc.PushStatus("Copying to "+config.CopyTo, 10*time.Second)
-			errch := copyToSent(composer.Worker(), config.CopyTo,
-				copyBuf.Len(), &copyBuf)
+		if ctx.copyto != "" && ctx.scheme != "jmap" {
+			app.PushStatus("Copying to "+ctx.copyto, 10*time.Second)
+			errch := copyToSent(ctx.copyto, copyBuf.Len(), &copyBuf,
+				composer)
 			err = <-errch
 			if err != nil {
 				errmsg := fmt.Sprintf(
 					"message sent, but copying to %v failed: %v",
-					config.CopyTo, err.Error())
-				aerc.PushError(errmsg)
-				composer.SetSent(archive)
+					ctx.copyto, err.Error())
+				app.PushError(errmsg)
+				composer.SetSent(ctx.archive)
 				composer.Close()
 				return
 			}
 		}
-		aerc.PushStatus("Message sent.", 10*time.Second)
-		composer.SetSent(archive)
+		app.PushStatus("Message sent.", 10*time.Second)
+		composer.SetSent(ctx.archive)
+		err = hooks.RunHook(&hooks.MailSent{
+			Account: composer.Account().Name(),
+			Header:  header,
+		})
+		if err != nil {
+			log.Errorf("failed to trigger mail-sent hook: %v", err)
+			composer.Account().PushError(fmt.Errorf("[hook.mail-sent] failed: %w", err))
+		}
 		composer.Close()
 	}()
 }
@@ -239,19 +273,18 @@ func listRecipients(h *mail.Header) ([]*mail.Address, error) {
 }
 
 type sendCtx struct {
-	uri    *url.URL
-	scheme string
-	auth   string
-	from   *mail.Address
-	rcpts  []*mail.Address
-	domain string
+	uri     *url.URL
+	scheme  string
+	auth    string
+	from    *mail.Address
+	rcpts   []*mail.Address
+	domain  string
+	copyto  string
+	archive string
 }
 
 func newSendmailSender(ctx sendCtx) (io.WriteCloser, error) {
-	args, err := shlex.Split(ctx.uri.Path)
-	if err != nil {
-		return nil, err
-	}
+	args := opt.SplitArgs(ctx.uri.Path)
 	if len(args) == 0 {
 		return nil, fmt.Errorf("no command specified")
 	}
@@ -263,6 +296,7 @@ func newSendmailSender(ctx sendCtx) (io.WriteCloser, error) {
 	args = append(args[1:], rs...)
 	cmd := exec.Command(bin, args...)
 	s := &sendmailSender{cmd: cmd}
+	var err error
 	s.stdin, err = s.cmd.StdinPipe()
 	if err != nil {
 		return nil, errors.Wrap(err, "cmd.StdinPipe")
@@ -507,23 +541,62 @@ func connectSmtps(host string) (*smtp.Client, error) {
 	return conn, nil
 }
 
-func copyToSent(worker *types.Worker, dest string,
-	n int, msg io.Reader,
-) <-chan error {
-	errCh := make(chan error)
-	worker.PostAction(&types.AppendMessage{
-		Destination: dest,
-		Flags:       models.SeenFlag,
-		Date:        time.Now(),
-		Reader:      msg,
-		Length:      n,
-	}, func(msg types.WorkerMessage) {
-		switch msg := msg.(type) {
-		case *types.Done:
-			errCh <- nil
-		case *types.Error:
-			errCh <- msg.Error
-		}
-	})
+func newJmapSender(
+	composer *app.Composer, header *mail.Header, ctx sendCtx,
+) (io.WriteCloser, error) {
+	var writer io.WriteCloser
+	done := make(chan error)
+
+	composer.Worker().PostAction(
+		&types.StartSendingMessage{Header: header},
+		func(msg types.WorkerMessage) {
+			switch msg := msg.(type) {
+			case *types.Done:
+				return
+			case *types.Unsupported:
+				done <- fmt.Errorf("unsupported by worker")
+			case *types.Error:
+				done <- msg.Error
+			case *types.MessageWriter:
+				writer = msg.Writer
+			default:
+				done <- fmt.Errorf("unexpected worker message: %#v", msg)
+			}
+			close(done)
+		},
+	)
+
+	err := <-done
+
+	return writer, err
+}
+
+func copyToSent(dest string, n int, msg io.Reader, composer *app.Composer) <-chan error {
+	errCh := make(chan error, 1)
+	acct := composer.Account()
+	if acct == nil {
+		errCh <- errors.New("No account selected")
+		return errCh
+	}
+	store := acct.Store()
+	if store == nil {
+		errCh <- errors.New("No message store selected")
+		return errCh
+	}
+	store.Append(
+		dest,
+		models.SeenFlag,
+		time.Now(),
+		msg,
+		n,
+		func(msg types.WorkerMessage) {
+			switch msg := msg.(type) {
+			case *types.Done:
+				errCh <- nil
+			case *types.Error:
+				errCh <- msg.Error
+			}
+		},
+	)
 	return errCh
 }
