@@ -1,32 +1,28 @@
 package compose
 
 import (
-	"errors"
 	"time"
 
-	"git.sr.ht/~rjarry/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 )
 
 type Sign struct{}
 
 func init() {
-	register(Sign{})
+	commands.Register(Sign{})
+}
+
+func (Sign) Context() commands.CommandContext {
+	return commands.COMPOSE
 }
 
 func (Sign) Aliases() []string {
 	return []string{"sign"}
 }
 
-func (Sign) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (Sign) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) != 1 {
-		return errors.New("Usage: sign")
-	}
-
-	composer, _ := aerc.SelectedTabContent().(*widgets.Composer)
+func (Sign) Execute(args []string) error {
+	composer, _ := app.SelectedTabContent().(*app.Composer)
 
 	err := composer.SetSign(!composer.Sign())
 	if err != nil {
@@ -41,7 +37,7 @@ func (Sign) Execute(aerc *widgets.Aerc, args []string) error {
 		statusline = "Message will not be signed."
 	}
 
-	aerc.PushStatus(statusline, 10*time.Second)
+	app.PushStatus(statusline, 10*time.Second)
 
 	return nil
 }
