@@ -1,43 +1,40 @@
 package commands
 
 import (
-	"errors"
 	"fmt"
 	"os"
 	"os/exec"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/log"
-	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
-type ExecCmd struct{}
+type ExecCmd struct {
+	Args []string `opt:"..."`
+}
 
 func init() {
-	register(ExecCmd{})
+	Register(ExecCmd{})
+}
+
+func (ExecCmd) Context() CommandContext {
+	return GLOBAL
 }
 
 func (ExecCmd) Aliases() []string {
 	return []string{"exec"}
 }
 
-func (ExecCmd) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (ExecCmd) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) < 2 {
-		return errors.New("Usage: exec [cmd...]")
-	}
-
-	cmd := exec.Command(args[1], args[2:]...)
+func (e ExecCmd) Execute(args []string) error {
+	cmd := exec.Command(e.Args[0], e.Args[1:]...)
 	env := os.Environ()
 
-	switch view := aerc.SelectedTabContent().(type) {
-	case *widgets.AccountView:
+	switch view := app.SelectedTabContent().(type) {
+	case *app.AccountView:
 		env = append(env, fmt.Sprintf("account=%s", view.AccountConfig().Name))
 		env = append(env, fmt.Sprintf("folder=%s", view.Directories().Selected()))
-	case *widgets.MessageViewer:
+	case *app.MessageViewer:
 		acct := view.SelectedAccount()
 		env = append(env, fmt.Sprintf("account=%s", acct.AccountConfig().Name))
 		env = append(env, fmt.Sprintf("folder=%s", acct.Directories().Selected()))
@@ -50,14 +47,14 @@ func (ExecCmd) Execute(aerc *widgets.Aerc, args []string) error {
 
 		err := cmd.Run()
 		if err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 		} else {
 			if cmd.ProcessState.ExitCode() != 0 {
-				aerc.PushError(fmt.Sprintf(
+				app.PushError(fmt.Sprintf(
 					"%s: completed with status %d", args[0],
 					cmd.ProcessState.ExitCode()))
 			} else {
-				aerc.PushStatus(fmt.Sprintf(
+				app.PushStatus(fmt.Sprintf(
 					"%s: completed with status %d", args[0],
 					cmd.ProcessState.ExitCode()), 10*time.Second)
 			}
