@@ -1,37 +1,34 @@
 package msg
 
 import (
-	"errors"
+	"fmt"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type Delete struct{}
 
 func init() {
-	register(Delete{})
+	commands.Register(Delete{})
+}
+
+func (Delete) Context() commands.CommandContext {
+	return commands.MESSAGE
 }
 
 func (Delete) Aliases() []string {
 	return []string{"delete", "delete-message"}
 }
 
-func (Delete) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (Delete) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) != 1 {
-		return errors.New("Usage: :delete")
-	}
-
-	h := newHelper(aerc)
+func (Delete) Execute(args []string) error {
+	h := newHelper()
 	store, err := h.store()
 	if err != nil {
 		return err
@@ -52,28 +49,34 @@ func (Delete) Execute(aerc *widgets.Aerc, args []string) error {
 	store.Delete(uids, func(msg types.WorkerMessage) {
 		switch msg := msg.(type) {
 		case *types.Done:
-			aerc.PushStatus("Messages deleted.", 10*time.Second)
-			mv, isMsgView := h.msgProvider.(*widgets.MessageViewer)
+			var s string
+			if len(uids) > 1 {
+				s = "%d messages deleted"
+			} else {
+				s = "%d message deleted"
+			}
+			app.PushStatus(fmt.Sprintf(s, len(uids)), 10*time.Second)
+			mv, isMsgView := h.msgProvider.(*app.MessageViewer)
 			if isMsgView {
 				if !config.Ui.NextMessageOnDelete {
-					aerc.RemoveTab(h.msgProvider, true)
+					app.RemoveTab(h.msgProvider, true)
 				} else {
 					// no more messages in the list
 					if next == nil {
-						aerc.RemoveTab(h.msgProvider, true)
+						app.RemoveTab(h.msgProvider, true)
 						acct.Messages().Select(-1)
 						ui.Invalidate()
 						return
 					}
 					lib.NewMessageStoreView(next, mv.MessageView().SeenFlagSet(),
-						store, aerc.Crypto, aerc.DecryptKeys,
+						store, app.CryptoProvider(), app.DecryptKeys,
 						func(view lib.MessageView, err error) {
 							if err != nil {
-								aerc.PushError(err.Error())
+								app.PushError(err.Error())
 								return
 							}
-							nextMv := widgets.NewMessageViewer(acct, view)
-							aerc.ReplaceTab(mv, nextMv, next.Envelope.Subject, true)
+							nextMv := app.NewMessageViewer(acct, view)
+							app.ReplaceTab(mv, nextMv, next.Envelope.Subject, true)
 						})
 				}
 			} else {
@@ -86,12 +89,12 @@ func (Delete) Execute(aerc *widgets.Aerc, args []string) error {
 		case *types.Error:
 			marker.Remark()
 			store.Select(sel.Uid)
-			aerc.PushError(msg.Error.Error())
+			app.PushError(msg.Error.Error())
 		case *types.Unsupported:
 			marker.Remark()
 			store.Select(sel.Uid)
 			// notmuch doesn't support it, we want the user to know
-			aerc.PushError(" error, unsupported for this worker")
+			app.PushError(" error, unsupported for this worker")
 		}
 	})
 	return nil
