@@ -12,57 +12,50 @@ import (
 	"strings"
 	"sync"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/format"
-	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 	"github.com/emersion/go-message/mail"
-
-	"git.sr.ht/~sircmpwn/getopt"
 )
 
-type forward struct{}
+type forward struct {
+	AttachAll  bool     `opt:"-A"`
+	AttachFull bool     `opt:"-F"`
+	Edit       bool     `opt:"-e"`
+	NoEdit     bool     `opt:"-E"`
+	Template   string   `opt:"-T" complete:"CompleteTemplate"`
+	To         []string `opt:"..." required:"false"`
+}
 
 func init() {
-	register(forward{})
+	commands.Register(forward{})
+}
+
+func (forward) Context() commands.CommandContext {
+	return commands.MESSAGE
 }
 
 func (forward) Aliases() []string {
 	return []string{"forward"}
 }
 
-func (forward) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
+func (*forward) CompleteTemplate(arg string) []string {
+	return commands.GetTemplates(arg)
 }
 
-func (forward) Execute(aerc *widgets.Aerc, args []string) error {
-	opts, optind, err := getopt.Getopts(args, "AFT:")
-	if err != nil {
-		return err
-	}
-	attachAll := false
-	attachFull := false
-	template := ""
-	for _, opt := range opts {
-		switch opt.Option {
-		case 'A':
-			attachAll = true
-		case 'F':
-			attachFull = true
-		case 'T':
-			template = opt.Value
-		}
-	}
-
-	if attachAll && attachFull {
+func (f forward) Execute(args []string) error {
+	if f.AttachAll && f.AttachFull {
 		return errors.New("Options -A and -F are mutually exclusive")
 	}
+	editHeaders := (config.Compose.EditHeaders || f.Edit) && !f.NoEdit
 
-	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
+	widget := app.SelectedTabContent().(app.ProvidesMessage)
 	acct := widget.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
@@ -82,7 +75,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 	h.SetSubject(subject)
 
 	var tolist []*mail.Address
-	to := strings.Join(args[optind:], ", ")
+	to := strings.Join(f.To, ", ")
 	if strings.Contains(to, "@") {
 		tolist, err = mail.ParseAddressList(to)
 		if err != nil {
@@ -99,32 +92,25 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 		RFC822Headers: msg.RFC822Headers,
 	}
 
-	addTab := func() (*widgets.Composer, error) {
-		composer, err := widgets.NewComposer(aerc, acct,
-			acct.AccountConfig(), acct.Worker(), template, h, original)
+	addTab := func() (*app.Composer, error) {
+		composer, err := app.NewComposer(acct,
+			acct.AccountConfig(), acct.Worker(), editHeaders,
+			f.Template, h, &original, nil)
 		if err != nil {
-			aerc.PushError("Error: " + err.Error())
+			app.PushError("Error: " + err.Error())
 			return nil, err
 		}
 
-		tab := aerc.NewTab(composer, subject)
+		composer.Tab = app.NewTab(composer, subject)
 		if !h.Has("to") {
 			composer.FocusEditor("to")
 		} else {
 			composer.FocusTerminal()
 		}
-		composer.OnHeaderChange("Subject", func(subject string) {
-			if subject == "" {
-				tab.Name = "New email"
-			} else {
-				tab.Name = subject
-			}
-			ui.Invalidate()
-		})
 		return composer, nil
 	}
 
-	if attachFull {
+	if f.AttachFull {
 		tmpDir, err := os.MkdirTemp("", "aerc-tmp-attachment")
 		if err != nil {
 			return err
@@ -153,25 +139,43 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 				return
 			}
 			composer.AddAttachment(tmpFileName)
-			composer.OnClose(func(_ *widgets.Composer) {
+			composer.OnClose(func(_ *app.Composer) {
 				os.RemoveAll(tmpDir)
 			})
 		})
 	} else {
-		if template == "" {
-			template = config.Templates.Forwards
+		if f.Template == "" {
+			f.Template = config.Templates.Forwards
 		}
 
-		part := lib.FindPlaintext(msg.BodyStructure, nil)
+		var fetchBodyPart func([]int, func(io.Reader))
+
+		mv, isMsgViewer := widget.(*app.MessageViewer)
+		if isMsgViewer {
+			fetchBodyPart = mv.MessageView().FetchBodyPart
+		} else {
+			fetchBodyPart = func(part []int, cb func(io.Reader)) {
+				store.FetchBodyPart(msg.Uid, part, cb)
+			}
+		}
+
+		if crypto.IsEncrypted(msg.BodyStructure) && !isMsgViewer {
+			return fmt.Errorf("message is encrypted. " +
+				"can only forward from the message viewer")
+		}
+
+		part := getMessagePart(msg, widget)
 		if part == nil {
 			part = lib.FindFirstNonMultipart(msg.BodyStructure, nil)
 			// if it's still nil here, we don't have a multipart msg, that's fine
 		}
+
 		err = addMimeType(msg, part, &original)
 		if err != nil {
 			return err
 		}
-		store.FetchBodyPart(msg.Uid, part, func(reader io.Reader) {
+
+		fetchBodyPart(part, func(reader io.Reader) {
 			buf := new(bytes.Buffer)
 			scanner := bufio.NewScanner(reader)
 			for scanner.Scan() {
@@ -186,7 +190,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 			}
 
 			// add attachments
-			if attachAll {
+			if f.AttachAll {
 				var mu sync.Mutex
 				parts := lib.FindAllNonMultipart(msg.BodyStructure, nil, nil)
 				for _, p := range parts {
@@ -198,7 +202,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 						log.Errorf("cannot get PartAtIndex %v: %v", p, err)
 						continue
 					}
-					store.FetchBodyPart(msg.Uid, p, func(reader io.Reader) {
+					fetchBodyPart(p, func(reader io.Reader) {
 						mime := bs.FullMIMEType()
 						params := lib.SetUtf8Charset(bs.Params)
 						name, ok := params["name"]
@@ -210,7 +214,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 						mu.Unlock()
 						if err != nil {
 							log.Errorf(err.Error())
-							aerc.PushError(err.Error())
+							app.PushError(err.Error())
 						}
 					})
 				}
