@@ -5,33 +5,36 @@ import (
 	"fmt"
 	"io"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/calendar"
 	"git.sr.ht/~rjarry/aerc/lib/format"
-	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
 )
 
-type invite struct{}
+type invite struct {
+	Edit   bool `opt:"-e"`
+	NoEdit bool `opt:"-E"`
+}
 
 func init() {
-	register(invite{})
+	commands.Register(invite{})
+}
+
+func (invite) Context() commands.CommandContext {
+	return commands.MESSAGE
 }
 
 func (invite) Aliases() []string {
 	return []string{"accept", "accept-tentative", "decline"}
 }
 
-func (invite) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
-func (invite) Execute(aerc *widgets.Aerc, args []string) error {
-	acct := aerc.SelectedAccount()
+func (i invite) Execute(args []string) error {
+	acct := app.SelectedAccount()
 	if acct == nil {
 		return errors.New("no account selected")
 	}
@@ -49,6 +52,8 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 		return fmt.Errorf("no invitation found (missing text/calendar)")
 	}
 
+	editHeaders := (config.Compose.EditHeaders || i.Edit) && !i.NoEdit
+
 	subject := trimLocalizedRe(msg.Envelope.Subject, acct.AccountConfig().LocalizedRe)
 	switch args[0] {
 	case "accept":
@@ -61,36 +66,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 		return fmt.Errorf("no participation status defined")
 	}
 
-	conf := acct.AccountConfig()
-	from, err := mail.ParseAddress(conf.From)
-	if err != nil {
-		return err
-	}
-	var aliases []*mail.Address
-	if conf.Aliases != "" {
-		aliases, err = mail.ParseAddressList(conf.Aliases)
-		if err != nil {
-			return err
-		}
-	}
-
-	// figure out the sending from address if we have aliases
-	if len(aliases) != 0 {
-		rec := newAddrSet()
-		rec.AddList(msg.Envelope.To)
-		rec.AddList(msg.Envelope.Cc)
-		// test the from first, it has priority over any present alias
-		if rec.Contains(from) {
-			// do nothing
-		} else {
-			for _, a := range aliases {
-				if rec.Contains(a) {
-					from = a
-					break
-				}
-			}
-		}
-	}
+	from := chooseFromAddr(acct.AccountConfig(), msg)
 
 	var to []*mail.Address
 
@@ -121,7 +97,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 	h.SetMsgIDList("in-reply-to", []string{msg.Envelope.MessageId})
 	err = setReferencesHeader(h, msg.RFC822Headers)
 	if err != nil {
-		aerc.PushError(fmt.Sprintf("could not set references: %v", err))
+		app.PushError(fmt.Sprintf("could not set references: %v", err))
 	}
 	original := models.OriginalMail{
 		From:          format.FormatAddresses(msg.Envelope.From),
@@ -148,31 +124,22 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	addTab := func(cr *calendar.Reply) error {
-		composer, err := widgets.NewComposer(aerc, acct,
-			acct.AccountConfig(), acct.Worker(), "", h, original)
+		composer, err := app.NewComposer(acct,
+			acct.AccountConfig(), acct.Worker(), editHeaders,
+			"", h, &original, cr.PlainText)
 		if err != nil {
-			aerc.PushError("Error: " + err.Error())
+			app.PushError("Error: " + err.Error())
 			return err
 		}
-
-		composer.SetContents(cr.PlainText)
 		err = composer.AppendPart(cr.MimeType, cr.Params, cr.CalendarText)
 		if err != nil {
 			return fmt.Errorf("failed to write invitation: %w", err)
 		}
 		composer.FocusTerminal()
 
-		tab := aerc.NewTab(composer, subject)
-		composer.OnHeaderChange("Subject", func(subject string) {
-			if subject == "" {
-				tab.Name = "New email"
-			} else {
-				tab.Name = subject
-			}
-			ui.Invalidate()
-		})
+		composer.Tab = app.NewTab(composer, subject)
 
-		composer.OnClose(func(c *widgets.Composer) {
+		composer.OnClose(func(c *app.Composer) {
 			if c.Sent() {
 				store.Answered([]uint32{msg.Uid}, true, nil)
 			}
@@ -183,7 +150,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 
 	store.FetchBodyPart(msg.Uid, part, func(reader io.Reader) {
 		if cr, err := handleInvite(reader); err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 			return
 		} else {
 			err := addTab(cr)
