@@ -1,35 +1,35 @@
 package msg
 
 import (
-	"errors"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
-type ModifyLabels struct{}
+type ModifyLabels struct {
+	Labels []string `opt:"..." metavar:"[+-]<label>" complete:"CompleteLabels"`
+}
 
 func init() {
-	register(ModifyLabels{})
+	commands.Register(ModifyLabels{})
+}
+
+func (ModifyLabels) Context() commands.CommandContext {
+	return commands.MESSAGE
 }
 
 func (ModifyLabels) Aliases() []string {
 	return []string{"modify-labels", "tag"}
 }
 
-func (ModifyLabels) Complete(aerc *widgets.Aerc, args []string) []string {
-	return commands.GetLabels(aerc, args)
+func (*ModifyLabels) CompleteLabels(arg string) []string {
+	return commands.GetLabels(arg)
 }
 
-func (ModifyLabels) Execute(aerc *widgets.Aerc, args []string) error {
-	changes := args[1:]
-	if len(changes) == 0 {
-		return errors.New("Usage: modify-labels <[+-]label> ...")
-	}
-
-	h := newHelper(aerc)
+func (m ModifyLabels) Execute(args []string) error {
+	h := newHelper()
 	store, err := h.store()
 	if err != nil {
 		return err
@@ -40,7 +40,7 @@ func (ModifyLabels) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	var add, remove []string
-	for _, l := range changes {
+	for _, l := range m.Labels {
 		switch l[0] {
 		case '+':
 			add = append(add, l[1:])
@@ -56,10 +56,10 @@ func (ModifyLabels) Execute(aerc *widgets.Aerc, args []string) error {
 	) {
 		switch msg := msg.(type) {
 		case *types.Done:
-			aerc.PushStatus("labels updated", 10*time.Second)
+			app.PushStatus("labels updated", 10*time.Second)
 			store.Marker().ClearVisualMark()
 		case *types.Error:
-			aerc.PushError(msg.Error.Error())
+			app.PushError(msg.Error.Error())
 		}
 	})
 	return nil
