@@ -8,20 +8,27 @@ import (
 	"strings"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/app"
+	"git.sr.ht/~rjarry/aerc/commands"
+	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
-	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/log"
-	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
 )
 
 // Unsubscribe helps people unsubscribe from mailing lists by way of the
 // List-Unsubscribe header.
-type Unsubscribe struct{}
+type Unsubscribe struct {
+	Edit   bool `opt:"-e"`
+	NoEdit bool `opt:"-E"`
+}
 
 func init() {
-	register(Unsubscribe{})
+	commands.Register(Unsubscribe{})
+}
+
+func (Unsubscribe) Context() commands.CommandContext {
+	return commands.MESSAGE
 }
 
 // Aliases returns a list of aliases for the :unsubscribe command
@@ -29,17 +36,11 @@ func (Unsubscribe) Aliases() []string {
 	return []string{"unsubscribe"}
 }
 
-// Complete returns a list of completions
-func (Unsubscribe) Complete(aerc *widgets.Aerc, args []string) []string {
-	return nil
-}
-
 // Execute runs the Unsubscribe command
-func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
-	if len(args) != 1 {
-		return errors.New("Usage: unsubscribe")
-	}
-	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
+func (u Unsubscribe) Execute(args []string) error {
+	editHeaders := (config.Compose.EditHeaders || u.Edit) && !u.NoEdit
+
+	widget := app.SelectedTabContent().(app.ProvidesMessage)
 	msg, err := widget.SelectedMessage()
 	if err != nil {
 		return err
@@ -63,14 +64,14 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 		var err error
 		switch strings.ToLower(method.Scheme) {
 		case "mailto":
-			err = unsubscribeMailto(aerc, method)
+			err = unsubscribeMailto(method, editHeaders)
 		case "http", "https":
-			err = unsubscribeHTTP(aerc, method)
+			err = unsubscribeHTTP(method)
 		default:
 			err = fmt.Errorf("unsubscribe: skipping unrecognized scheme: %s", method.Scheme)
 		}
 		if err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 		}
 	}
 
@@ -84,18 +85,18 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 		options[i] = method.Scheme
 	}
 
-	dialog := widgets.NewSelectorDialog(
+	dialog := app.NewSelectorDialog(
 		title,
 		"Press <Enter> to confirm or <ESC> to cancel",
-		options, 0, aerc.SelectedAccountUiConfig(),
+		options, 0, app.SelectedAccountUiConfig(),
 		func(option string, err error) {
-			aerc.CloseDialog()
+			app.CloseDialog()
 			if err != nil {
-				if errors.Is(err, widgets.ErrNoOptionSelected) {
-					aerc.PushStatus("Unsubscribe: "+err.Error(),
+				if errors.Is(err, app.ErrNoOptionSelected) {
+					app.PushStatus("Unsubscribe: "+err.Error(),
 						5*time.Second)
 				} else {
-					aerc.PushError("Unsubscribe: " + err.Error())
+					app.PushError("Unsubscribe: " + err.Error())
 				}
 				return
 			}
@@ -105,10 +106,10 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 					return
 				}
 			}
-			aerc.PushError("Unsubscribe: selected method not found")
+			app.PushError("Unsubscribe: selected method not found")
 		},
 	)
-	aerc.AddDialog(dialog)
+	app.AddDialog(dialog)
 
 	return nil
 }
@@ -135,8 +136,8 @@ func parseUnsubscribeMethods(header string) (methods []*url.URL) {
 	}
 }
 
-func unsubscribeMailto(aerc *widgets.Aerc, u *url.URL) error {
-	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
+func unsubscribeMailto(u *url.URL, editHeaders bool) error {
+	widget := app.SelectedTabContent().(app.ProvidesMessage)
 	acct := widget.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
@@ -148,51 +149,46 @@ func unsubscribeMailto(aerc *widgets.Aerc, u *url.URL) error {
 		h.SetAddressList("to", to)
 	}
 
-	composer, err := widgets.NewComposer(
-		aerc,
+	composer, err := app.NewComposer(
+
 		acct,
 		acct.AccountConfig(),
 		acct.Worker(),
+		editHeaders,
 		"",
 		h,
-		models.OriginalMail{},
+		nil,
+		strings.NewReader(u.Query().Get("body")),
 	)
 	if err != nil {
 		return err
 	}
-	composer.SetContents(strings.NewReader(u.Query().Get("body")))
-	tab := aerc.NewTab(composer, "unsubscribe")
-	composer.OnHeaderChange("Subject", func(subject string) {
-		if subject == "" {
-			tab.Name = "unsubscribe"
-		} else {
-			tab.Name = subject
-		}
-		ui.Invalidate()
-	})
+	composer.Tab = app.NewTab(composer, "unsubscribe")
 	composer.FocusTerminal()
 	return nil
 }
 
-func unsubscribeHTTP(aerc *widgets.Aerc, u *url.URL) error {
-	confirm := widgets.NewSelectorDialog(
+func unsubscribeHTTP(u *url.URL) error {
+	confirm := app.NewSelectorDialog(
 		"Do you want to open this link?",
 		u.String(),
-		[]string{"No", "Yes"}, 0, aerc.SelectedAccountUiConfig(),
+		[]string{"No", "Yes"}, 0, app.SelectedAccountUiConfig(),
 		func(option string, _ error) {
-			aerc.CloseDialog()
+			app.CloseDialog()
 			switch option {
 			case "Yes":
 				go func() {
-					if err := lib.XDGOpen(u.String()); err != nil {
-						aerc.PushError("Unsubscribe:" + err.Error())
+					defer log.PanicHandler()
+					mime := fmt.Sprintf("x-scheme-handler/%s", u.Scheme)
+					if err := lib.XDGOpenMime(u.String(), mime, ""); err != nil {
+						app.PushError("Unsubscribe:" + err.Error())
 					}
 				}()
 			default:
-				aerc.PushError("Unsubscribe: link will not be opened")
+				app.PushError("Unsubscribe: link will not be opened")
 			}
 		},
 	)
-	aerc.AddDialog(confirm)
+	app.AddDialog(confirm)
 	return nil
 }
