@@ -2,19 +2,31 @@ package msg
 
 import (
 	"errors"
+	"time"
 
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
+	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
 type helper struct {
-	msgProvider widgets.ProvidesMessages
+	msgProvider app.ProvidesMessages
+	statusInfo  func(string)
 }
 
-func newHelper(aerc *widgets.Aerc) *helper {
-	return &helper{aerc.SelectedTab().(widgets.ProvidesMessages)}
+func newHelper() *helper {
+	msgProvider, ok := app.SelectedTabContent().(app.ProvidesMessages)
+	if !ok {
+		msgProvider = app.SelectedAccount()
+	}
+	return &helper{
+		msgProvider: msgProvider,
+		statusInfo: func(s string) {
+			app.PushStatus(s, 10*time.Second)
+		},
+	}
 }
 
 func (h *helper) markedOrSelectedUids() ([]uint32, error) {
@@ -29,7 +41,7 @@ func (h *helper) store() (*lib.MessageStore, error) {
 	return store, nil
 }
 
-func (h *helper) account() (*widgets.AccountView, error) {
+func (h *helper) account() (*app.AccountView, error) {
 	acct := h.msgProvider.SelectedAccount()
 	if acct == nil {
 		return nil, errors.New("No account selected")
@@ -46,5 +58,19 @@ func (h *helper) messages() ([]*models.MessageInfo, error) {
 	if err != nil {
 		return nil, err
 	}
-	return commands.MsgInfoFromUids(store, uid)
+	return commands.MsgInfoFromUids(store, uid, h.statusInfo)
+}
+
+func getMessagePart(msg *models.MessageInfo, provider app.ProvidesMessage) []int {
+	p := provider.SelectedMessagePart()
+	if p != nil {
+		return p.Index
+	}
+	for _, mime := range config.Viewer.Alternatives {
+		part := lib.FindMIMEPart(mime, msg.BodyStructure, nil)
+		if part != nil {
+			return part
+		}
+	}
+	return nil
 }
