@@ -9,110 +9,73 @@ import (
 	"strings"
 	"time"
 
-	"git.sr.ht/~sircmpwn/getopt"
-	"github.com/mitchellh/go-homedir"
-
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
 	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
-type Save struct{}
+type Save struct {
+	Force          bool   `opt:"-f"`
+	CreateDirs     bool   `opt:"-p"`
+	Attachments    bool   `opt:"-a"`
+	AllAttachments bool   `opt:"-A"`
+	Path           string `opt:"path" required:"false" complete:"CompletePath"`
+}
 
 func init() {
-	register(Save{})
+	commands.Register(Save{})
+}
+
+func (Save) Context() commands.CommandContext {
+	return commands.MESSAGE_VIEWER
+}
+
+func (Save) Options() string {
+	return "fpaA"
 }
 
 func (Save) Aliases() []string {
 	return []string{"save"}
 }
 
-func (Save) Complete(aerc *widgets.Aerc, args []string) []string {
-	_, optind, _ := getopt.Getopts(args, "fpa")
-	if optind < len(args) {
-		args = args[optind:]
-	}
-	path := strings.Join(args, " ")
+func (*Save) CompletePath(arg string) []string {
 	defaultPath := config.General.DefaultSavePath
-	if defaultPath != "" && !isAbsPath(path) {
-		path = filepath.Join(defaultPath, path)
+	if defaultPath != "" && !isAbsPath(arg) {
+		arg = filepath.Join(defaultPath, arg)
 	}
-	path, _ = homedir.Expand(path)
-	return commands.CompletePath(path)
+	return commands.CompletePath(arg, false)
 }
 
-type saveParams struct {
-	force         bool
-	createDirs    bool
-	trailingSlash bool
-	attachments   bool
-}
-
-func (Save) Execute(aerc *widgets.Aerc, args []string) error {
-	opts, optind, err := getopt.Getopts(args, "fpa")
-	if err != nil {
-		return err
-	}
-
-	var params saveParams
-
-	for _, opt := range opts {
-		switch opt.Option {
-		case 'f':
-			params.force = true
-		case 'p':
-			params.createDirs = true
-		case 'a':
-			params.attachments = true
-		}
-	}
-
-	defaultPath := config.General.DefaultSavePath
+func (s Save) Execute(args []string) error {
 	// we either need a path or a defaultPath
-	if defaultPath == "" && len(args) == optind {
-		return errors.New("Usage: :save [-fpa] <path>")
-	}
-
-	// as a convenience we join with spaces, so that the user doesn't need to
-	// quote filenames containing spaces
-	path := strings.Join(args[optind:], " ")
-
-	// needs to be determined prior to calling filepath.Clean / filepath.Join
-	// it gets stripped by Clean.
-	// we auto generate a name if a directory was given
-	if len(path) > 0 {
-		params.trailingSlash = path[len(path)-1] == '/'
-	} else if len(defaultPath) > 0 && len(path) == 0 {
-		// empty path, so we might have a default that ends in a trailingSlash
-		params.trailingSlash = defaultPath[len(defaultPath)-1] == '/'
+	if s.Path == "" && config.General.DefaultSavePath == "" {
+		return errors.New("No default save path in config")
 	}
 
 	// Absolute paths are taken as is so that the user can override the default
 	// if they want to
-	if !isAbsPath(path) {
-		path = filepath.Join(defaultPath, path)
+	if !isAbsPath(s.Path) {
+		s.Path = filepath.Join(config.General.DefaultSavePath, s.Path)
 	}
 
-	path, err = homedir.Expand(path)
-	if err != nil {
-		return err
-	}
+	s.Path = xdg.ExpandHome(s.Path)
 
-	mv, ok := aerc.SelectedTabContent().(*widgets.MessageViewer)
+	mv, ok := app.SelectedTabContent().(*app.MessageViewer)
 	if !ok {
 		return fmt.Errorf("SelectedTabContent is not a MessageViewer")
 	}
 
-	if params.attachments {
-		parts := mv.AttachmentParts()
+	if s.Attachments || s.AllAttachments {
+		parts := mv.AttachmentParts(s.AllAttachments)
 		if len(parts) == 0 {
 			return fmt.Errorf("This message has no attachments")
 		}
-		params.trailingSlash = true
+		names := make(map[string]struct{})
 		for _, pi := range parts {
-			if err := savePart(pi, path, mv, aerc, &params); err != nil {
+			if err := s.savePart(pi, mv, names); err != nil {
 				return err
 			}
 		}
@@ -120,30 +83,32 @@ func (Save) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	pi := mv.SelectedMessagePart()
-	return savePart(pi, path, mv, aerc, &params)
+	return s.savePart(pi, mv, make(map[string]struct{}))
 }
 
-func savePart(
-	pi *widgets.PartInfo,
-	path string,
-	mv *widgets.MessageViewer,
-	aerc *widgets.Aerc,
-	params *saveParams,
+func (s *Save) savePart(
+	pi *app.PartInfo,
+	mv *app.MessageViewer,
+	names map[string]struct{},
 ) error {
-	if params.trailingSlash || isDirExists(path) {
+	path := s.Path
+	if s.Attachments || s.AllAttachments || isDirExists(path) {
 		filename := generateFilename(pi.Part)
 		path = filepath.Join(path, filename)
 	}
 
 	dir := filepath.Dir(path)
-	if params.createDirs && dir != "" {
+	if s.CreateDirs && dir != "" {
 		err := os.MkdirAll(dir, 0o755)
 		if err != nil {
 			return err
 		}
 	}
 
-	if pathExists(path) && !params.force {
+	path = getCollisionlessFilename(path, names)
+	names[path] = struct{}{}
+
+	if pathExists(path) && !s.Force {
 		return fmt.Errorf("%q already exists and -f not given", path)
 	}
 
@@ -169,14 +134,27 @@ func savePart(
 
 		err := <-ch
 		if err != nil {
-			aerc.PushError(fmt.Sprintf("Save failed: %v", err))
+			app.PushError(fmt.Sprintf("Save failed: %v", err))
 			return
 		}
-		aerc.PushStatus("Saved to "+path, 10*time.Second)
+		app.PushStatus("Saved to "+path, 10*time.Second)
 	}()
 	return nil
 }
 
+func getCollisionlessFilename(path string, existing map[string]struct{}) string {
+	ext := filepath.Ext(path)
+	name := strings.TrimSuffix(path, ext)
+	_, exists := existing[path]
+	counter := 1
+	for exists {
+		path = fmt.Sprintf("%s_%d%s", name, counter, ext)
+		counter++
+		_, exists = existing[path]
+	}
+	return path
+}
+
 // isDir returns true if path is a directory and exists
 func isDirExists(path string) bool {
 	pathinfo, err := os.Stat(path)
