@@ -12,38 +12,39 @@ import (
 
 	"github.com/lithammer/fuzzysearch/fuzzy"
 
+	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
-	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"git.sr.ht/~rjarry/go-opt"
 	"github.com/gdamore/tcell/v2"
-	"github.com/mitchellh/go-homedir"
 )
 
 // QuickTerm is an ephemeral terminal for running a single command and quitting.
-func QuickTerm(aerc *widgets.Aerc, args []string, stdin io.Reader) (*widgets.Terminal, error) {
+func QuickTerm(args []string, stdin io.Reader) (*app.Terminal, error) {
 	cmd := exec.Command(args[0], args[1:]...)
 	pipe, err := cmd.StdinPipe()
 	if err != nil {
 		return nil, err
 	}
 
-	term, err := widgets.NewTerminal(cmd)
+	term, err := app.NewTerminal(cmd)
 	if err != nil {
 		return nil, err
 	}
 
 	term.OnClose = func(err error) {
 		if err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 			// remove the tab on error, otherwise it gets stuck
-			aerc.RemoveTab(term)
+			app.RemoveTab(term, false)
 		} else {
-			aerc.PushStatus("Process complete, press any key to close.",
+			app.PushStatus("Process complete, press any key to close.",
 				10*time.Second)
 			term.OnEvent = func(event tcell.Event) bool {
-				aerc.RemoveTab(term)
+				app.RemoveTab(term, true)
 				return true
 			}
 		}
@@ -62,7 +63,7 @@ func QuickTerm(aerc *widgets.Aerc, args []string, stdin io.Reader) (*widgets.Ter
 
 		err := <-status
 		if err != nil {
-			aerc.PushError(err.Error())
+			app.PushError(err.Error())
 		}
 	}
 
@@ -70,69 +71,40 @@ func QuickTerm(aerc *widgets.Aerc, args []string, stdin io.Reader) (*widgets.Ter
 }
 
 // CompletePath provides filesystem completions given a starting path.
-func CompletePath(path string) []string {
-	if path == "" {
-		// default to cwd
-		cwd, err := os.Getwd()
-		if err != nil {
-			return nil
-		}
-		path = cwd
+func CompletePath(path string, onlyDirs bool) []string {
+	if path == ".." || strings.HasSuffix(path, "/..") {
+		return []string{path + "/"}
+	}
+	if path == "~" || strings.HasPrefix(path, "~/") {
+		path = xdg.HomeDir() + strings.TrimPrefix(path, "~")
+	}
+	includeHidden := path == "."
+	if i := strings.LastIndex(path, "/"); i != -1 && i < len(path)-1 {
+		includeHidden = strings.HasPrefix(path[i+1:], ".")
 	}
 
-	path, err := homedir.Expand(path)
-	if err != nil {
+	matches, err := filepath.Glob(path + "*")
+	if err != nil || matches == nil {
 		return nil
 	}
 
-	// strip trailing slashes, etc.
-	path = filepath.Clean(path)
+	results := make([]string, 0, len(matches))
 
-	if _, err := os.Stat(path); os.IsNotExist(err) {
-		// if the path doesn't exist, it is likely due to it being a partial path
-		// in this case, we want to return possible matches (ie /hom* should match
-		// /home)
-		matches, err := filepath.Glob(fmt.Sprintf("%s*", path))
-		if err != nil {
-			return nil
+	for _, m := range matches {
+		if isDir(m) {
+			m += "/"
+		} else if onlyDirs {
+			continue
 		}
-
-		if !strings.HasPrefix(path, ".") && !strings.Contains(path, "/.") {
-			log.Debugf("removing hidden files from glob results")
-			for i := len(matches) - 1; i >= 0; i-- {
-				if strings.HasPrefix(filepath.Base(matches[i]), ".") {
-					if i == len(matches)-1 {
-						matches = matches[:i]
-						continue
-					}
-					matches = append(matches[:i], matches[i+1:]...)
-				}
-			}
+		if strings.HasPrefix(filepath.Base(m), ".") && !includeHidden {
+			continue
 		}
-
-		for i, m := range matches {
-			if isDir(m) {
-				matches[i] = m + "/"
-			}
-		}
-
-		sort.Strings(matches)
-		return matches
+		results = append(results, opt.QuoteArg(xdg.TildeHome(m)))
 	}
 
-	files := listDir(path, false)
+	sort.Strings(results)
 
-	for i, f := range files {
-		f = filepath.Join(path, f)
-		if isDir(f) {
-			f += "/"
-		}
-
-		files[i] = f
-	}
-
-	sort.Strings(files)
-	return files
+	return results
 }
 
 func isDir(path string) bool {
@@ -172,20 +144,49 @@ func listDir(path string, hidden bool) []string {
 
 // MarkedOrSelected returns either all marked messages if any are marked or the
 // selected message instead
-func MarkedOrSelected(pm widgets.ProvidesMessages) ([]uint32, error) {
+func MarkedOrSelected(pm app.ProvidesMessages) ([]uint32, error) {
 	// marked has priority over the selected message
 	marked, err := pm.MarkedMessages()
 	if err != nil {
 		return nil, err
 	}
 	if len(marked) > 0 {
+		marked = expandFoldedThreads(pm, marked)
 		return marked, nil
 	}
 	msg, err := pm.SelectedMessage()
 	if err != nil {
 		return nil, err
 	}
-	return []uint32{msg.Uid}, nil
+	return expandFoldedThreads(pm, []uint32{msg.Uid}), nil
+}
+
+func expandFoldedThreads(pm app.ProvidesMessages, uids []uint32) []uint32 {
+	store := pm.Store()
+	if store == nil {
+		return uids
+	}
+	expanded := make([]uint32, len(uids))
+	copy(expanded, uids)
+	for _, uid := range uids {
+		thread, err := store.Thread(uid)
+		if err != nil {
+			continue
+		}
+		if thread != nil && thread.FirstChild != nil && thread.FirstChild.Hidden > 0 {
+			_ = thread.Walk(func(t *types.Thread, _ int, __ error) error {
+				if t.Uid != uid {
+					expanded = append(expanded, t.Uid)
+				}
+				return nil
+			})
+		}
+
+	}
+	if len(uids) != len(expanded) {
+		log.Debugf("expand folded threads: %v -> %v\n", uids, expanded)
+	}
+	return expanded
 }
 
 // UidsFromMessageInfos extracts a uid slice from a slice of MessageInfos
@@ -230,18 +231,29 @@ func MsgInfoFromUids(store *lib.MessageStore, uids []uint32, statusInfo func(str
 	return infos, nil
 }
 
+func QuoteSpace(s string) string {
+	return opt.QuoteArg(s) + " "
+}
+
 // FilterList takes a list of valid completions and filters it, either
-// by case smart prefix, or by fuzzy matching, prepending "prefix" to each completion
-func FilterList(valid []string, search, prefix string, isFuzzy bool) []string {
-	out := make([]string, 0)
-	if isFuzzy {
+// by case smart prefix, or by fuzzy matching
+// An optional post processing function can be passed to prepend, append or
+// quote each value.
+func FilterList(
+	valid []string, search string, postProc func(string) string,
+) []string {
+	if postProc == nil {
+		postProc = opt.QuoteArg
+	}
+	out := make([]string, 0, len(valid))
+	if app.SelectedAccountUiConfig().FuzzyComplete {
 		for _, v := range fuzzy.RankFindFold(search, valid) {
-			out = append(out, prefix+v.Target)
+			out = append(out, postProc(v.Target))
 		}
 	} else {
 		for _, v := range valid {
 			if hasCaseSmartPrefix(v, search) {
-				out = append(out, prefix+v)
+				out = append(out, postProc(v))
 			}
 		}
 	}
