@@ -57,6 +57,8 @@ type KeyBindings struct {
 	Globals bool
 	// Which key opens the ex line (default is :)
 	ExKey KeyStroke
+	// Which key triggers completion (default is <tab>)
+	CompleteKey KeyStroke
 
 	// private
 	contextualBinds  []*BindingConfigContext
@@ -81,6 +83,7 @@ func defaultBindsConfig() *BindingConfig {
 	// These bindings are not configurable
 	wizard := NewKeyBindings()
 	wizard.ExKey = KeyStroke{Key: tcell.KeyCtrlE}
+	wizard.Globals = false
 	quit, _ := ParseBinding("<C-q>", ":quit<Enter>")
 	wizard.Add(quit)
 	return &BindingConfig{
@@ -98,14 +101,7 @@ func defaultBindsConfig() *BindingConfig {
 
 var Binds = defaultBindsConfig()
 
-func parseBinds(root string) error {
-	filename := path.Join(root, "binds.conf")
-	if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
-		fmt.Printf("%s not found, installing the system default", filename)
-		if err := installTemplate(root, "binds.conf"); err != nil {
-			return err
-		}
-	}
+func parseBindsFromFile(root string, filename string) error {
 	log.Debugf("Parsing key bindings configuration from %s", filename)
 	binds, err := ini.LoadSources(ini.LoadOptions{
 		KeyValueDelimiters: "=",
@@ -150,10 +146,29 @@ func parseBinds(root string) error {
 	return nil
 }
 
+func parseBinds(root string, filename string) error {
+	if filename == "" {
+		filename = path.Join(root, "binds.conf")
+		if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
+			fmt.Printf("%s not found, installing the system default\n", filename)
+			if err := installTemplate(root, "binds.conf"); err != nil {
+				return err
+			}
+		}
+	}
+
+	if err := parseBindsFromFile(root, filename); err != nil {
+		return fmt.Errorf("%s: %w", filename, err)
+	}
+
+	return nil
+}
+
 func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 	bindings := NewKeyBindings()
 	for key, value := range sec.KeysHash() {
-		if key == "$ex" {
+		switch key {
+		case "$ex":
 			strokes, err := ParseKeyStrokes(value)
 			if err != nil {
 				return nil, err
@@ -162,9 +177,7 @@ func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 				return nil, errors.New("Invalid binding")
 			}
 			bindings.ExKey = strokes[0]
-			continue
-		}
-		if key == "$noinherit" {
+		case "$noinherit":
 			if value == "false" {
 				continue
 			}
@@ -172,13 +185,22 @@ func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 				return nil, errors.New("Invalid binding")
 			}
 			bindings.Globals = false
-			continue
+		case "$complete":
+			strokes, err := ParseKeyStrokes(value)
+			if err != nil {
+				return nil, err
+			}
+			if len(strokes) != 1 {
+				return nil, errors.New("Invalid binding")
+			}
+			bindings.CompleteKey = strokes[0]
+		default:
+			binding, err := ParseBinding(key, value)
+			if err != nil {
+				return nil, err
+			}
+			bindings.Add(binding)
 		}
-		binding, err := ParseBinding(key, value)
-		if err != nil {
-			return nil, err
-		}
-		bindings.Add(binding)
 	}
 	return bindings, nil
 }
@@ -199,8 +221,8 @@ func LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings) error
 	}
 
 	for _, sectionName := range binds.SectionStrings() {
-		if !strings.Contains(sectionName, baseName+":") ||
-			strings.Contains(sectionName, baseName+"::") {
+		if !strings.HasPrefix(sectionName, baseName+":") ||
+			strings.HasPrefix(sectionName, baseName+"::") {
 			continue
 		}
 
@@ -265,18 +287,71 @@ func LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings) error
 func NewKeyBindings() *KeyBindings {
 	return &KeyBindings{
 		ExKey:            KeyStroke{tcell.ModNone, tcell.KeyRune, ':'},
+		CompleteKey:      KeyStroke{tcell.ModNone, tcell.KeyTab, 0},
 		Globals:          true,
 		contextualCache:  make(map[bindsContextKey]*KeyBindings),
 		contextualCounts: make(map[bindsContextType]int),
 	}
 }
 
+func areBindingsInputsEqual(a, b *Binding) bool {
+	if len(a.Input) != len(b.Input) {
+		return false
+	}
+
+	for idx := range a.Input {
+		if a.Input[idx] != b.Input[idx] {
+			return false
+		}
+	}
+
+	return true
+}
+
+// this scans the bindings slice for copies and leaves just the first ones
+// it also removes empty bindings, the ones that do nothing, so you can
+// override and erase parent bindings with the context ones
+func filterAndCleanBindings(bindings []*Binding) []*Binding {
+	// 1. remove a binding if we already have one with the same input
+	res1 := []*Binding{}
+	for _, b := range bindings {
+		// do we already have one here?
+		found := false
+		for _, r := range res1 {
+			if areBindingsInputsEqual(b, r) {
+				found = true
+				break
+			}
+		}
+
+		// add it if we don't
+		if !found {
+			res1 = append(res1, b)
+		}
+	}
+
+	// 2. clean up the empty bindings
+	res2 := []*Binding{}
+	for _, b := range res1 {
+		if len(b.Output) > 0 {
+			res2 = append(res2, b)
+		}
+	}
+
+	return res2
+}
+
 func MergeBindings(bindings ...*KeyBindings) *KeyBindings {
 	merged := NewKeyBindings()
 	for _, b := range bindings {
 		merged.Bindings = append(merged.Bindings, b.Bindings...)
+		if !b.Globals {
+			break
+		}
 	}
+	merged.Bindings = filterAndCleanBindings(merged.Bindings)
 	merged.ExKey = bindings[0].ExKey
+	merged.CompleteKey = bindings[0].CompleteKey
 	merged.Globals = bindings[0].Globals
 	return merged
 }
@@ -392,11 +467,16 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 			if ks.Modifiers == stroke.Modifiers && ks.Key == stroke.Key && ks.Rune == stroke.Rune {
 				switch name {
 				case "cr", "c-m":
-					name = "enter"
+					s = "<enter>"
 				case "c-i":
-					name = "tab"
+					s = "<tab>"
+				case "space":
+					s = " "
+				case "semicolon":
+					s = ";"
+				default:
+					s = fmt.Sprintf("<%s>", name)
 				}
-				s = fmt.Sprintf("<%s>", name)
 				break
 			}
 		}
@@ -406,9 +486,20 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 		sb.WriteString(s)
 	}
 
-	return sb.String()
+	// replace leading & trailing spaces with explicit <space> keystrokes
+	buf := sb.String()
+	match := spaceTrimRe.FindStringSubmatch(buf)
+	if len(match) == 4 {
+		prefix := strings.ReplaceAll(match[1], " ", "<space>")
+		suffix := strings.ReplaceAll(match[3], " ", "<space>")
+		buf = prefix + match[2] + suffix
+	}
+
+	return buf
 }
 
+var spaceTrimRe = regexp.MustCompile(`^(\s*)(.*?)(\s*)$`)
+
 var keyNames = map[string]KeyStroke{
 	"space":     {tcell.ModNone, tcell.KeyRune, ' '},
 	"semicolon": {tcell.ModNone, tcell.KeyRune, ';'},
@@ -442,6 +533,9 @@ var keyNames = map[string]KeyStroke{
 	"end":       {tcell.ModNone, tcell.KeyEnd, 0},
 	"insert":    {tcell.ModNone, tcell.KeyInsert, 0},
 	"delete":    {tcell.ModNone, tcell.KeyDelete, 0},
+	"c-delete":  {tcell.ModCtrl, tcell.KeyDelete, 0},
+	"a-delete":  {tcell.ModAlt, tcell.KeyDelete, 0},
+	"backspace": {tcell.ModNone, tcell.KeyBackspace2, 0},
 	"help":      {tcell.ModNone, tcell.KeyHelp, 0},
 	"exit":      {tcell.ModNone, tcell.KeyExit, 0},
 	"clear":     {tcell.ModNone, tcell.KeyClear, 0},
@@ -546,6 +640,16 @@ var keyNames = map[string]KeyStroke{
 	"c-^":       {tcell.ModCtrl, tcell.KeyCtrlCarat, 0},
 	"c-_":       {tcell.ModCtrl, tcell.KeyCtrlUnderscore, 0},
 	"a-space":   {tcell.ModAlt, tcell.KeyRune, ' '},
+	"a-0":       {tcell.ModAlt, tcell.KeyRune, '0'},
+	"a-1":       {tcell.ModAlt, tcell.KeyRune, '1'},
+	"a-2":       {tcell.ModAlt, tcell.KeyRune, '2'},
+	"a-3":       {tcell.ModAlt, tcell.KeyRune, '3'},
+	"a-4":       {tcell.ModAlt, tcell.KeyRune, '4'},
+	"a-5":       {tcell.ModAlt, tcell.KeyRune, '5'},
+	"a-6":       {tcell.ModAlt, tcell.KeyRune, '6'},
+	"a-7":       {tcell.ModAlt, tcell.KeyRune, '7'},
+	"a-8":       {tcell.ModAlt, tcell.KeyRune, '8'},
+	"a-9":       {tcell.ModAlt, tcell.KeyRune, '9'},
 	"a-a":       {tcell.ModAlt, tcell.KeyRune, 'a'},
 	"a-b":       {tcell.ModAlt, tcell.KeyRune, 'b'},
 	"a-c":       {tcell.ModAlt, tcell.KeyRune, 'c'},
