@@ -5,10 +5,14 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/log"
 	"github.com/go-ini/ini"
-	"github.com/google/shlex"
 )
 
-var Openers = make(map[string][]string)
+type Opener struct {
+	Mime string
+	Args string
+}
+
+var Openers []Opener
 
 func parseOpeners(file *ini.File) error {
 	openers, err := file.GetSection("openers")
@@ -16,13 +20,9 @@ func parseOpeners(file *ini.File) error {
 		goto out
 	}
 
-	for mimeType, command := range openers.KeysHash() {
-		mimeType = strings.ToLower(mimeType)
-		if args, err := shlex.Split(command); err != nil {
-			return err
-		} else {
-			Openers[mimeType] = args
-		}
+	for _, key := range openers.Keys() {
+		mime := strings.ToLower(key.Name())
+		Openers = append(Openers, Opener{Mime: mime, Args: key.Value()})
 	}
 
 out:
