@@ -65,6 +65,7 @@ var (
 )
 
 func (d *dummyData) Account() string                 { return "work" }
+func (d *dummyData) Signature() string               { return "" }
 func (d *dummyData) Folder() string                  { return "INBOX" }
 func (d *dummyData) To() []*mail.Address             { return []*mail.Address{&addr1} }
 func (d *dummyData) Cc() []*mail.Address             { return nil }
@@ -76,11 +77,25 @@ func (d *dummyData) Date() time.Time                 { return time.Now() }
 func (d *dummyData) DateAutoFormat(time.Time) string { return "" }
 func (d *dummyData) Header(string) string            { return "" }
 func (d *dummyData) ThreadPrefix() string            { return "└─>" }
+func (d *dummyData) ThreadCount() int                { return 0 }
+func (d *dummyData) ThreadUnread() int               { return 0 }
+func (d *dummyData) ThreadFolded() bool              { return false }
+func (d *dummyData) ThreadContext() bool             { return true }
 func (d *dummyData) Subject() string                 { return "Re: [PATCH] hey" }
 func (d *dummyData) SubjectBase() string             { return "[PATCH] hey" }
+func (d *dummyData) Attach(string) string            { return "" }
 func (d *dummyData) Number() int                     { return 0 }
 func (d *dummyData) Labels() []string                { return nil }
+func (d *dummyData) Filename() string                { return "" }
+func (d *dummyData) Filenames() []string             { return nil }
 func (d *dummyData) Flags() []string                 { return nil }
+func (d *dummyData) IsReplied() bool                 { return true }
+func (d *dummyData) HasAttachment() bool             { return true }
+func (d *dummyData) IsRecent() bool                  { return false }
+func (d *dummyData) IsUnread() bool                  { return false }
+func (d *dummyData) IsFlagged() bool                 { return false }
+func (d *dummyData) IsDraft() bool                   { return false }
+func (d *dummyData) IsMarked() bool                  { return false }
 func (d *dummyData) MessageId() string               { return "123456789@foo.org" }
 func (d *dummyData) Size() int                       { return 420 }
 func (d *dummyData) OriginalText() string            { return "Blah blah blah" }
@@ -102,3 +117,5 @@ func (d *dummyData) Role() string                    { return "inbox" }
 
 func (d *dummyData) Style(string, string) string               { return "" }
 func (d *dummyData) StyleSwitch(string, ...models.Case) string { return "" }
+
+func (d *dummyData) StyleMap([]string, ...models.Case) []string { return []string{} }
