@@ -2,7 +2,6 @@
 /* Copyright (c) 2023 Robin Jarry */
 
 #include <ctype.h>
-#include <errno.h>
 #include <fnmatch.h>
 #include <getopt.h>
 #include <regex.h>
@@ -14,10 +13,13 @@
 
 static void usage(void)
 {
-	puts("usage: colorize [-h] [-s FILE] [-f FILE]");
+	puts("usage: colorize [-h] [-8] [-s FILE] [-f FILE]");
+	puts("");
+	puts("Add terminal escape codes to colorize plain text email bodies.");
 	puts("");
 	puts("options:");
 	puts("  -h       show this help message");
+	puts("  -8       emit OSC 8 hyperlink sequences (default $AERC_OSC8_URLS)");
 	puts("  -s FILE  use styleset file (default $AERC_STYLESET)");
 	puts("  -f FILE  read from filename (default stdin)");
 }
@@ -62,7 +64,7 @@ static void *xmalloc(size_t s)
 #define RESET "\x1b[0m"
 #define LONGEST_SEQ "\x1b[1;2;3;4;5;7;38;2;255;255;255;48;2;255;255;255m"
 
-const char *seq(struct style *s) {
+static const char *seq(struct style *s) {
 	if (!s->sequence) {
 		char *b, *buf = xmalloc(strlen(LONGEST_SEQ) + 1);
 		const char *sep = "";
@@ -144,6 +146,7 @@ struct styles {
 	struct style signature;
 	struct style diff_meta;
 	struct style diff_chunk;
+	struct style diff_chunk_func;
 	struct style diff_add;
 	struct style diff_del;
 	struct style quote_1;
@@ -154,13 +157,15 @@ struct styles {
 };
 
 static FILE *in_file;
+static bool osc8_urls;
 static const char *styleset;
 static struct styles styles = {
 	.url = { .underline = true, .fg = { .type = RGB, .rgb = 0xffffaf } },
 	.header = { .bold = true, .fg = { .type = RGB, .rgb = 0xaf87ff } },
 	.signature = { .dim = true, .fg = { .type = RGB, .rgb = 0xaf87ff } },
 	.diff_meta = { .bold = true, .fg = { .type = RGB, .rgb = 0xffffff } },
-	.diff_chunk = { .fg = { .type = RGB, .rgb = 0x00cdcd } },
+	.diff_chunk = { .bold = true, .fg = { .type = RGB, .rgb = 0x00cdcd } },
+	.diff_chunk_func = { .fg = { .type = RGB, .rgb = 0x00cdcd } },
 	.diff_add = { .fg = { .type = RGB, .rgb = 0x00cd00 } },
 	.diff_del = { .fg = { .type = RGB, .rgb = 0xcd0000 } },
 	.quote_1 = { .fg = { .type = RGB, .rgb = 0x5fafff } },
@@ -246,14 +251,14 @@ static int parse_color(struct color *c, const char *val)
 	} else if (sscanf(val, "#%x", &color) == 1 && color <= 0xffffff) {
 		c->type = RGB;
 		c->rgb = color;
-	} else if (sscanf(val, "%d", &color) == 1 && color <= 256) {
+	} else if (sscanf(val, "%u", &color) == 1 && color <= 256) {
 		c->type = PALETTE;
 		c->index = color;
 	} else if (!color_name(val, &color)) {
 		c->type = RGB;
 		c->rgb = color;
 	} else {
-		fprintf(stderr, "error: invalid color value %s\n", val);
+		fprintf(stderr, "error: invalid color value '%s'\n", val);
 		return 1;
 	}
 	return 0;
@@ -268,7 +273,7 @@ static int parse_bool(bool *b, const char *val)
 	} else if (!strcmp(val, "toggle")) {
 		*b = !*b;
 	} else {
-		fprintf(stderr, "error: invalid bool value %s\n", val);
+		fprintf(stderr, "error: invalid bool value '%s'\n", val);
 		return 1;
 	}
 	return 0;
@@ -310,7 +315,7 @@ static int set_attr(struct style *s, const char *attr, const char *val)
 		s->fg.type = NONE;
 		s->fg.type = NONE;
 	} else {
-		fprintf(stderr, "error: invalid style attribute %s\n", attr);
+		fprintf(stderr, "error: invalid style attribute '%s'\n", attr);
 		return 1;
 	}
 	return 0;
@@ -322,6 +327,7 @@ static struct {const char *n; struct style *s;} ini_objects[] = {
 	{"signature", &styles.signature},
 	{"diff_meta", &styles.diff_meta},
 	{"diff_chunk", &styles.diff_chunk},
+	{"diff_chunk_func", &styles.diff_chunk_func},
 	{"diff_add", &styles.diff_add},
 	{"diff_del", &styles.diff_del},
 	{"quote_1", &styles.quote_1},
@@ -331,6 +337,9 @@ static struct {const char *n; struct style *s;} ini_objects[] = {
 	{"quote_x", &styles.quote_x},
 };
 
+/*                         object            attribute           value */
+#define STYLE_LINE_FORMAT "%127[0-9A-Za-z_-*?].%127[0-9a-zA-Z_-] = %127[#a-zA-Z0-9]s"
+
 static int parse_styleset(void)
 {
 	bool in_section = false;
@@ -351,10 +360,10 @@ static int parse_styleset(void)
 		/* strip LF, CR, CRLF, LFCR */
 		buf[strcspn(buf, "\r\n")] = '\0';
 		if (in_section) {
-			char obj[64], attr[64], val[64];
+			char obj[128], attr[128], val[128];
 			bool changed = false;
 
-			if (sscanf(buf, "%63[^.].%63[^=] = %63s", obj, attr, val) != 3) {
+			if (sscanf(buf, STYLE_LINE_FORMAT, obj, attr, val) != 3) {
 				if (buf[0] == '[') {
 					/* start of another section */
 					break;
@@ -441,7 +450,9 @@ static void diff_chunk(const char *in)
 		len++;
 	in += print_notabs(in, len);
 	print(RESET);
+	print(seq(&styles.diff_chunk_func));
 	print_notabs(in, BUFSIZ);
+	print(RESET);
 }
 
 static inline bool isurichar(char c)
@@ -537,9 +548,13 @@ static void urls(const char *in, struct style *ctx)
 		}
 		print(seq(&styles.url));
 		bool email = groups[2].rm_so == -1 && groups[1].rm_so == -1;
-		print_osc8(in, len, url_id, email);
+		if (osc8_urls) {
+			print_osc8(in, len, url_id, email);
+		}
 		in += print_notabs(in, len);
-		print_osc8(NULL, 0, url_id, email);
+		if (osc8_urls) {
+			print_osc8(NULL, 0, url_id, email);
+		}
 		url_id++;
 		print(RESET);
 		if (ctx) {
@@ -571,8 +586,11 @@ static void header(const char *in)
 	urls(in, NULL);
 }
 
+#define DIFF_START_RE "^(diff (--git|-up|-u)|---) [[:graph:]]"
+static regex_t diff_start_re;
+
 #define DIFF_META_RE \
-	"^(diff --git|(new|deleted) file|similarity" \
+	"^(diff (--git|-up|-u)|(new|deleted) file|similarity" \
 	" index|(rename|copy) (to|from)|index|---|\\+\\+\\+) "
 static regex_t diff_meta_re;
 
@@ -640,7 +658,7 @@ static void colorize_line(const char *in)
 
 	switch (state) {
 	case DIFF:
-		if (!strcmp(in, "--") || !strcmp(in, "-- ")) {
+		if (!strcmp(in, "-- ")) {
 			state = SIGNATURE;
 			signature(in);
 		} else if (startswith(in, "@@ ")) {
@@ -665,26 +683,11 @@ static void colorize_line(const char *in)
 	case SIGNATURE:
 		signature(in);
 		break;
-	case BODY:
-		if (startswith(in, ">")) {
-			quote(in);
-		} else if (startswith(in, "diff --git ")) {
+	default: /* BODY, INIT */
+		if (!regexec(&diff_start_re, in, 8, groups, 0)) {
 			state = DIFF;
 			print_style(in, &styles.diff_meta);
-		} else if (!strcmp(in, "--") || !strcmp(in, "-- ")) {
-			state = SIGNATURE;
-			signature(in);
-		} else if (!regexec(&header_re, in, 8, groups, 0)) {
-			header(in);
-		} else {
-			urls(in, NULL);
-		}
-		break;
-	default: /* INIT */
-		if (startswith(in, "diff --git ")) {
-			state = DIFF;
-			print_style(in, &styles.diff_meta);
-		} else if (!strcmp(in, "--") || !strcmp(in, "-- ")) {
+		} else if (!strcmp(in, "-- ")) {
 			state = SIGNATURE;
 			signature(in);
 		} else {
@@ -701,15 +704,19 @@ static void colorize_line(const char *in)
 	}
 }
 
-int parse_args(int argc, char **argv)
+static int parse_args(int argc, char **argv)
 {
-	const char *filename = NULL;
+	const char *filename = NULL, *osc8 = NULL;
 	int c;
 
 	styleset = getenv("AERC_STYLESET");
+	osc8 = getenv("AERC_OSC8_URLS");
 
-	while ((c = getopt(argc, argv, "hs:f:")) != -1) {
+	while ((c = getopt(argc, argv, "h8s:f:")) != -1) {
 		switch (c) {
+		case '8':
+			osc8 = "1";
+			break;
 		case 's':
 			styleset = optarg;
 			break;
@@ -736,6 +743,8 @@ int parse_args(int argc, char **argv)
 			return 1;
 		}
 	}
+	osc8_urls = osc8 != NULL;
+
 	return 0;
 }
 
@@ -745,6 +754,7 @@ int main(int argc, char **argv)
 	int err;
 
 	regcomp(&header_re, HEADER_RE, REG_EXTENDED);
+	regcomp(&diff_start_re, DIFF_START_RE, REG_EXTENDED);
 	regcomp(&diff_meta_re, DIFF_META_RE, REG_EXTENDED);
 	regcomp(&url_re, URL_RE, REG_EXTENDED);
 
