@@ -91,23 +91,6 @@ func TruncateHead(s string, w int, head string) string {
 	return head + s[pos:]
 }
 
-func ShellQuote(args []string) string {
-	quoted := make([]string, len(args))
-
-	for i, arg := range args {
-		if strings.ContainsAny(arg, " '\"|&!#$;[](){}<>*\n\t") {
-			if strings.ContainsAny(arg, "!\"$") {
-				arg = "'" + arg + "'"
-			} else {
-				arg = "\"" + arg + "\""
-			}
-		}
-		quoted[i] = arg
-	}
-
-	return strings.Join(quoted, " ")
-}
-
 func DummyIfZeroDate(date time.Time, format string, todayFormat string,
 	thisWeekFormat string, thisYearFormat string,
 ) string {
