@@ -14,7 +14,6 @@ import (
 )
 
 // TODO: Attach history providers
-// TODO: scrolling
 
 type TextInput struct {
 	sync.Mutex
@@ -35,6 +34,7 @@ type TextInput struct {
 	completeDelay     time.Duration
 	completeDebouncer *time.Timer
 	completeMinChars  int
+	completeKey       *config.KeyStroke
 	uiConfig          *config.UIConfig
 }
 
@@ -62,12 +62,12 @@ func (ti *TextInput) Prompt(prompt string) *TextInput {
 
 func (ti *TextInput) TabComplete(
 	tabcomplete func(s string) ([]string, string),
-	d time.Duration,
-	minChars int,
+	d time.Duration, minChars int, key *config.KeyStroke,
 ) *TextInput {
 	ti.tabcomplete = tabcomplete
 	ti.completeDelay = d
 	ti.completeMinChars = minChars
+	ti.completeKey = key
 	return ti
 }
 
@@ -128,28 +128,19 @@ func (ti *TextInput) drawPopover(ctx *Context) {
 	if len(ti.completions) == 0 {
 		return
 	}
-	cmp := &completions{
-		options:    ti.completions,
-		idx:        ti.completeIndex,
-		stringLeft: ti.StringLeft(),
-		onSelect: func(idx int) {
-			ti.completeIndex = idx
-			ti.Invalidate()
-		},
-		onExec: func() {
-			ti.executeCompletion()
-			ti.invalidateCompletions()
-			ti.Invalidate()
-		},
-		onStem: func(stem string) {
-			ti.Set(ti.prefix + stem + ti.StringRight())
-			ti.Invalidate()
-		},
-		uiConfig: ti.uiConfig,
-	}
+	cmp := &completions{ti: ti}
 	width := maxLen(ti.completions) + 3
 	height := len(ti.completions)
-	ctx.Popover(0, 0, width, height, cmp)
+
+	pos := len(ti.prefix) - ti.scroll
+	if pos+width > ctx.Width() {
+		pos = ctx.Width() - width
+	}
+	if pos < 0 {
+		pos = 0
+	}
+
+	ctx.Popover(pos, 0, width, height, cmp)
 }
 
 func (ti *TextInput) MouseEvent(localX int, localY int, event tcell.Event) {
@@ -209,7 +200,7 @@ func (ti *TextInput) deleteWord() {
 	for i >= 0 && ti.text[i] == ' ' {
 		i--
 	}
-	if strings.ContainsRune(separators, ti.text[i]) {
+	if i >= 0 && strings.ContainsRune(separators, ti.text[i]) {
 		for i >= 0 && strings.ContainsRune(separators, ti.text[i]) {
 			i--
 		}
@@ -296,12 +287,16 @@ func (ti *TextInput) updateCompletions() {
 		// no completer
 		return
 	}
+	if ti.completeMinChars == config.MANUAL_COMPLETE {
+		// only manually triggered completion
+		return
+	}
 	if ti.completeDebouncer == nil {
 		ti.completeDebouncer = time.AfterFunc(ti.completeDelay, func() {
 			defer log.PanicHandler()
 			ti.Lock()
 			if len(ti.StringLeft()) >= ti.completeMinChars {
-				ti.showCompletions()
+				ti.showCompletions(false)
 			}
 			ti.Unlock()
 		})
@@ -311,14 +306,22 @@ func (ti *TextInput) updateCompletions() {
 	}
 }
 
-func (ti *TextInput) showCompletions() {
+func (ti *TextInput) showCompletions(explicit bool) {
 	if ti.tabcomplete == nil {
 		// no completer
 		return
 	}
 	ti.completions, ti.prefix = ti.tabcomplete(ti.StringLeft())
-	ti.completeIndex = -1
-	QueueRedraw()
+
+	if explicit && len(ti.completions) == 1 {
+		// automatically accept if there is only one choice
+		ti.completeIndex = 0
+		ti.executeCompletion()
+		ti.invalidateCompletions()
+	} else {
+		ti.completeIndex = -1
+	}
+	Invalidate()
 }
 
 func (ti *TextInput) OnChange(onChange func(ti *TextInput)) {
@@ -333,55 +336,48 @@ func (ti *TextInput) Event(event tcell.Event) bool {
 	ti.Lock()
 	defer ti.Unlock()
 	if event, ok := event.(*tcell.EventKey); ok {
+		c := ti.completeKey
+		if c != nil && c.Key == event.Key() && c.Modifiers == event.Modifiers() {
+			ti.showCompletions(true)
+			return true
+		}
+
+		ti.invalidateCompletions()
+
 		switch event.Key() {
 		case tcell.KeyBackspace, tcell.KeyBackspace2:
-			ti.invalidateCompletions()
 			ti.backspace()
 		case tcell.KeyCtrlD, tcell.KeyDelete:
-			ti.invalidateCompletions()
 			ti.deleteChar()
 		case tcell.KeyCtrlB, tcell.KeyLeft:
-			ti.invalidateCompletions()
 			if ti.index > 0 {
 				ti.index--
 				ti.ensureScroll()
 				ti.Invalidate()
 			}
 		case tcell.KeyCtrlF, tcell.KeyRight:
-			ti.invalidateCompletions()
 			if ti.index < len(ti.text) {
 				ti.index++
 				ti.ensureScroll()
 				ti.Invalidate()
 			}
 		case tcell.KeyCtrlA, tcell.KeyHome:
-			ti.invalidateCompletions()
 			ti.index = 0
 			ti.ensureScroll()
 			ti.Invalidate()
 		case tcell.KeyCtrlE, tcell.KeyEnd:
-			ti.invalidateCompletions()
 			ti.index = len(ti.text)
 			ti.ensureScroll()
 			ti.Invalidate()
 		case tcell.KeyCtrlK:
-			ti.invalidateCompletions()
 			ti.deleteLineForward()
 		case tcell.KeyCtrlW:
-			ti.invalidateCompletions()
 			ti.deleteWord()
 		case tcell.KeyCtrlU:
-			ti.invalidateCompletions()
 			ti.deleteLineBackward()
 		case tcell.KeyESC:
-			if ti.completions != nil {
-				ti.invalidateCompletions()
-				ti.Invalidate()
-			}
-		case tcell.KeyTab:
-			ti.showCompletions()
+			ti.Invalidate()
 		case tcell.KeyRune:
-			ti.invalidateCompletions()
 			ti.insert(event.Rune())
 		}
 	}
@@ -389,19 +385,20 @@ func (ti *TextInput) Event(event tcell.Event) bool {
 }
 
 type completions struct {
-	options    []string
-	stringLeft string
-	idx        int
-	onSelect   func(int)
-	onExec     func()
-	onStem     func(string)
-	uiConfig   *config.UIConfig
+	ti *TextInput
+}
+
+func unquote(s string) string {
+	if strings.HasPrefix(s, "'") && strings.HasSuffix(s, "'") {
+		s = strings.ReplaceAll(s[1:len(s)-1], `'"'"'`, "'")
+	}
+	return s
 }
 
 func maxLen(ss []string) int {
 	max := 0
 	for _, s := range ss {
-		l := runewidth.StringWidth(s)
+		l := runewidth.StringWidth(unquote(s))
 		if l > max {
 			max = l
 		}
@@ -410,36 +407,36 @@ func maxLen(ss []string) int {
 }
 
 func (c *completions) Draw(ctx *Context) {
-	bg := c.uiConfig.GetStyle(config.STYLE_COMPLETION_DEFAULT)
-	gutter := c.uiConfig.GetStyle(config.STYLE_COMPLETION_GUTTER)
-	pill := c.uiConfig.GetStyle(config.STYLE_COMPLETION_PILL)
-	sel := c.uiConfig.GetStyleSelected(config.STYLE_COMPLETION_DEFAULT)
+	bg := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_DEFAULT)
+	gutter := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_GUTTER)
+	pill := c.ti.uiConfig.GetStyle(config.STYLE_COMPLETION_PILL)
+	sel := c.ti.uiConfig.GetStyleSelected(config.STYLE_COMPLETION_DEFAULT)
 
 	ctx.Fill(0, 0, ctx.Width(), ctx.Height(), ' ', bg)
 
 	numVisible := ctx.Height()
 	startIdx := 0
-	if len(c.options) > numVisible && c.idx+1 > numVisible {
-		startIdx = c.idx - (numVisible - 1)
+	if len(c.ti.completions) > numVisible && c.index()+1 > numVisible {
+		startIdx = c.index() - (numVisible - 1)
 	}
 	endIdx := startIdx + numVisible - 1
 
-	for idx, opt := range c.options {
+	for idx, opt := range c.ti.completions {
 		if idx < startIdx {
 			continue
 		}
 		if idx > endIdx {
 			continue
 		}
-		if c.idx == idx {
+		if c.index() == idx {
 			ctx.Fill(0, idx-startIdx, ctx.Width(), 1, ' ', sel)
-			ctx.Printf(0, idx-startIdx, sel, " %s ", opt)
+			ctx.Printf(0, idx-startIdx, sel, " %s ", unquote(opt))
 		} else {
-			ctx.Printf(0, idx-startIdx, bg, " %s ", opt)
+			ctx.Printf(0, idx-startIdx, bg, " %s ", unquote(opt))
 		}
 	}
 
-	percentVisible := float64(numVisible) / float64(len(c.options))
+	percentVisible := float64(numVisible) / float64(len(c.ti.completions))
 	if percentVisible >= 1.0 {
 		return
 	}
@@ -448,44 +445,59 @@ func (c *completions) Draw(ctx *Context) {
 	ctx.Fill(ctx.Width()-1, 0, 1, ctx.Height(), ' ', gutter)
 
 	pillSize := int(math.Ceil(float64(ctx.Height()) * percentVisible))
-	percentScrolled := float64(startIdx) / float64(len(c.options))
+	percentScrolled := float64(startIdx) / float64(len(c.ti.completions))
 	pillOffset := int(math.Floor(float64(ctx.Height()) * percentScrolled))
 	ctx.Fill(ctx.Width()-1, pillOffset, 1, pillSize, ' ', pill)
 }
 
+func (c *completions) index() int {
+	return c.ti.completeIndex
+}
+
 func (c *completions) next() {
-	idx := c.idx
-	idx++
-	if idx > len(c.options)-1 {
-		idx = -1
+	index := c.index()
+	index++
+	if index >= len(c.ti.completions) {
+		index = -1
 	}
-	c.onSelect(idx)
+	c.ti.completeIndex = index
+	Invalidate()
 }
 
 func (c *completions) prev() {
-	idx := c.idx
-	idx--
-	if idx < -1 {
-		idx = len(c.options) - 1
+	index := c.index()
+	index--
+	if index < -1 {
+		index = len(c.ti.completions) - 1
 	}
-	c.onSelect(idx)
+	c.ti.completeIndex = index
+	Invalidate()
+}
+
+func (c *completions) exec() {
+	c.ti.executeCompletion()
+	c.ti.invalidateCompletions()
+	Invalidate()
 }
 
 func (c *completions) Event(e tcell.Event) bool {
 	if e, ok := e.(*tcell.EventKey); ok {
-		switch e.Key() {
-		case tcell.KeyTab:
-			if len(c.options) == 1 && c.idx >= 0 {
-				c.onExec()
+		k := c.ti.completeKey
+		if k != nil && k.Key == e.Key() && k.Modifiers == e.Modifiers() {
+			if len(c.ti.completions) == 1 {
+				c.ti.completeIndex = 0
+				c.exec()
 			} else {
-				stem := findStem(c.options)
-				if stem != "" && stem != c.stringLeft {
-					c.onStem(stem)
-				} else {
-					c.next()
+				stem := findStem(c.ti.completions)
+				if c.needsStem(stem) {
+					c.stem(stem)
 				}
+				c.next()
 			}
 			return true
+		}
+
+		switch e.Key() {
 		case tcell.KeyCtrlN, tcell.KeyDown:
 			c.next()
 			return true
@@ -493,8 +505,8 @@ func (c *completions) Event(e tcell.Event) bool {
 			c.prev()
 			return true
 		case tcell.KeyEnter:
-			if c.idx >= 0 {
-				c.onExec()
+			if c.index() >= 0 {
+				c.exec()
 				return true
 			}
 		}
@@ -502,6 +514,21 @@ func (c *completions) Event(e tcell.Event) bool {
 	return false
 }
 
+func (c *completions) needsStem(stem string) bool {
+	if stem == "" || c.index() >= 0 {
+		return false
+	}
+	if len(stem)+len(c.ti.prefix) > len(c.ti.StringLeft()) {
+		return true
+	}
+	return false
+}
+
+func (c *completions) stem(stem string) {
+	c.ti.Set(c.ti.prefix + stem + c.ti.StringRight())
+	c.ti.index = runewidth.StringWidth(c.ti.prefix + stem)
+}
+
 func findStem(words []string) string {
 	if len(words) == 0 {
 		return ""
