@@ -7,16 +7,17 @@ import (
 	"strings"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/lib/parse"
 	"github.com/emersion/go-message/mail"
 )
 
-// Flag is an abstraction around the different flags which can be present in
+// Flags is an abstraction around the different flags which can be present in
 // different email backends and represents a flag that we use in the UI.
-type Flag int
+type Flags uint32
 
 const (
 	// SeenFlag marks a message as having been seen previously
-	SeenFlag Flag = iota
+	SeenFlag Flags = 1 << iota
 
 	// RecentFlag marks a message as being recent
 	RecentFlag
@@ -29,32 +30,62 @@ const (
 
 	// FlaggedFlag marks a message with a user flag
 	FlaggedFlag
+
+	// DraftFlag marks a message as a draft
+	DraftFlag
+)
+
+func (f Flags) Has(flags Flags) bool {
+	return f&flags == flags
+}
+
+type Role string
+
+var Roles = map[string]Role{
+	"all":     AllRole,
+	"archive": ArchiveRole,
+	"drafts":  DraftsRole,
+	"inbox":   InboxRole,
+	"junk":    JunkRole,
+	"sent":    SentRole,
+	"trash":   TrashRole,
+	"query":   QueryRole,
+}
+
+const (
+	AllRole     Role = "all"
+	ArchiveRole Role = "archive"
+	DraftsRole  Role = "drafts"
+	InboxRole   Role = "inbox"
+	JunkRole    Role = "junk"
+	SentRole    Role = "sent"
+	TrashRole   Role = "trash"
+	// Custom aerc roles
+	QueryRole Role = "query"
+	// virtual node created by the directory tree
+	VirtualRole Role = "virtual"
 )
 
 type Directory struct {
-	Name       string
-	Attributes []string
+	Name string
+	// Exists messages in the Directory
+	Exists int
+	// Recent messages in the Directory
+	Recent int
+	// Unseen messages in the Directory
+	Unseen int
+	// IANA role
+	Role Role
 }
 
 type DirectoryInfo struct {
-	Name     string
-	Flags    []string
-	ReadOnly bool
-
+	Name string
 	// The total number of messages in this mailbox.
 	Exists int
-
 	// The number of messages not seen since the last time the mailbox was opened.
 	Recent int
-
 	// The number of unread messages
 	Unseen int
-
-	// set to true if the value counts are accurate
-	AccurateCounts bool
-
-	// Caps contains the backend capabilities
-	Caps *Capabilities
 }
 
 // Capabilities provides the backend capabilities
@@ -67,8 +98,9 @@ type Capabilities struct {
 type MessageInfo struct {
 	BodyStructure *BodyStructure
 	Envelope      *Envelope
-	Flags         []Flag
+	Flags         Flags
 	Labels        []string
+	Filenames     []string
 	InternalDate  time.Time
 	RFC822Headers *mail.Header
 	Refs          []string
@@ -97,10 +129,7 @@ func (mi *MessageInfo) InReplyTo() (msgid string, err error) {
 	if mi.RFC822Headers == nil {
 		return "", errors.New("header is nil")
 	}
-	list, err := mi.RFC822Headers.MsgIDList("In-Reply-To")
-	if err != nil {
-		return "", err
-	}
+	list := parse.MsgIDList(mi.RFC822Headers, "In-Reply-To")
 	if len(list) == 0 {
 		return "", errors.New("no results")
 	}
@@ -117,14 +146,11 @@ func (mi *MessageInfo) References() ([]string, error) {
 	if mi.RFC822Headers == nil {
 		return []string{}, errors.New("header is nil")
 	}
-	list, err := mi.RFC822Headers.MsgIDList("References")
-	if err != nil {
-		return []string{}, err
-	}
+	list := parse.MsgIDList(mi.RFC822Headers, "References")
 	if len(list) == 0 {
 		return []string{}, errors.New("no results")
 	}
-	return list, err
+	return list, nil
 }
 
 // A MessageBodyPart can be displayed in the message viewer
