@@ -20,12 +20,27 @@ type TemplateData interface {
 	DateAutoFormat(date time.Time) string
 	Header(name string) string
 	ThreadPrefix() string
+	ThreadCount() int
+	ThreadUnread() int
+	ThreadFolded() bool
+	ThreadContext() bool
 	Subject() string
 	SubjectBase() string
 	Number() int
 	Labels() []string
+	Filename() string
+	Filenames() []string
 	Flags() []string
+	IsReplied() bool
+	HasAttachment() bool
+	Attach(string) string
+	IsFlagged() bool
+	IsRecent() bool
+	IsUnread() bool
+	IsMarked() bool
+	IsDraft() bool
 	MessageId() string
+	Role() string
 	Size() int
 	OriginalText() string
 	OriginalDate() time.Time
@@ -44,9 +59,12 @@ type TemplateData interface {
 	PendingKeys() string
 	Style(string, string) string
 	StyleSwitch(string, ...Case) string
+	StyleMap([]string, ...Case) []string
+	Signature() string
 }
 
 type Case interface {
 	Matches(string) bool
 	Value() string
+	Skip() bool
 }
