@@ -6,18 +6,17 @@ import (
 	"encoding/gob"
 	"errors"
 	"fmt"
-	"os"
-	"path"
+	"strings"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/lib/parse"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 	"github.com/emersion/go-message"
 	"github.com/emersion/go-message/mail"
 	"github.com/emersion/go-message/textproto"
-	"github.com/mitchellh/go-homedir"
 	"github.com/syndtr/goleveldb/leveldb"
 )
 
@@ -35,45 +34,46 @@ var (
 	// cacheTag should be updated when changing the cache
 	// structure; this will ensure that the user's cache is cleared and
 	// reloaded when the underlying cache structure changes
-	cacheTag    = []byte("0001")
+	cacheTag    = []byte("0002")
 	cacheTagKey = []byte("cache.tag")
 )
 
 // initCacheDb opens (or creates) the database for the cache. One database is
 // created per account
 func (w *IMAPWorker) initCacheDb(acct string) {
-	cd, err := cacheDir()
-	if err != nil {
-		w.cache = nil
-		log.Errorf("unable to find cache directory: %v", err)
-		return
+	switch {
+	case len(w.config.headersExclude) > 0:
+		headerTag := strings.Join(w.config.headersExclude, "")
+		cacheTag = append(cacheTag, headerTag...)
+	case len(w.config.headers) > 0:
+		headerTag := strings.Join(w.config.headers, "")
+		cacheTag = append(cacheTag, headerTag...)
 	}
-	p := path.Join(cd, acct)
+	p := xdg.CachePath("aerc", acct)
 	db, err := leveldb.OpenFile(p, nil)
 	if err != nil {
 		w.cache = nil
-		log.Errorf("failed opening cache db: %v", err)
+		w.worker.Errorf("failed opening cache db: %v", err)
 		return
 	}
 	w.cache = db
-	log.Debugf("cache db opened: %s", p)
+	w.worker.Debugf("cache db opened: %s", p)
 
 	tag, err := w.cache.Get(cacheTagKey, nil)
 	clearCache := errors.Is(err, leveldb.ErrNotFound) ||
 		!bytes.Equal(tag, cacheTag)
 	switch {
 	case clearCache:
-		log.Infof("current cache tag is '%s' but found '%s'",
+		w.worker.Infof("current cache tag is '%s' but found '%s'",
 			cacheTag, tag)
-		log.Warnf("tag mismatch: clear cache")
+		w.worker.Warnf("tag mismatch: clear cache")
 		w.clearCache()
 		if err = w.cache.Put(cacheTagKey, cacheTag, nil); err != nil {
-			log.Errorf("could not set the current cache tag")
+			w.worker.Errorf("could not set the current cache tag")
 		}
 	case err != nil:
-		log.Errorf("could not get the cache tag from db")
+		w.worker.Errorf("could not get the cache tag from db")
 	default:
-		log.Tracef("cache version match")
 		if w.config.cacheMaxAge.Hours() > 0 {
 			go w.cleanCache(p)
 		}
@@ -81,13 +81,12 @@ func (w *IMAPWorker) initCacheDb(acct string) {
 }
 
 func (w *IMAPWorker) cacheHeader(mi *models.MessageInfo) {
-	uv := fmt.Sprintf("%d", w.selected.UidValidity)
-	uid := fmt.Sprintf("%d", mi.Uid)
-	log.Debugf("caching header for message %s.%s", uv, uid)
+	key := w.headerKey(mi.Uid)
+	w.worker.Debugf("caching header for message %s", key)
 	hdr := bytes.NewBuffer(nil)
 	err := textproto.WriteHeader(hdr, mi.RFC822Headers.Header.Header)
 	if err != nil {
-		log.Errorf("cannot write header %s.%s: %v", uv, uid, err)
+		w.worker.Errorf("cannot write header %s: %v", key, err)
 		return
 	}
 	h := &CachedHeader{
@@ -103,23 +102,22 @@ func (w *IMAPWorker) cacheHeader(mi *models.MessageInfo) {
 	enc := gob.NewEncoder(data)
 	err = enc.Encode(h)
 	if err != nil {
-		log.Errorf("cannot encode message %s.%s: %v", uv, uid, err)
+		w.worker.Errorf("cannot encode message %s: %v", key, err)
 		return
 	}
-	err = w.cache.Put([]byte("header."+uv+"."+uid), data.Bytes(), nil)
+	err = w.cache.Put(key, data.Bytes(), nil)
 	if err != nil {
-		log.Errorf("cannot write header for message %s.%s: %v", uv, uid, err)
+		w.worker.Errorf("cannot write header for message %s: %v", key, err)
 		return
 	}
 }
 
 func (w *IMAPWorker) getCachedHeaders(msg *types.FetchMessageHeaders) []uint32 {
-	log.Tracef("Retrieving headers from cache: %v", msg.Uids)
+	w.worker.Tracef("Retrieving headers from cache: %v", msg.Uids)
 	var need []uint32
-	uv := fmt.Sprintf("%d", w.selected.UidValidity)
 	for _, uid := range msg.Uids {
-		u := fmt.Sprintf("%d", uid)
-		data, err := w.cache.Get([]byte("header."+uv+"."+u), nil)
+		key := w.headerKey(uid)
+		data, err := w.cache.Get(key, nil)
 		if err != nil {
 			need = append(need, uid)
 			continue
@@ -128,14 +126,14 @@ func (w *IMAPWorker) getCachedHeaders(msg *types.FetchMessageHeaders) []uint32 {
 		dec := gob.NewDecoder(bytes.NewReader(data))
 		err = dec.Decode(ch)
 		if err != nil {
-			log.Errorf("cannot decode cached header %s.%s: %v", uv, u, err)
+			w.worker.Errorf("cannot decode cached header %s: %v", key, err)
 			need = append(need, uid)
 			continue
 		}
 		hr := bytes.NewReader(ch.Header)
 		textprotoHeader, err := textproto.ReadHeader(bufio.NewReader(hr))
 		if err != nil {
-			log.Errorf("cannot read cached header %s.%s: %v", uv, u, err)
+			w.worker.Errorf("cannot read cached header %s: %v", key, err)
 			need = append(need, uid)
 			continue
 		}
@@ -150,7 +148,6 @@ func (w *IMAPWorker) getCachedHeaders(msg *types.FetchMessageHeaders) []uint32 {
 			Refs:          parse.MsgIDList(hdr, "references"),
 			Size:          ch.Size,
 		}
-		log.Tracef("located cached header %s.%s", uv, u)
 		w.worker.PostMessage(&types.MessageInfo{
 			Message:    types.RespondTo(msg),
 			Info:       mi,
@@ -160,15 +157,10 @@ func (w *IMAPWorker) getCachedHeaders(msg *types.FetchMessageHeaders) []uint32 {
 	return need
 }
 
-func cacheDir() (string, error) {
-	dir, err := os.UserCacheDir()
-	if err != nil {
-		dir, err = homedir.Expand("~/.cache")
-		if err != nil {
-			return "", err
-		}
-	}
-	return path.Join(dir, "aerc"), nil
+func (w *IMAPWorker) headerKey(uid uint32) []byte {
+	key := fmt.Sprintf("header.%s.%d.%d",
+		w.selected.Name, w.selected.UidValidity, uid)
+	return []byte(key)
 }
 
 // cleanCache removes stale entries from the selected mailbox cachedb
@@ -186,14 +178,16 @@ func (w *IMAPWorker) cleanCache(path string) {
 		dec := gob.NewDecoder(bytes.NewReader(data))
 		err := dec.Decode(ch)
 		if err != nil {
-			log.Errorf("cannot clean database %d: %v", w.selected.UidValidity, err)
+			w.worker.Errorf("cannot clean database %d: %v",
+				w.selected.UidValidity, err)
 			continue
 		}
 		exp := ch.Created.Add(w.config.cacheMaxAge)
 		if exp.Before(time.Now()) {
 			err = w.cache.Delete(iter.Key(), nil)
 			if err != nil {
-				log.Errorf("cannot clean database %d: %v", w.selected.UidValidity, err)
+				w.worker.Errorf("cannot clean database %d: %v",
+					w.selected.UidValidity, err)
 				continue
 			}
 			removed++
@@ -202,7 +196,7 @@ func (w *IMAPWorker) cleanCache(path string) {
 	}
 	iter.Release()
 	elapsed := time.Since(start)
-	log.Debugf("%s: removed %d/%d expired entries in %s",
+	w.worker.Debugf("%s: removed %d/%d expired entries in %s",
 		path, removed, scanned, elapsed)
 }
 
@@ -211,7 +205,7 @@ func (w *IMAPWorker) clearCache() {
 	iter := w.cache.NewIterator(nil, nil)
 	for iter.Next() {
 		if err := w.cache.Delete(iter.Key(), nil); err != nil {
-			log.Errorf("error clearing cache: %v", err)
+			w.worker.Errorf("error clearing cache: %v", err)
 		}
 	}
 	iter.Release()
