@@ -82,7 +82,7 @@ func (w *IMAPWorker) connect() (*client.Client, error) {
 			}
 		} else if w.config.xoauth2.Enabled {
 			if err := w.config.xoauth2.Authenticate(
-				username, password, w.worker.Name, c); err != nil {
+				username, password, w.config.name, c); err != nil {
 				return nil, err
 			}
 		} else if err := c.Login(username, password); err != nil {
@@ -94,6 +94,17 @@ func (w *IMAPWorker) connect() (*client.Client, error) {
 		return nil, err
 	}
 
+	info := make(chan *imap.MailboxInfo, 1)
+	if err := c.List("", "", info); err != nil {
+		return nil, fmt.Errorf("failed to retrieve delimiter: %w", err)
+	}
+	mailboxinfo := <-info
+	w.delimiter = mailboxinfo.Delimiter
+	if w.delimiter == "" {
+		// just in case some implementation does not follow standards
+		w.delimiter = "/"
+	}
+
 	return c, nil
 }
 
@@ -167,12 +178,12 @@ func (w *IMAPWorker) setKeepaliveParameters(conn *net.TCPConn) error {
 		// Max number of probes before failure
 		err := lib.SetTcpKeepaliveProbes(fd, w.config.keepalive_probes)
 		if err != nil {
-			log.Errorf("cannot set tcp keepalive probes: %v", err)
+			w.worker.Errorf("cannot set tcp keepalive probes: %v", err)
 		}
 		// Wait time after an unsuccessful probe
 		err = lib.SetTcpKeepaliveInterval(fd, w.config.keepalive_interval)
 		if err != nil {
-			log.Errorf("cannot set tcp keepalive interval: %v", err)
+			w.worker.Errorf("cannot set tcp keepalive interval: %v", err)
 		}
 	})
 	return err
