@@ -19,6 +19,12 @@ import (
 func (imapw *IMAPWorker) handleFetchMessageHeaders(
 	msg *types.FetchMessageHeaders,
 ) {
+	if msg.Context.Err() != nil {
+		imapw.worker.PostMessage(&types.Cancelled{
+			Message: types.RespondTo(msg),
+		}, nil)
+		return
+	}
 	toFetch := msg.Uids
 	if imapw.config.cacheEnabled && imapw.cache != nil {
 		toFetch = imapw.getCachedHeaders(msg)
@@ -28,12 +34,20 @@ func (imapw *IMAPWorker) handleFetchMessageHeaders(
 			nil)
 		return
 	}
-	log.Tracef("Fetching message headers: %v", toFetch)
+	imapw.worker.Tracef("Fetching message headers: %v", toFetch)
+	hdrBodyPart := imap.BodyPartName{
+		Specifier: imap.HeaderSpecifier,
+	}
+	switch {
+	case len(imapw.config.headersExclude) > 0:
+		hdrBodyPart.NotFields = true
+		hdrBodyPart.Fields = imapw.config.headersExclude
+	case len(imapw.config.headers) > 0:
+		hdrBodyPart.Fields = imapw.config.headers
+	}
 	section := &imap.BodySectionName{
-		BodyPartName: imap.BodyPartName{
-			Specifier: imap.HeaderSpecifier,
-		},
-		Peek: true,
+		BodyPartName: hdrBodyPart,
+		Peek:         true,
 	}
 
 	items := []imap.FetchItem{
@@ -84,7 +98,7 @@ func (imapw *IMAPWorker) handleFetchMessageHeaders(
 func (imapw *IMAPWorker) handleFetchMessageBodyPart(
 	msg *types.FetchMessageBodyPart,
 ) {
-	log.Tracef("Fetching message %d part: %v", msg.Uid, msg.Part)
+	imapw.worker.Tracef("Fetching message %d part: %v", msg.Uid, msg.Part)
 
 	var partHeaderSection imap.BodySectionName
 	partHeaderSection.Peek = true
@@ -130,7 +144,7 @@ func (imapw *IMAPWorker) handleFetchMessageBodyPart(
 			part, err := message.New(message.Header{Header: h},
 				_msg.GetBody(&partBodySection))
 			if message.IsUnknownCharset(err) {
-				log.Warnf("unknown charset encountered "+
+				imapw.worker.Warnf("unknown charset encountered "+
 					"for uid %d", _msg.Uid)
 			} else if err != nil {
 				return fmt.Errorf("failed to create message reader: %w", err)
@@ -158,7 +172,7 @@ func (imapw *IMAPWorker) handleFetchMessageBodyPart(
 func (imapw *IMAPWorker) handleFetchFullMessages(
 	msg *types.FetchFullMessages,
 ) {
-	log.Tracef("Fetching full messages: %v", msg.Uids)
+	imapw.worker.Tracef("Fetching full messages: %v", msg.Uids)
 	section := &imap.BodySectionName{
 		Peek: true,
 	}
@@ -202,6 +216,12 @@ func (imapw *IMAPWorker) handleFetchMessageFlags(msg *types.FetchMessageFlags) {
 		imap.FetchFlags,
 		imap.FetchUid,
 	}
+	if msg.Context.Err() != nil {
+		imapw.worker.PostMessage(&types.Cancelled{
+			Message: types.RespondTo(msg),
+		}, nil)
+		return
+	}
 	imapw.handleFetchMessages(msg, msg.Uids, items,
 		func(_msg *imap.Message) error {
 			imapw.worker.PostMessage(&types.MessageInfo{
@@ -219,22 +239,19 @@ func (imapw *IMAPWorker) handleFetchMessages(
 	msg types.WorkerMessage, uids []uint32, items []imap.FetchItem,
 	procFunc func(*imap.Message) error,
 ) {
+	var err error
 	messages := make(chan *imap.Message)
-	done := make(chan error)
+	done := make(chan []error)
 
 	go func() {
 		defer log.PanicHandler()
 
-		var reterr error
-		for _msg := range messages {
-			err := procFunc(_msg)
+		var reterr []error
+		for msg := range messages {
+			err := procFunc(msg)
 			if err != nil {
-				if reterr == nil {
-					reterr = err
-				}
-				// drain the channel upon error
-				for range messages {
-				}
+				log.Errorf("failed to process message <%d>: %v", msg.Uid, err)
+				reterr = append(reterr, err)
 			}
 		}
 		done <- reterr
@@ -248,11 +265,15 @@ func (imapw *IMAPWorker) handleFetchMessages(
 	}
 
 	set := toSeqSet(uids)
-	if err := imapw.client.UidFetch(set, items, messages); err != nil {
+	if err = imapw.client.UidFetch(set, items, messages); err != nil {
 		emitErr(err)
 		return
 	}
-	if err := <-done; err != nil {
+	if errs := <-done; len(errs) != 0 {
+		err = errs[0]
+		if len(errs) > 1 {
+			err = fmt.Errorf("parsing of %d messages failed", len(errs))
+		}
 		emitErr(err)
 		return
 	}
