@@ -34,7 +34,6 @@ func (imapw *IMAPWorker) handleAnsweredMessages(msg *types.AnsweredMessages) {
 	flags := []interface{}{imap.AnsweredFlag}
 	if !msg.Answered {
 		item = imap.FormatFlagsOp(imap.RemoveFlags, false)
-		flags = []interface{}{imap.AnsweredFlag}
 	}
 	imapw.handleStoreOps(msg, msg.Uids, item, flags,
 		func(_msg *imap.Message) error {
@@ -50,7 +49,7 @@ func (imapw *IMAPWorker) handleAnsweredMessages(msg *types.AnsweredMessages) {
 }
 
 func (imapw *IMAPWorker) handleFlagMessages(msg *types.FlagMessages) {
-	flags := []interface{}{flagToImap[msg.Flag]}
+	flags := []interface{}{flagToImap[msg.Flags]}
 	item := imap.FormatFlagsOp(imap.AddFlags, false)
 	if !msg.Enable {
 		item = imap.FormatFlagsOp(imap.RemoveFlags, false)
@@ -109,6 +108,9 @@ func (imapw *IMAPWorker) handleStoreOps(
 		emitErr(err)
 		return
 	}
+	imapw.worker.PostAction(&types.CheckMail{
+		Directories: []string{imapw.selected.Name},
+	}, nil)
 	imapw.worker.PostMessage(
 		&types.Done{Message: types.RespondTo(msg)}, nil)
 }
