@@ -12,7 +12,7 @@ import (
 
 func (imapw *IMAPWorker) handleListDirectories(msg *types.ListDirectories) {
 	mailboxes := make(chan *imap.MailboxInfo)
-	log.Tracef("Listing mailboxes")
+	imapw.worker.Tracef("Listing mailboxes")
 	done := make(chan interface{})
 
 	go func() {
@@ -94,9 +94,12 @@ func (imapw *IMAPWorker) handleListDirectories(msg *types.ListDirectories) {
 		&types.Done{Message: types.RespondTo(msg)}, nil)
 }
 
+const NonExistentAttr = "\\NonExistent"
+
 func canOpen(mbox *imap.MailboxInfo) bool {
 	for _, attr := range mbox.Attributes {
-		if attr == imap.NoSelectAttr {
+		if attr == imap.NoSelectAttr ||
+			attr == NonExistentAttr {
 			return false
 		}
 	}
@@ -111,10 +114,13 @@ func (imapw *IMAPWorker) handleSearchDirectory(msg *types.SearchDirectory) {
 		}, nil)
 	}
 
-	log.Tracef("Executing search")
-	criteria, err := parseSearch(msg.Argv)
-	if err != nil {
-		emitError(err)
+	imapw.worker.Tracef("Executing search")
+	criteria := translateSearch(msg.Criteria)
+
+	if msg.Context.Err() != nil {
+		imapw.worker.PostMessage(&types.Cancelled{
+			Message: types.RespondTo(msg),
+		}, nil)
 		return
 	}
 
@@ -124,6 +130,13 @@ func (imapw *IMAPWorker) handleSearchDirectory(msg *types.SearchDirectory) {
 		return
 	}
 
+	if msg.Context.Err() != nil {
+		imapw.worker.PostMessage(&types.Cancelled{
+			Message: types.RespondTo(msg),
+		}, nil)
+		return
+	}
+
 	imapw.worker.PostMessage(&types.SearchResults{
 		Message: types.RespondTo(msg),
 		Uids:    uids,
