@@ -18,14 +18,14 @@ type observer struct {
 	sync.Mutex
 	config        imapConfig
 	client        *imapClient
-	worker        *types.Worker
+	worker        types.WorkerInteractor
 	done          chan struct{}
 	autoReconnect bool
 	retries       int
 	running       bool
 }
 
-func newObserver(cfg imapConfig, w *types.Worker) *observer {
+func newObserver(cfg imapConfig, w types.WorkerInteractor) *observer {
 	return &observer{config: cfg, worker: w, done: make(chan struct{})}
 }
 
@@ -66,7 +66,6 @@ func (o *observer) IsRunning() bool {
 
 func (o *observer) Start() {
 	if o.running {
-		o.log("runs already")
 		return
 	}
 	if o.client == nil {
@@ -76,16 +75,16 @@ func (o *observer) Start() {
 		return
 	}
 	go func() {
+		defer log.PanicHandler()
 		select {
 		case <-o.client.LoggedOut():
-			o.log("<-logout")
 			if o.autoReconnect {
 				o.emit("logged out")
 			} else {
 				o.log("ignore logout (auto-reconnect off)")
 			}
 		case <-o.done:
-			o.log("<-done")
+			break
 		}
 		o.running = false
 		o.log("stopped")
@@ -106,9 +105,6 @@ func (o *observer) Stop() {
 }
 
 func (o *observer) DelayedReconnect() error {
-	if o.client == nil {
-		return nil
-	}
 	var wait time.Duration
 	var reterr error
 
@@ -129,6 +125,7 @@ func (o *observer) DelayedReconnect() error {
 	}
 
 	go func() {
+		defer log.PanicHandler()
 		<-time.After(wait)
 		o.emit(reterr.Error())
 	}()
@@ -138,11 +135,9 @@ func (o *observer) DelayedReconnect() error {
 }
 
 func (o *observer) emit(errMsg string) {
-	o.log("disconnect done->")
 	o.worker.PostMessage(&types.Done{
 		Message: types.RespondTo(&types.Disconnect{}),
 	}, nil)
-	o.log("connection error->")
 	o.worker.PostMessage(&types.ConnError{
 		Error: fmt.Errorf(errMsg),
 	}, nil)
@@ -150,5 +145,5 @@ func (o *observer) emit(errMsg string) {
 
 func (o *observer) log(format string, args ...interface{}) {
 	msg := fmt.Sprintf(format, args...)
-	log.Tracef("observer (%p) [running:%t] %s", o, o.running, msg)
+	o.worker.Tracef("observer (%p) [running:%t] %s", o, o.running, msg)
 }
