@@ -12,10 +12,10 @@ import (
 	"github.com/syndtr/goleveldb/leveldb"
 
 	"git.sr.ht/~rjarry/aerc/lib"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/imap/extensions"
+	"git.sr.ht/~rjarry/aerc/worker/middleware"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
@@ -39,10 +39,13 @@ type imapClient struct {
 }
 
 type imapConfig struct {
+	name              string
 	scheme            string
 	insecure          bool
 	addr              string
 	user              *url.Userinfo
+	headers           []string
+	headersExclude    []string
 	folders           []string
 	oauthBearer       lib.OAuthBearer
 	xoauth2           lib.Xoauth2
@@ -56,16 +59,18 @@ type imapConfig struct {
 	keepalive_interval int
 	cacheEnabled       bool
 	cacheMaxAge        time.Duration
+	useXGMEXT          bool
 }
 
 type IMAPWorker struct {
 	config imapConfig
 
-	client   *imapClient
-	selected *imap.MailboxStatus
-	updates  chan client.Update
-	worker   *types.Worker
-	seqMap   SeqMap
+	client    *imapClient
+	selected  *imap.MailboxStatus
+	updates   chan client.Update
+	worker    types.WorkerInteractor
+	seqMap    SeqMap
+	delimiter string
 
 	idler    *idler
 	observer *observer
@@ -75,58 +80,67 @@ type IMAPWorker struct {
 
 	threadAlgorithm sortthread.ThreadAlgorithm
 	liststatus      bool
+
+	executeIdle chan struct{}
 }
 
 func NewIMAPWorker(worker *types.Worker) (types.Backend, error) {
 	return &IMAPWorker{
-		updates:  make(chan client.Update, 50),
-		worker:   worker,
-		selected: &imap.MailboxStatus{},
-		idler:    newIdler(imapConfig{}, worker),
-		observer: newObserver(imapConfig{}, worker),
-		caps:     &models.Capabilities{},
+		updates:     make(chan client.Update, 50),
+		worker:      worker,
+		selected:    &imap.MailboxStatus{},
+		idler:       nil, // will be set in configure()
+		observer:    nil, // will be set in configure()
+		caps:        &models.Capabilities{},
+		executeIdle: make(chan struct{}),
 	}, nil
 }
 
 func (w *IMAPWorker) newClient(c *client.Client) {
-	c.Updates = w.updates
+	c.Updates = nil
 	w.client = &imapClient{
 		c,
 		sortthread.NewThreadClient(c),
 		sortthread.NewSortClient(c),
 		extensions.NewListStatusClient(c),
 	}
-	w.idler.SetClient(w.client)
-	w.observer.SetClient(w.client)
+	if w.idler != nil {
+		w.idler.SetClient(w.client)
+		c.Updates = w.updates
+	}
+	if w.observer != nil {
+		w.observer.SetClient(w.client)
+	}
 	sort, err := w.client.sort.SupportSort()
 	if err == nil && sort {
 		w.caps.Sort = true
-		log.Debugf("Server Capability found: Sort")
+		w.worker.Debugf("Server Capability found: Sort")
 	}
 	for _, alg := range []sortthread.ThreadAlgorithm{sortthread.References, sortthread.OrderedSubject} {
 		ok, err := w.client.Support(fmt.Sprintf("THREAD=%s", string(alg)))
 		if err == nil && ok {
 			w.threadAlgorithm = alg
 			w.caps.Thread = true
-			log.Debugf("Server Capability found: Thread (algorithm: %s)", string(alg))
+			w.worker.Debugf("Server Capability found: Thread (algorithm: %s)", string(alg))
 			break
 		}
 	}
 	lStatus, err := w.client.liststatus.SupportListStatus()
 	if err == nil && lStatus {
 		w.liststatus = true
-		log.Debugf("Server Capability found: LIST-STATUS")
+		w.worker.Debugf("Server Capability found: LIST-STATUS")
+	}
+	xgmext, err := w.client.Support("X-GM-EXT-1")
+	if err == nil && xgmext && w.config.useXGMEXT {
+		w.worker.Debugf("Server Capability found: X-GM-EXT-1")
+		w.worker = middleware.NewGmailWorker(w.worker, w.client.Client)
+	}
+	if err == nil && !xgmext && w.config.useXGMEXT {
+		w.worker.Infof("X-GM-EXT-1 requested, but it is not supported")
 	}
 }
 
 func (w *IMAPWorker) handleMessage(msg types.WorkerMessage) error {
-	defer func() {
-		w.idler.Start()
-	}()
-	if err := w.idler.Stop(); err != nil {
-		return err
-	}
-
 	var reterr error // will be returned at the end, needed to support idle
 
 	// when client is nil allow only certain messages to be handled
@@ -187,12 +201,14 @@ func (w *IMAPWorker) handleMessage(msg types.WorkerMessage) error {
 	case *types.Disconnect:
 		w.observer.SetAutoReconnect(false)
 		w.observer.Stop()
-		if w.client == nil || w.client.State() != imap.SelectedState {
+
+		if w.client == nil || (w.client != nil && w.client.State() != imap.SelectedState) {
 			reterr = errNotConnected
 			break
 		}
 
 		if err := w.client.Logout(); err != nil {
+			w.terminate()
 			reterr = err
 			break
 		}
@@ -246,7 +262,7 @@ func (w *IMAPWorker) handleMessage(msg types.WorkerMessage) error {
 }
 
 func (w *IMAPWorker) handleImapUpdate(update client.Update) {
-	log.Tracef("(= %T", update)
+	w.worker.Tracef("(= %T", update)
 	switch update := update.(type) {
 	case *client.MailboxUpdate:
 		w.worker.PostAction(&types.CheckMail{
@@ -256,7 +272,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 		msg := update.Message
 		if msg.Uid == 0 {
 			if uid, found := w.seqMap.Get(msg.SeqNum); !found {
-				log.Errorf("MessageUpdate unknown seqnum: %d", msg.SeqNum)
+				w.worker.Errorf("MessageUpdate unknown seqnum: %d", msg.SeqNum)
 				return
 			} else {
 				msg.Uid = uid
@@ -276,7 +292,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 		}, nil)
 	case *client.ExpungeUpdate:
 		if uid, found := w.seqMap.Pop(update.SeqNum); !found {
-			log.Errorf("ExpungeUpdate unknown seqnum: %d", update.SeqNum)
+			w.worker.Errorf("ExpungeUpdate unknown seqnum: %d", update.SeqNum)
 		} else {
 			w.worker.PostMessage(&types.MessagesDeleted{
 				Uids: []uint32{uid},
@@ -285,10 +301,64 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 	}
 }
 
+func (w *IMAPWorker) terminate() {
+	if w.observer != nil {
+		w.observer.Stop()
+		w.observer.SetClient(nil)
+	}
+
+	if w.client != nil {
+		w.client.Updates = nil
+		if err := w.client.Terminate(); err != nil {
+			w.worker.Errorf("could not terminate connection: %v", err)
+		}
+	}
+
+	w.client = nil
+	w.selected = &imap.MailboxStatus{}
+
+	if w.idler != nil {
+		w.idler.SetClient(nil)
+	}
+}
+
+func (w *IMAPWorker) stopIdler() error {
+	if w.idler == nil {
+		return nil
+	}
+
+	if err := w.idler.Stop(); err != nil {
+		w.terminate()
+		w.observer.EmitIfNotConnected()
+		w.worker.Errorf("idler stopped with error:%v", err)
+		return err
+	}
+
+	return nil
+}
+
+func (w *IMAPWorker) startIdler() {
+	if w.idler == nil {
+		return
+	}
+
+	w.idler.Start()
+}
+
 func (w *IMAPWorker) Run() {
 	for {
 		select {
-		case msg := <-w.worker.Actions:
+		case msg := <-w.worker.Actions():
+
+			if err := w.stopIdler(); err != nil {
+				w.worker.PostMessage(&types.Error{
+					Message: types.RespondTo(msg),
+					Error:   err,
+				}, nil)
+				break
+			}
+			w.worker.Tracef("ready to handle %T", msg)
+
 			msg = w.worker.ProcessAction(msg)
 
 			if err := w.handleMessage(msg); errors.Is(err, errUnsupported) {
@@ -302,8 +372,13 @@ func (w *IMAPWorker) Run() {
 				}, nil)
 			}
 
+			w.startIdler()
+
 		case update := <-w.updates:
 			w.handleImapUpdate(update)
+
+		case <-w.executeIdle:
+			w.idler.Execute()
 		}
 	}
 }
@@ -311,3 +386,10 @@ func (w *IMAPWorker) Run() {
 func (w *IMAPWorker) Capabilities() *models.Capabilities {
 	return w.caps
 }
+
+func (w *IMAPWorker) PathSeparator() string {
+	if w.delimiter == "" {
+		return "/"
+	}
+	return w.delimiter
+}
