@@ -6,6 +6,7 @@ import (
 
 	"github.com/emersion/go-maildir"
 
+	"git.sr.ht/~rjarry/aerc/lib/rfc822"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -74,25 +75,43 @@ func (m Message) Remove() error {
 
 // MessageInfo populates a models.MessageInfo struct for the message.
 func (m Message) MessageInfo() (*models.MessageInfo, error) {
-	info, err := lib.MessageInfo(m)
+	info, err := rfc822.MessageInfo(m)
 	if err != nil {
 		return nil, err
 	}
-	// if size retrieval fails, just return info and log error
-	if name, err := m.dir.Filename(m.key); err != nil {
-		log.Errorf("failed to obtain filename: %v", err)
-	} else {
-		if info.Size, err = lib.FileSize(name); err != nil {
-			log.Errorf("failed to obtain file size: %v", err)
-		}
+	info.Size, err = m.Size()
+	if err != nil {
+		// don't care if size retrieval fails
+		log.Debugf("message size: %v", err)
 	}
 	return info, nil
 }
 
+func (m Message) Size() (uint32, error) {
+	name, err := m.dir.Filename(m.key)
+	if err != nil {
+		return 0, fmt.Errorf("failed to get filename: %w", err)
+	}
+	size, err := lib.FileSize(name)
+	if err != nil {
+		return 0, fmt.Errorf("failed to get filesize: %w", err)
+	}
+	return size, nil
+}
+
 // MessageHeaders populates a models.MessageInfo struct for the message with
 // minimal information, used for sorting and threading.
 func (m Message) MessageHeaders() (*models.MessageInfo, error) {
-	return lib.MessageHeaders(m)
+	info, err := rfc822.MessageHeaders(m)
+	if err != nil {
+		return nil, err
+	}
+	info.Size, err = m.Size()
+	if err != nil {
+		// don't care if size retrieval fails
+		log.Debugf("message size failed: %v", err)
+	}
+	return info, nil
 }
 
 // NewBodyPartReader creates a new io.Reader for the requested body part(s) of
@@ -103,11 +122,11 @@ func (m Message) NewBodyPartReader(requestedParts []int) (io.Reader, error) {
 		return nil, err
 	}
 	defer f.Close()
-	msg, err := lib.ReadMessage(f)
+	msg, err := rfc822.ReadMessage(f)
 	if err != nil {
 		return nil, fmt.Errorf("could not read message: %w", err)
 	}
-	return lib.FetchEntityPartReader(msg, requestedParts)
+	return rfc822.FetchEntityPartReader(msg, requestedParts)
 }
 
 func (m Message) UID() uint32 {
