@@ -1,6 +1,7 @@
 package maildir
 
 import (
+	"bufio"
 	"bytes"
 	"context"
 	"errors"
@@ -20,10 +21,13 @@ import (
 
 	aercLib "git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/iterator"
+	"git.sr.ht/~rjarry/aerc/lib/watchers"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
+	"git.sr.ht/~rjarry/aerc/worker/middleware"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
@@ -40,18 +44,20 @@ type Worker struct {
 	selected            *maildir.Dir
 	selectedName        string
 	selectedInfo        *models.DirectoryInfo
-	worker              *types.Worker
-	watcher             types.FSWatcher
+	worker              types.WorkerInteractor
+	watcher             watchers.FSWatcher
 	watcherDebounce     *time.Timer
 	fsEvents            chan struct{}
 	currentSortCriteria []*types.SortCriterion
 	maildirpp           bool // whether to use Maildir++ directory layout
 	capabilities        *models.Capabilities
+	headers             []string
+	headersExclude      []string
 }
 
 // NewWorker creates a new maildir worker with the provided worker.
 func NewWorker(worker *types.Worker) (types.Backend, error) {
-	watch, err := handlers.NewWatcher()
+	watch, err := watchers.NewWatcher()
 	if err != nil {
 		return nil, fmt.Errorf("could not create file system watcher: %w", err)
 	}
@@ -68,7 +74,7 @@ func NewWorker(worker *types.Worker) (types.Backend, error) {
 
 // NewMaildirppWorker creates a new Maildir++ worker with the provided worker.
 func NewMaildirppWorker(worker *types.Worker) (types.Backend, error) {
-	watch, err := handlers.NewWatcher()
+	watch, err := watchers.NewWatcher()
 	if err != nil {
 		return nil, fmt.Errorf("could not create file system watcher: %w", err)
 	}
@@ -87,7 +93,7 @@ func NewMaildirppWorker(worker *types.Worker) (types.Backend, error) {
 func (w *Worker) Run() {
 	for {
 		select {
-		case action := <-w.worker.Actions:
+		case action := <-w.worker.Actions():
 			w.handleAction(action)
 		case <-w.watcher.Events():
 			if w.watcherDebounce != nil {
@@ -108,6 +114,10 @@ func (w *Worker) Capabilities() *models.Capabilities {
 	return w.capabilities
 }
 
+func (w *Worker) PathSeparator() string {
+	return string(os.PathSeparator)
+}
+
 func (w *Worker) handleAction(action types.WorkerMessage) {
 	msg := w.worker.ProcessAction(action)
 	switch msg := msg.(type) {
@@ -123,6 +133,10 @@ func (w *Worker) handleAction(action types.WorkerMessage) {
 			w.worker.PostMessage(&types.Unsupported{
 				Message: types.RespondTo(msg),
 			}, nil)
+		case errors.Is(err, context.Canceled):
+			w.worker.PostMessage(&types.Cancelled{
+				Message: types.RespondTo(msg),
+			}, nil)
 		case err != nil:
 			w.worker.PostMessage(&types.Error{
 				Message: types.RespondTo(msg),
@@ -141,7 +155,7 @@ func (w *Worker) handleFSEvent() {
 	}
 	err := w.c.SyncNewMail(*w.selected)
 	if err != nil {
-		log.Errorf("could not move new to cur : %v", err)
+		w.worker.Errorf("could not move new to cur : %v", err)
 		return
 	}
 
@@ -212,18 +226,18 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 		for _, v := range files {
 			key, flags, err := splitMaildirFile(v)
 			if err != nil {
-				log.Errorf("%q: error parsing flags (%q): %v", v, key, err)
+				w.worker.Errorf("%q: error parsing flags (%q): %v", v, key, err)
 				continue
 			}
 			keyFlags[key] = flags
 		}
 	} else {
-		log.Tracef("disabled flags cache: %q: %v", dir, err)
+		w.worker.Tracef("disabled flags cache: %q: %v", dir, err)
 	}
 
 	uids, err := w.c.UIDs(dir)
-	if err != nil {
-		log.Errorf("could not get uids: %v", err)
+	if err != nil && len(uids) == 0 {
+		w.worker.Errorf("could not get uids: %v", err)
 		return dirInfo
 	}
 
@@ -231,7 +245,7 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 	for _, uid := range uids {
 		message, err := w.c.Message(dir, uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.worker.Errorf("could not get message: %v", err)
 			continue
 		}
 		var flags []maildir.Flag
@@ -239,18 +253,18 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 			ok := false
 			flags, ok = keyFlags[message.key]
 			if !ok {
-				log.Tracef("message (key=%q uid=%d) not found in map cache",
+				w.worker.Tracef("message (key=%q uid=%d) not found in map cache",
 					message.key, message.uid)
 				flags, err = message.Flags()
 				if err != nil {
-					log.Errorf("could not get flags: %v", err)
+					w.worker.Errorf("could not get flags: %v", err)
 					continue
 				}
 			}
 		} else {
 			flags, err = message.Flags()
 			if err != nil {
-				log.Errorf("could not get flags: %v", err)
+				w.worker.Errorf("could not get flags: %v", err)
 				continue
 			}
 		}
@@ -318,7 +332,7 @@ func (w *Worker) handleMessage(msg types.WorkerMessage) error {
 func (w *Worker) handleConfigure(msg *types.Configure) error {
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
-		log.Errorf("error configuring maildir worker: %v", err)
+		w.worker.Errorf("error configuring maildir worker: %v", err)
 		return err
 	}
 	dir := u.Path
@@ -334,7 +348,7 @@ func (w *Worker) handleConfigure(msg *types.Configure) error {
 	}
 	c, err := NewContainer(dir, w.maildirpp)
 	if err != nil {
-		log.Errorf("could not configure maildir: %s", dir)
+		w.worker.Errorf("could not configure maildir: %s", dir)
 		return err
 	}
 	w.c = c
@@ -342,7 +356,24 @@ func (w *Worker) handleConfigure(msg *types.Configure) error {
 	if err != nil {
 		return err
 	}
-	log.Debugf("configured base maildir: %s", dir)
+	w.headers = msg.Config.Headers
+	w.headersExclude = msg.Config.HeadersExclude
+	w.worker.Debugf("configured base maildir: %s", dir)
+
+	if name, ok := msg.Config.Params["folder-map"]; ok {
+		file := xdg.ExpandHome(name)
+		f, err := os.Open(file)
+		if err != nil {
+			return err
+		}
+		defer f.Close()
+		fmap, order, err := lib.ParseFolderMap(bufio.NewReader(f))
+		if err != nil {
+			return err
+		}
+		w.worker = middleware.NewFolderMapper(w.worker, fmap, order)
+	}
+
 	return nil
 }
 
@@ -359,7 +390,7 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 	}
 	dirs, err := w.c.Store.FolderMap()
 	if err != nil {
-		log.Errorf("failed listing directories: %v", err)
+		w.worker.Errorf("failed listing directories: %v", err)
 		return err
 	}
 	for name := range dirs {
@@ -378,7 +409,7 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 }
 
 func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
-	log.Debugf("opening %s", msg.Directory)
+	w.worker.Debugf("opening %s", msg.Directory)
 
 	// open the directory
 	dir, err := w.c.OpenDirectory(msg.Directory)
@@ -430,26 +461,27 @@ func (w *Worker) handleFetchDirectoryContents(
 		uids []uint32
 		err  error
 	)
-	// FilterCriteria always contains "filter" as first item
-	if len(msg.FilterCriteria) > 1 {
-		filter, err := parseSearch(msg.FilterCriteria)
-		if err != nil {
-			return err
-		}
-		uids, err = w.search(filter)
+	if msg.Filter != nil {
+		uids, err = w.search(msg.Context, msg.Filter)
 		if err != nil {
 			return err
 		}
 	} else {
 		uids, err = w.c.UIDs(*w.selected)
-		if err != nil {
-			log.Errorf("failed scanning uids: %v", err)
+		if err != nil && len(uids) == 0 {
+			w.worker.Errorf("failed scanning uids: %v", err)
 			return err
 		}
+
+		if err != nil {
+			w.worker.PostMessage(&types.Error{
+				Error: fmt.Errorf("could not get all uids for %s: %w", *w.selected, err),
+			}, nil)
+		}
 	}
-	sortedUids, err := w.sort(uids, msg.SortCriteria)
+	sortedUids, err := w.sort(msg.Context, uids, msg.SortCriteria)
 	if err != nil {
-		log.Errorf("failed sorting directory: %v", err)
+		w.worker.Errorf("failed sorting directory: %v", err)
 		return err
 	}
 	w.currentSortCriteria = msg.SortCriteria
@@ -460,7 +492,7 @@ func (w *Worker) handleFetchDirectoryContents(
 	return nil
 }
 
-func (w *Worker) sort(uids []uint32, criteria []*types.SortCriterion) ([]uint32, error) {
+func (w *Worker) sort(ctx context.Context, uids []uint32, criteria []*types.SortCriterion) ([]uint32, error) {
 	if len(criteria) == 0 {
 		// At least sort by uid, parallel searching can create random
 		// order
@@ -476,28 +508,33 @@ func (w *Worker) sort(uids []uint32, criteria []*types.SortCriterion) ([]uint32,
 	max := runtime.NumCPU() * 2
 	limit := make(chan struct{}, max)
 	for _, uid := range uids {
-		limit <- struct{}{}
-		wg.Add(1)
-		go func(uid uint32) {
-			defer log.PanicHandler()
-			defer wg.Done()
-			info, err := w.msgHeadersFromUid(uid)
-			if err != nil {
-				log.Errorf("could not get message info: %v", err)
+		select {
+		case <-ctx.Done():
+			return nil, context.Canceled
+		default:
+			limit <- struct{}{}
+			wg.Add(1)
+			go func(uid uint32) {
+				defer log.PanicHandler()
+				defer wg.Done()
+				info, err := w.msgHeadersFromUid(uid)
+				if err != nil {
+					w.worker.Errorf("could not get message info: %v", err)
+					<-limit
+					return
+				}
+				mu.Lock()
+				msgInfos = append(msgInfos, info)
+				mu.Unlock()
 				<-limit
-				return
-			}
-			mu.Lock()
-			msgInfos = append(msgInfos, info)
-			mu.Unlock()
-			<-limit
-		}(uid)
+			}(uid)
+		}
 	}
 
 	wg.Wait()
 	sortedUids, err := lib.Sort(msgInfos, criteria)
 	if err != nil {
-		log.Errorf("could not sort the messages: %v", err)
+		w.worker.Errorf("could not sort the messages: %v", err)
 		return nil, err
 	}
 	return sortedUids, nil
@@ -510,25 +547,21 @@ func (w *Worker) handleFetchDirectoryThreaded(
 		uids []uint32
 		err  error
 	)
-	if len(msg.FilterCriteria) > 1 {
-		filter, err := parseSearch(msg.FilterCriteria)
-		if err != nil {
-			return err
-		}
-		uids, err = w.search(filter)
+	if msg.Filter != nil {
+		uids, err = w.search(msg.Context, msg.Filter)
 		if err != nil {
 			return err
 		}
 	} else {
 		uids, err = w.c.UIDs(*w.selected)
 		if err != nil {
-			log.Errorf("failed scanning uids: %v", err)
+			w.worker.Errorf("failed scanning uids: %v", err)
 			return err
 		}
 	}
-	threads, err := w.threads(uids, msg.SortCriteria)
+	threads, err := w.threads(msg.Context, uids, msg.SortCriteria)
 	if err != nil {
-		log.Errorf("failed sorting directory: %v", err)
+		w.worker.Errorf("failed sorting directory: %v", err)
 		return err
 	}
 	w.currentSortCriteria = msg.SortCriteria
@@ -539,7 +572,9 @@ func (w *Worker) handleFetchDirectoryThreaded(
 	return nil
 }
 
-func (w *Worker) threads(uids []uint32, criteria []*types.SortCriterion) ([]*types.Thread, error) {
+func (w *Worker) threads(ctx context.Context, uids []uint32,
+	criteria []*types.SortCriterion,
+) ([]*types.Thread, error) {
 	builder := aercLib.NewThreadBuilder(iterator.NewFactory(false))
 	msgInfos := make([]*models.MessageInfo, 0, len(uids))
 	mu := sync.Mutex{}
@@ -547,23 +582,28 @@ func (w *Worker) threads(uids []uint32, criteria []*types.SortCriterion) ([]*typ
 	max := runtime.NumCPU() * 2
 	limit := make(chan struct{}, max)
 	for _, uid := range uids {
-		limit <- struct{}{}
-		wg.Add(1)
-		go func(uid uint32) {
-			defer log.PanicHandler()
-			defer wg.Done()
-			info, err := w.msgHeadersFromUid(uid)
-			if err != nil {
-				log.Errorf("could not get message info: %v", err)
+		select {
+		case <-ctx.Done():
+			return nil, context.Canceled
+		default:
+			limit <- struct{}{}
+			wg.Add(1)
+			go func(uid uint32) {
+				defer log.PanicHandler()
+				defer wg.Done()
+				info, err := w.msgHeadersFromUid(uid)
+				if err != nil {
+					w.worker.Errorf("could not get message info: %v", err)
+					<-limit
+					return
+				}
+				mu.Lock()
+				builder.Update(info)
+				msgInfos = append(msgInfos, info)
+				mu.Unlock()
 				<-limit
-				return
-			}
-			mu.Lock()
-			builder.Update(info)
-			msgInfos = append(msgInfos, info)
-			mu.Unlock()
-			<-limit
-		}(uid)
+			}(uid)
+		}
 	}
 	wg.Wait()
 	var err error
@@ -575,7 +615,7 @@ func (w *Worker) threads(uids []uint32, criteria []*types.SortCriterion) ([]*typ
 	default:
 		uids, err = lib.Sort(msgInfos, criteria)
 		if err != nil {
-			log.Errorf("could not sort the messages: %v", err)
+			w.worker.Errorf("could not sort the messages: %v", err)
 			return nil, err
 		}
 	}
@@ -586,7 +626,7 @@ func (w *Worker) threads(uids []uint32, criteria []*types.SortCriterion) ([]*typ
 func (w *Worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 	dir := w.c.Store.Dir(msg.Directory)
 	if err := dir.Init(); err != nil {
-		log.Errorf("could not create directory %s: %v",
+		w.worker.Errorf("could not create directory %s: %v",
 			msg.Directory, err)
 		return err
 	}
@@ -596,7 +636,7 @@ func (w *Worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 func (w *Worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
 	dir := w.c.Store.Dir(msg.Directory)
 	if err := os.RemoveAll(string(dir)); err != nil {
-		log.Errorf("could not remove directory %s: %v",
+		w.worker.Errorf("could not remove directory %s: %v",
 			msg.Directory, err)
 		return err
 	}
@@ -609,10 +649,25 @@ func (w *Worker) handleFetchMessageHeaders(
 	for _, uid := range msg.Uids {
 		info, err := w.msgInfoFromUid(uid)
 		if err != nil {
+			w.worker.Errorf("could not get message info: %v", err)
 			log.Errorf("could not get message info: %v", err)
-			w.worker.PostMessageInfoError(msg, uid, err)
+			w.worker.PostMessage(&types.MessageInfo{
+				Info: &models.MessageInfo{
+					Envelope: &models.Envelope{},
+					Flags:    models.SeenFlag,
+					Uid:      uid,
+					Error:    err,
+				},
+				Message: types.RespondTo(msg),
+			}, nil)
 			continue
 		}
+		switch {
+		case len(w.headersExclude) > 0:
+			info.RFC822Headers = lib.LimitHeaders(info.RFC822Headers, w.headersExclude, true)
+		case len(w.headers) > 0:
+			info.RFC822Headers = lib.LimitHeaders(info.RFC822Headers, w.headers, false)
+		}
 		w.worker.PostMessage(&types.MessageInfo{
 			Message: types.RespondTo(msg),
 			Info:    info,
@@ -628,12 +683,12 @@ func (w *Worker) handleFetchMessageBodyPart(
 	// get reader
 	m, err := w.c.Message(*w.selected, msg.Uid)
 	if err != nil {
-		log.Errorf("could not get message %d: %v", msg.Uid, err)
+		w.worker.Errorf("could not get message %d: %v", msg.Uid, err)
 		return err
 	}
 	r, err := m.NewBodyPartReader(msg.Part)
 	if err != nil {
-		log.Errorf(
+		w.worker.Errorf(
 			"could not get body part reader for message=%d, parts=%#v: %w",
 			msg.Uid, msg.Part, err)
 		return err
@@ -653,12 +708,12 @@ func (w *Worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			log.Errorf("could not get message %d: %v", uid, err)
+			w.worker.Errorf("could not get message %d: %v", uid, err)
 			return err
 		}
 		r, err := m.NewReader()
 		if err != nil {
-			log.Errorf("could not get message reader: %v", err)
+			w.worker.Errorf("could not get message reader: %v", err)
 			return err
 		}
 		defer r.Close()
@@ -689,7 +744,7 @@ func (w *Worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 		}, nil)
 	}
 	if err != nil {
-		log.Errorf("failed removing messages: %v", err)
+		w.worker.Errorf("failed removing messages: %v", err)
 		return err
 	}
 	return nil
@@ -699,18 +754,18 @@ func (w *Worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.worker.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.MarkReplied(msg.Answered); err != nil {
-			log.Errorf("could not mark message as answered: %v", err)
+			w.worker.Errorf("could not mark message as answered: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			log.Errorf("could not get message info: %v", err)
+			w.worker.Errorf("could not get message info: %v", err)
 			w.err(msg, err)
 			continue
 		}
@@ -731,19 +786,19 @@ func (w *Worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.worker.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		flag := lib.FlagToMaildir[msg.Flags]
 		if err := m.SetOneFlag(flag, msg.Enable); err != nil {
-			log.Errorf("could change flag %v to %v on message: %v", flag, msg.Enable, err)
+			w.worker.Errorf("could change flag %v to %v on message: %v", flag, msg.Enable, err)
 			w.err(msg, err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			log.Errorf("could not get message info: %v", err)
+			w.worker.Errorf("could not get message info: %v", err)
 			w.err(msg, err)
 			continue
 		}
@@ -813,13 +868,8 @@ func (w *Worker) handleAppendMessage(msg *types.AppendMessage) error {
 }
 
 func (w *Worker) handleSearchDirectory(msg *types.SearchDirectory) error {
-	log.Debugf("Searching directory %v with args: %v", *w.selected, msg.Argv)
-	criteria, err := parseSearch(msg.Argv)
-	if err != nil {
-		return err
-	}
-	log.Tracef("Searching with parsed criteria: %#v", criteria)
-	uids, err := w.search(criteria)
+	w.worker.Tracef("Searching with criteria: %#v", msg.Criteria)
+	uids, err := w.search(msg.Context, msg.Criteria)
 	if err != nil {
 		return err
 	}
@@ -839,6 +889,11 @@ func (w *Worker) msgInfoFromUid(uid uint32) (*models.MessageInfo, error) {
 	if err != nil {
 		return nil, err
 	}
+	name, err := m.dir.Filename(m.key)
+	if err != nil {
+		return nil, err
+	}
+	info.Filenames = []string{name}
 	if w.c.IsRecent(uid) {
 		info.Flags |= models.RecentFlag
 	}
@@ -869,7 +924,14 @@ func (w *Worker) handleCheckMail(msg *types.CheckMail) {
 	ch := make(chan error)
 	go func() {
 		defer log.PanicHandler()
-		err := cmd.Run()
+
+		_, err := cmd.Output()
+		if err != nil {
+			var exitError *exec.ExitError
+			if errors.As(err, &exitError) {
+				err = fmt.Errorf("%w\n%s", err, string(exitError.Stderr))
+			}
+		}
 		ch <- err
 	}()
 	select {
