@@ -10,12 +10,12 @@ import (
 	"path/filepath"
 	"sort"
 
+	"git.sr.ht/~rjarry/aerc/lib/rfc822"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
 	"git.sr.ht/~rjarry/aerc/worker/types"
-	"github.com/miolini/datacounter"
 )
 
 func init() {
@@ -30,7 +30,9 @@ type mboxWorker struct {
 	folder *container
 	worker *types.Worker
 
-	capabilities *models.Capabilities
+	capabilities   *models.Capabilities
+	headers        []string
+	headersExclude []string
 }
 
 func NewWorker(worker *types.Worker) (types.Backend, error) {
@@ -68,6 +70,8 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 		} else {
 			dir = filepath.Join(u.Host, u.Path)
 		}
+		w.headers = msg.Config.Headers
+		w.headersExclude = msg.Config.HeadersExclude
 		w.data, err = createMailboxContainer(dir)
 		if err != nil || w.data == nil {
 			w.data = &mailboxContainer{
@@ -76,7 +80,7 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 			reterr = err
 			break
 		} else {
-			log.Debugf("configured with mbox file %s", dir)
+			w.worker.Debugf("configured with mbox file %s", dir)
 		}
 
 	case *types.Connect, *types.Reconnect, *types.Disconnect:
@@ -112,10 +116,10 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 			Info: w.data.DirectoryInfo(msg.Directory),
 		}, nil)
 		w.worker.PostMessage(&types.Done{Message: types.RespondTo(msg)}, nil)
-		log.Debugf("%s opened", msg.Directory)
+		w.worker.Debugf("%s opened", msg.Directory)
 
 	case *types.FetchDirectoryContents:
-		uids, err := filterUids(w.folder, w.folder.Uids(), msg.FilterCriteria)
+		uids, err := filterUids(w.folder, w.folder.Uids(), msg.Filter)
 		if err != nil {
 			reterr = err
 			break
@@ -158,9 +162,23 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 			}
 			msgInfo, err := messageInfo(m, true)
 			if err != nil {
-				w.worker.PostMessageInfoError(msg, uid, err)
-				break
+				w.worker.PostMessage(&types.MessageInfo{
+					Info: &models.MessageInfo{
+						Envelope: &models.Envelope{},
+						Flags:    models.SeenFlag,
+						Uid:      uid,
+						Error:    err,
+					},
+					Message: types.RespondTo(msg),
+				}, nil)
+				continue
 			} else {
+				switch {
+				case len(w.headersExclude) > 0:
+					msgInfo.RFC822Headers = lib.LimitHeaders(msgInfo.RFC822Headers, w.headersExclude, true)
+				case len(w.headers) > 0:
+					msgInfo.RFC822Headers = lib.LimitHeaders(msgInfo.RFC822Headers, w.headers, false)
+				}
 				w.worker.PostMessage(&types.MessageInfo{
 					Message: types.RespondTo(msg),
 					Info:    msgInfo,
@@ -173,7 +191,7 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 	case *types.FetchMessageBodyPart:
 		m, err := w.folder.Message(msg.Uid)
 		if err != nil {
-			log.Errorf("could not get message %d: %v", msg.Uid, err)
+			w.worker.Errorf("could not get message %d: %v", msg.Uid, err)
 			reterr = err
 			break
 		}
@@ -184,15 +202,15 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 			break
 		}
 
-		fullMsg, err := lib.ReadMessage(contentReader)
+		fullMsg, err := rfc822.ReadMessage(contentReader)
 		if err != nil {
 			reterr = fmt.Errorf("could not read message: %w", err)
 			break
 		}
 
-		r, err := lib.FetchEntityPartReader(fullMsg, msg.Part)
+		r, err := rfc822.FetchEntityPartReader(fullMsg, msg.Part)
 		if err != nil {
-			log.Errorf(
+			w.worker.Errorf(
 				"could not get body part reader for message=%d, parts=%#v: %w",
 				msg.Uid, msg.Part, err)
 			reterr = err
@@ -211,18 +229,18 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 		for _, uid := range msg.Uids {
 			m, err := w.folder.Message(uid)
 			if err != nil {
-				log.Errorf("could not get message for uid %d: %v", uid, err)
+				w.worker.Errorf("could not get message for uid %d: %v", uid, err)
 				continue
 			}
 			r, err := m.NewReader()
 			if err != nil {
-				log.Errorf("could not get message reader: %v", err)
+				w.worker.Errorf("could not get message reader: %v", err)
 				continue
 			}
 			defer r.Close()
 			b, err := io.ReadAll(r)
 			if err != nil {
-				log.Errorf("could not get message reader: %v", err)
+				w.worker.Errorf("could not get message reader: %v", err)
 				continue
 			}
 			w.worker.PostMessage(&types.FullMessage{
@@ -257,17 +275,17 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 		for _, uid := range msg.Uids {
 			m, err := w.folder.Message(uid)
 			if err != nil {
-				log.Errorf("could not get message: %v", err)
+				w.worker.Errorf("could not get message: %v", err)
 				continue
 			}
 			if err := m.(*message).SetFlag(msg.Flags, msg.Enable); err != nil {
-				log.Errorf("could not change flag %v to %t on message: %v",
+				w.worker.Errorf("could not change flag %v to %t on message: %v",
 					msg.Flags, msg.Enable, err)
 				continue
 			}
-			info, err := lib.MessageInfo(m)
+			info, err := rfc822.MessageInfo(m)
 			if err != nil {
-				log.Errorf("could not get message info: %v", err)
+				w.worker.Errorf("could not get message info: %v", err)
 				continue
 			}
 
@@ -321,7 +339,7 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 			&types.Done{Message: types.RespondTo(msg)}, nil)
 
 	case *types.SearchDirectory:
-		uids, err := filterUids(w.folder, w.folder.Uids(), msg.Argv)
+		uids, err := filterUids(w.folder, w.folder.Uids(), msg.Criteria)
 		if err != nil {
 			reterr = err
 			break
@@ -364,7 +382,7 @@ func (w *mboxWorker) handleMessage(msg types.WorkerMessage) error {
 }
 
 func (w *mboxWorker) Run() {
-	for msg := range w.worker.Actions {
+	for msg := range w.worker.Actions() {
 		msg = w.worker.ProcessAction(msg)
 		if err := w.handleMessage(msg); errors.Is(err, errUnsupported) {
 			w.worker.PostMessage(&types.Unsupported{
@@ -383,13 +401,13 @@ func (w *mboxWorker) Capabilities() *models.Capabilities {
 	return w.capabilities
 }
 
-func filterUids(folder *container, uids []uint32, args []string) ([]uint32, error) {
-	criteria, err := lib.GetSearchCriteria(args)
-	if err != nil {
-		return nil, err
-	}
+func (w *mboxWorker) PathSeparator() string {
+	return "/"
+}
+
+func filterUids(folder *container, uids []uint32, criteria *types.SearchCriteria) ([]uint32, error) {
 	log.Debugf("Search with parsed criteria: %#v", criteria)
-	m := make([]lib.RawMessage, 0, len(uids))
+	m := make([]rfc822.RawMessage, 0, len(uids))
 	for _, uid := range uids {
 		msg, err := folder.Message(uid)
 		if err != nil {
@@ -427,19 +445,22 @@ func sortUids(folder *container, uids []uint32,
 	return lib.Sort(infos, criteria)
 }
 
-func messageInfo(m lib.RawMessage, needSize bool) (*models.MessageInfo, error) {
-	info, err := lib.MessageInfo(m)
+func messageInfo(m rfc822.RawMessage, needSize bool) (*models.MessageInfo, error) {
+	info, err := rfc822.MessageInfo(m)
 	if err != nil {
 		return nil, err
 	}
-	if needSize {
-		if r, err := m.NewReader(); err == nil {
-			var buf bytes.Buffer
-			ctr := datacounter.NewWriterCounter(&buf)
-			if _, err := io.Copy(ctr, r); err == nil {
-				info.Size = uint32(ctr.Count())
-			}
-		}
+	if !needSize {
+		return info, nil
 	}
+	r, err := m.NewReader()
+	if err != nil {
+		return nil, err
+	}
+	size, err := io.Copy(io.Discard, r)
+	if err != nil {
+		return nil, err
+	}
+	info.Size = uint32(size)
 	return info, nil
 }
