@@ -12,6 +12,7 @@ import (
 
 	"github.com/emersion/go-maildir"
 
+	"git.sr.ht/~rjarry/aerc/lib/rfc822"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -35,7 +36,7 @@ func (m *Message) NewReader() (io.ReadCloser, error) {
 
 // MessageInfo populates a models.MessageInfo struct for the message.
 func (m *Message) MessageInfo() (*models.MessageInfo, error) {
-	info, err := lib.MessageInfo(m)
+	info, err := rfc822.MessageInfo(m)
 	if err != nil {
 		return nil, err
 	}
@@ -47,6 +48,10 @@ func (m *Message) MessageInfo() (*models.MessageInfo, error) {
 			log.Errorf("failed to obtain file size: %v", err)
 		}
 	}
+	filenames, err := m.db.MsgFilenames(m.key)
+	if err == nil {
+		info.Filenames = filenames
+	}
 	return info, nil
 }
 
@@ -62,32 +67,24 @@ func (m *Message) NewBodyPartReader(requestedParts []int) (io.Reader, error) {
 		return nil, err
 	}
 	defer f.Close()
-	msg, err := lib.ReadMessage(f)
+	msg, err := rfc822.ReadMessage(f)
 	if err != nil {
 		return nil, fmt.Errorf("could not read message: %w", err)
 	}
-	return lib.FetchEntityPartReader(msg, requestedParts)
+	return rfc822.FetchEntityPartReader(msg, requestedParts)
 }
 
 // SetFlag adds or removes a flag from the message.
 // Notmuch doesn't support all the flags, and for those this errors.
 func (m *Message) SetFlag(flag models.Flags, enable bool) error {
 	// Translate the flag into a notmuch tag, ignoring no-op flags.
-	var tag string
-	switch flag {
-	case models.SeenFlag:
-		// Note: Inverted properly later
-		tag = "unread"
-	case models.AnsweredFlag:
-		tag = "replied"
-	case models.FlaggedFlag:
-		tag = "flagged"
-	default:
+	tag, ok := flagToTag[flag]
+	if !ok {
 		return fmt.Errorf("Notmuch doesn't support flag %v", flag)
 	}
 
 	// Get the current state of the flag.
-	// Note that notmuch handles models.SeenFlag in an inverted sense.
+	// Note that notmuch handles some flags in an inverted sense
 	oldState := false
 	tags, err := m.Tags()
 	if err != nil {
@@ -99,27 +96,18 @@ func (m *Message) SetFlag(flag models.Flags, enable bool) error {
 			break
 		}
 	}
-	if flag == models.SeenFlag {
-		oldState = !oldState
+
+	if flagToInvert[flag] {
+		enable = !enable
 	}
 
-	// Skip if flag already in correct state.
-	if oldState == enable {
+	switch {
+	case oldState == enable:
 		return nil
-	}
-
-	if !enable {
-		if flag == models.SeenFlag {
-			return m.AddTag("unread")
-		} else {
-			return m.RemoveTag(tag)
-		}
-	} else {
-		if flag == models.SeenFlag {
-			return m.RemoveTag("unread")
-		} else {
-			return m.AddTag(tag)
-		}
+	case enable:
+		return m.AddTag(tag)
+	default:
+		return m.RemoveTag(tag)
 	}
 }
 
@@ -149,13 +137,11 @@ func (m *Message) ModelFlags() (models.Flags, error) {
 		return 0, err
 	}
 	for _, tag := range tags {
-		switch tag {
-		case "replied":
-			flags |= models.AnsweredFlag
-		case "flagged":
-			flags |= models.FlaggedFlag
-		case "unread":
-			flags &^= models.SeenFlag
+		flag := tagToFlag[tag]
+		if flagToInvert[flag] {
+			flags &^= flag
+		} else {
+			flags |= flag
 		}
 	}
 	return flags, nil
