@@ -4,7 +4,6 @@
 package notmuch
 
 import (
-	"bufio"
 	"bytes"
 	"context"
 	"errors"
@@ -13,19 +12,21 @@ import (
 	"net/url"
 	"os"
 	"os/exec"
+	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib/watchers"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
 	notmuch "git.sr.ht/~rjarry/aerc/worker/notmuch/lib"
 	"git.sr.ht/~rjarry/aerc/worker/types"
-	"github.com/mitchellh/go-homedir"
 )
 
 func init() {
@@ -34,27 +35,41 @@ func init() {
 
 var errUnsupported = fmt.Errorf("unsupported command")
 
-const backgroundRefreshDelay = 1 * time.Minute
-
 type worker struct {
 	w                   *types.Worker
-	nmEvents            chan eventType
+	nmStateChange       chan bool
 	query               string
 	currentQueryName    string
 	queryMapOrder       []string
 	nameQueryMap        map[string]string
 	store               *lib.MaildirStore
+	maildirAccountPath  string
 	db                  *notmuch.DB
 	setupErr            error
 	currentSortCriteria []*types.SortCriterion
+	watcher             watchers.FSWatcher
+	watcherDebounce     *time.Timer
+	capabilities        *models.Capabilities
+	headers             []string
+	headersExclude      []string
+	state               uint64
 }
 
 // NewWorker creates a new notmuch worker with the provided worker.
 func NewWorker(w *types.Worker) (types.Backend, error) {
-	events := make(chan eventType, 20)
+	events := make(chan bool, 20)
+	watcher, err := watchers.NewWatcher()
+	if err != nil {
+		return nil, fmt.Errorf("could not create file system watcher: %w", err)
+	}
 	return &worker{
-		w:        w,
-		nmEvents: events,
+		w:             w,
+		nmStateChange: events,
+		watcher:       watcher,
+		capabilities: &models.Capabilities{
+			Sort:   true,
+			Thread: true,
+		},
 	}, nil
 }
 
@@ -62,29 +77,55 @@ func NewWorker(w *types.Worker) (types.Backend, error) {
 func (w *worker) Run() {
 	for {
 		select {
-		case action := <-w.w.Actions:
+		case action := <-w.w.Actions():
 			msg := w.w.ProcessAction(action)
-			if err := w.handleMessage(msg); errors.Is(err, errUnsupported) {
+			err := w.handleMessage(msg)
+			switch {
+			case errors.Is(err, errUnsupported):
 				w.w.PostMessage(&types.Unsupported{
 					Message: types.RespondTo(msg),
 				}, nil)
-				log.Errorf("ProcessAction(%T) unsupported: %v", msg, err)
-			} else if err != nil {
+				w.w.Errorf("ProcessAction(%T) unsupported: %v", msg, err)
+			case errors.Is(err, context.Canceled):
+				w.w.PostMessage(&types.Cancelled{
+					Message: types.RespondTo(msg),
+				}, nil)
+			case err != nil:
 				w.w.PostMessage(&types.Error{
 					Message: types.RespondTo(msg),
 					Error:   err,
 				}, nil)
-				log.Errorf("ProcessAction(%T) failure: %v", msg, err)
+				w.w.Errorf("ProcessAction(%T) failure: %v", msg, err)
 			}
-		case nmEvent := <-w.nmEvents:
-			err := w.handleNotmuchEvent(nmEvent)
+		case <-w.nmStateChange:
+			err := w.handleNotmuchEvent()
 			if err != nil {
-				log.Errorf("notmuch event failure: %v", err)
+				w.w.Errorf("notmuch event failure: %v", err)
 			}
+		case <-w.watcher.Events():
+			if w.watcherDebounce != nil {
+				w.watcherDebounce.Stop()
+			}
+			// Debounce FS changes
+			w.watcherDebounce = time.AfterFunc(50*time.Millisecond, func() {
+				defer log.PanicHandler()
+				w.nmStateChange <- true
+			})
 		}
 	}
 }
 
+func (w *worker) Capabilities() *models.Capabilities {
+	return w.capabilities
+}
+
+func (w *worker) PathSeparator() string {
+	// make it configurable?
+	// <rockorager> You can use those in query maps to force a tree
+	// <rockorager> Might be nice to be configurable? I see some notmuch people namespace with "::"
+	return "/"
+}
+
 func (w *worker) done(msg types.WorkerMessage) {
 	w.w.PostMessage(&types.Done{Message: types.RespondTo(msg)}, nil)
 }
@@ -104,6 +145,13 @@ func (w *worker) handleMessage(msg types.WorkerMessage) error {
 			return w.setupErr
 		}
 	}
+	if w.db != nil {
+		err := w.db.Connect()
+		if err != nil {
+			return err
+		}
+		defer w.db.Close()
+	}
 
 	switch msg := msg.(type) {
 	case *types.Unsupported:
@@ -165,13 +213,10 @@ func (w *worker) handleConfigure(msg *types.Configure) error {
 
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
-		log.Errorf("error configuring notmuch worker: %v", err)
+		w.w.Errorf("error configuring notmuch worker: %v", err)
 		return err
 	}
-	home, err := homedir.Expand(u.Hostname())
-	if err != nil {
-		return fmt.Errorf("could not resolve home directory: %w", err)
-	}
+	home := xdg.ExpandHome(u.Hostname())
 	pathToDB := filepath.Join(home, u.Path)
 	err = w.loadQueryMap(msg.Config)
 	if err != nil {
@@ -182,35 +227,34 @@ func (w *worker) handleConfigure(msg *types.Configure) error {
 
 	val, ok := msg.Config.Params["maildir-store"]
 	if ok {
-		path, err := homedir.Expand(val)
-		if err != nil {
-			return err
-		}
+		path := xdg.ExpandHome(val)
+		w.maildirAccountPath = msg.Config.Params["maildir-account-path"]
+
+		path = filepath.Join(path, w.maildirAccountPath)
 		store, err := lib.NewMaildirStore(path, false)
 		if err != nil {
 			return fmt.Errorf("Cannot initialize maildir store: %w", err)
 		}
 		w.store = store
 	}
+	w.headers = msg.Config.Headers
+	w.headersExclude = msg.Config.HeadersExclude
 
 	return nil
 }
 
 func (w *worker) handleConnect(msg *types.Connect) error {
-	err := w.db.Connect()
+	w.done(msg)
+	w.emitLabelList()
+	// Get initial db state
+	w.state = w.db.State()
+	// Watch all the files in the xapian folder for changes. We'll debounce
+	// changes, so catching multiple is ok
+	dbPath := path.Join(w.db.Path(), ".notmuch", "xapian")
+	err := w.watcher.Configure(dbPath)
 	if err != nil {
 		return err
 	}
-	w.done(msg)
-	w.emitLabelList()
-	go func() {
-		defer log.PanicHandler()
-
-		for {
-			w.nmEvents <- &updateDirCounts{}
-			time.Sleep(backgroundRefreshDelay)
-		}
-	}()
 	return nil
 }
 
@@ -218,15 +262,14 @@ func (w *worker) handleListDirectories(msg *types.ListDirectories) error {
 	if w.store != nil {
 		folders, err := w.store.FolderMap()
 		if err != nil {
-			log.Errorf("failed listing directories: %v", err)
+			w.w.Errorf("failed listing directories: %v", err)
 			return err
 		}
 		for name := range folders {
 			w.w.PostMessage(&types.Directory{
 				Message: types.RespondTo(msg),
 				Dir: &models.Directory{
-					Name:       name,
-					Attributes: []string{},
+					Name: name,
 				},
 			}, nil)
 		}
@@ -236,32 +279,29 @@ func (w *worker) handleListDirectories(msg *types.ListDirectories) error {
 		w.w.PostMessage(&types.Directory{
 			Message: types.RespondTo(msg),
 			Dir: &models.Directory{
-				Name:       name,
-				Attributes: []string{},
+				Name: name,
+				Role: models.QueryRole,
 			},
 		}, nil)
 	}
+	// Update dir counts when listing directories
+	err := w.updateDirCounts()
+	if err != nil {
+		return err
+	}
 	w.done(msg)
 	return nil
 }
 
 func (w *worker) getDirectoryInfo(name string, query string) *models.DirectoryInfo {
 	dirInfo := &models.DirectoryInfo{
-		Name:     name,
-		Flags:    []string{},
-		ReadOnly: false,
+		Name: name,
 		// total messages
 		Exists: 0,
 		// new messages since mailbox was last opened
 		Recent: 0,
 		// total unread
-		Unseen:         0,
-		AccurateCounts: true,
-
-		Caps: &models.Capabilities{
-			Sort:   true,
-			Thread: true,
-		},
+		Unseen: 0,
 	}
 
 	count, err := w.db.QueryCountMessages(query)
@@ -275,7 +315,10 @@ func (w *worker) getDirectoryInfo(name string, query string) *models.DirectoryIn
 }
 
 func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
-	log.Tracef("opening %s", msg.Directory)
+	if msg.Context.Err() != nil {
+		return context.Canceled
+	}
+	w.w.Tracef("opening %s", msg.Directory)
 
 	var isDynamicFolder bool
 	q := ""
@@ -283,7 +326,8 @@ func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 		folders, _ := w.store.FolderMap()
 		dir, ok := folders[msg.Directory]
 		if ok {
-			q = fmt.Sprintf("folder:%s", strconv.Quote(msg.Directory))
+			folder := filepath.Join(w.maildirAccountPath, msg.Directory)
+			q = fmt.Sprintf("folder:%s", strconv.Quote(folder))
 			if err := w.processNewMaildirFiles(string(dir)); err != nil {
 				return err
 			}
@@ -295,6 +339,13 @@ func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 		if !ok {
 			q = msg.Directory
 			isDynamicFolder = true
+			w.w.PostMessage(&types.Directory{
+				Message: types.RespondTo(msg),
+				Dir: &models.Directory{
+					Name: q,
+					Role: models.QueryRole,
+				},
+			}, nil)
 		}
 	}
 	w.query = q
@@ -344,14 +395,14 @@ func (w *worker) handleFetchMessageHeaders(
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
-			w.w.PostMessageInfoError(msg, uid, err)
+			w.w.Errorf("could not get message: %v", err)
+			w.emitMessageInfoError(msg, uid, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			log.Errorf("could not emit message info: %v", err)
-			w.w.PostMessageInfoError(msg, uid, err)
+			w.w.Errorf("could not emit message info: %v", err)
+			w.emitMessageInfoError(msg, uid, err)
 			continue
 		}
 	}
@@ -359,8 +410,8 @@ func (w *worker) handleFetchMessageHeaders(
 	return nil
 }
 
-func (w *worker) uidsFromQuery(query string) ([]uint32, error) {
-	msgIDs, err := w.db.MsgIDsFromQuery(query)
+func (w *worker) uidsFromQuery(ctx context.Context, query string) ([]uint32, error) {
+	msgIDs, err := w.db.MsgIDsFromQuery(ctx, query)
 	if err != nil {
 		return nil, err
 	}
@@ -391,12 +442,12 @@ func (w *worker) handleFetchMessageBodyPart(
 ) error {
 	m, err := w.msgFromUid(msg.Uid)
 	if err != nil {
-		log.Errorf("could not get message %d: %v", msg.Uid, err)
+		w.w.Errorf("could not get message %d: %v", msg.Uid, err)
 		return err
 	}
 	r, err := m.NewBodyPartReader(msg.Part)
 	if err != nil {
-		log.Errorf(
+		w.w.Errorf(
 			"could not get body part reader for message=%d, parts=%#v: %w",
 			msg.Uid, msg.Part, err)
 		return err
@@ -417,12 +468,12 @@ func (w *worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message %d: %v", uid, err)
+			w.w.Errorf("could not get message %d: %v", uid, err)
 			return err
 		}
 		r, err := m.NewReader()
 		if err != nil {
-			log.Errorf("could not get message reader: %v", err)
+			w.w.Errorf("could not get message reader: %v", err)
 			return err
 		}
 		defer r.Close()
@@ -446,25 +497,16 @@ func (w *worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.w.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.MarkAnswered(msg.Answered); err != nil {
-			log.Errorf("could not mark message as answered: %v", err)
-			w.err(msg, err)
-			continue
-		}
-		err = w.emitMessageInfo(m, msg)
-		if err != nil {
-			log.Errorf("could not emit message info: %v", err)
+			w.w.Errorf("could not mark message as answered: %v", err)
 			w.err(msg, err)
 			continue
 		}
 	}
-	w.w.PostMessage(&types.DirectoryInfo{
-		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
-	}, nil)
 	w.done(msg)
 	return nil
 }
@@ -473,38 +515,25 @@ func (w *worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.w.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
-		if err := m.SetFlag(msg.Flag, msg.Enable); err != nil {
-			log.Errorf("could not set flag %v as %t for message: %v", msg.Flag, msg.Enable, err)
-			w.err(msg, err)
-			continue
-		}
-		err = w.emitMessageInfo(m, msg)
-		if err != nil {
-			log.Errorf("could not emit message info: %v", err)
+		if err := m.SetFlag(msg.Flags, msg.Enable); err != nil {
+			w.w.Errorf("could not set flag %v as %t for message: %v",
+				msg.Flags, msg.Enable, err)
 			w.err(msg, err)
 			continue
 		}
 	}
-	w.w.PostMessage(&types.DirectoryInfo{
-		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
-	}, nil)
 	w.done(msg)
 	return nil
 }
 
 func (w *worker) handleSearchDirectory(msg *types.SearchDirectory) error {
-	// the first item is the command (search / filter)
-	s := strings.Join(msg.Argv[1:], " ")
-	// we only want to search in the current query, so merge the two together
-	search := w.query
-	if s != "" {
-		search = fmt.Sprintf("(%v) and (%v)", w.query, s)
-	}
-	uids, err := w.uidsFromQuery(search)
+	search := notmuch.AndQueries(w.query, translate(msg.Criteria))
+	log.Debugf("search query: '%s'", search)
+	uids, err := w.uidsFromQuery(msg.Context, search)
 	if err != nil {
 		return err
 	}
@@ -525,22 +554,7 @@ func (w *worker) handleModifyLabels(msg *types.ModifyLabels) error {
 		if err != nil {
 			return fmt.Errorf("could not modify message tags: %w", err)
 		}
-		err = w.emitMessageInfo(m, msg)
-		if err != nil {
-			return err
-		}
 	}
-	// tags changed, most probably some messages shifted to other folders
-	// so we need to re-enumerate the query content
-	err := w.emitDirectoryContents(msg)
-	if err != nil {
-		return err
-	}
-	// and update the list of possible tags
-	w.emitLabelList()
-	w.w.PostMessage(&types.DirectoryInfo{
-		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
-	}, nil)
 	w.done(msg)
 	return nil
 }
@@ -551,31 +565,14 @@ func (w *worker) loadQueryMap(acctConfig *config.AccountConfig) error {
 		// nothing to do
 		return nil
 	}
-	file, err := homedir.Expand(raw)
-	if err != nil {
-		return err
-	}
+	file := xdg.ExpandHome(raw)
 	f, err := os.Open(file)
 	if err != nil {
 		return err
 	}
 	defer f.Close()
-	w.nameQueryMap = make(map[string]string)
-	scanner := bufio.NewScanner(f)
-	for scanner.Scan() {
-		line := scanner.Text()
-		if line == "" || line[0] == '#' {
-			continue
-		}
-
-		split := strings.SplitN(line, "=", 2)
-		if len(split) != 2 {
-			return fmt.Errorf("%v: invalid line %q, want name=query", file, line)
-		}
-		w.nameQueryMap[strings.TrimSpace(split[0])] = split[1]
-		w.queryMapOrder = append(w.queryMapOrder, strings.TrimSpace(split[0]))
-	}
-	return nil
+	w.nameQueryMap, w.queryMapOrder, err = lib.ParseFolderMap(f)
+	return err
 }
 
 func (w *worker) loadExcludeTags(
@@ -595,19 +592,19 @@ func (w *worker) loadExcludeTags(
 
 func (w *worker) emitDirectoryContents(parent types.WorkerMessage) error {
 	query := w.query
+	ctx := context.Background()
 	if msg, ok := parent.(*types.FetchDirectoryContents); ok {
-		s := strings.Join(msg.FilterCriteria[1:], " ")
-		if s != "" {
-			query = fmt.Sprintf("(%v) and (%v)", query, s)
-		}
+		query = notmuch.AndQueries(query, translate(msg.Filter))
+		log.Debugf("filter query: '%s'", query)
+		ctx = msg.Context
 	}
-	uids, err := w.uidsFromQuery(query)
+	uids, err := w.uidsFromQuery(ctx, query)
 	if err != nil {
 		return fmt.Errorf("could not fetch uids: %w", err)
 	}
 	sortedUids, err := w.sort(uids, w.currentSortCriteria)
 	if err != nil {
-		log.Errorf("error sorting directory: %v", err)
+		w.w.Errorf("error sorting directory: %v", err)
 		return err
 	}
 	w.w.PostMessage(&types.DirectoryContents{
@@ -619,13 +616,15 @@ func (w *worker) emitDirectoryContents(parent types.WorkerMessage) error {
 
 func (w *worker) emitDirectoryThreaded(parent types.WorkerMessage) error {
 	query := w.query
+	ctx := context.Background()
+	threadContext := false
 	if msg, ok := parent.(*types.FetchDirectoryThreaded); ok {
-		s := strings.Join(msg.FilterCriteria[1:], " ")
-		if s != "" {
-			query = fmt.Sprintf("(%v) and (%v)", query, s)
-		}
+		query = notmuch.AndQueries(query, translate(msg.Filter))
+		log.Debugf("filter query: '%s'", query)
+		ctx = msg.Context
+		threadContext = msg.ThreadContext
 	}
-	threads, err := w.db.ThreadsFromQuery(query)
+	threads, err := w.db.ThreadsFromQuery(ctx, query, threadContext)
 	if err != nil {
 		return err
 	}
@@ -635,6 +634,18 @@ func (w *worker) emitDirectoryThreaded(parent types.WorkerMessage) error {
 	return nil
 }
 
+func (w *worker) emitMessageInfoError(msg types.WorkerMessage, uid uint32, err error) {
+	w.w.PostMessage(&types.MessageInfo{
+		Info: &models.MessageInfo{
+			Envelope: &models.Envelope{},
+			Flags:    models.SeenFlag,
+			Uid:      uid,
+			Error:    err,
+		},
+		Message: types.RespondTo(msg),
+	}, nil)
+}
+
 func (w *worker) emitMessageInfo(m *Message,
 	parent types.WorkerMessage,
 ) error {
@@ -642,19 +653,28 @@ func (w *worker) emitMessageInfo(m *Message,
 	if err != nil {
 		return fmt.Errorf("could not get MessageInfo: %w", err)
 	}
-	w.w.PostMessage(&types.MessageInfo{
-		Message: types.RespondTo(parent),
-		Info:    info,
-	}, nil)
+	switch {
+	case len(w.headersExclude) > 0:
+		info.RFC822Headers = lib.LimitHeaders(info.RFC822Headers, w.headersExclude, true)
+	case len(w.headers) > 0:
+		info.RFC822Headers = lib.LimitHeaders(info.RFC822Headers, w.headers, false)
+	}
+	switch parent {
+	case nil:
+		w.w.PostMessage(&types.MessageInfo{
+			Info: info,
+		}, nil)
+	default:
+		w.w.PostMessage(&types.MessageInfo{
+			Message: types.RespondTo(parent),
+			Info:    info,
+		}, nil)
+	}
 	return nil
 }
 
 func (w *worker) emitLabelList() {
-	tags, err := w.db.ListTags()
-	if err != nil {
-		log.Errorf("could not load tags: %v", err)
-		return
-	}
+	tags := w.db.ListTags()
 	w.w.PostMessage(&types.LabelList{Labels: tags}, nil)
 }
 
@@ -668,49 +688,41 @@ func (w *worker) sort(uids []uint32,
 	for _, uid := range uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.w.Errorf("could not get message: %v", err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			log.Errorf("could not get message info: %v", err)
+			w.w.Errorf("could not get message info: %v", err)
 			continue
 		}
 		msgInfos = append(msgInfos, info)
 	}
 	sortedUids, err := lib.Sort(msgInfos, criteria)
 	if err != nil {
-		log.Errorf("could not sort the messages: %v", err)
+		w.w.Errorf("could not sort the messages: %v", err)
 		return nil, err
 	}
 	return sortedUids, nil
 }
 
 func (w *worker) handleCheckMail(msg *types.CheckMail) {
+	defer log.PanicHandler()
 	if msg.Command == "" {
-		w.err(msg, fmt.Errorf("checkmail: no command specified"))
+		w.err(msg, fmt.Errorf("(%s) checkmail: no command specified", msg.Account()))
 		return
 	}
 	ctx, cancel := context.WithTimeout(context.Background(), msg.Timeout)
 	defer cancel()
 	cmd := exec.CommandContext(ctx, "sh", "-c", msg.Command)
-	ch := make(chan error)
-	go func() {
-		err := cmd.Run()
-		ch <- err
-	}()
-	select {
-	case <-ctx.Done():
-		w.err(msg, fmt.Errorf("checkmail: timed out"))
-	case err := <-ch:
-		if err != nil {
-			w.err(msg, fmt.Errorf("checkmail: error running command: %w", err))
-		} else {
-			w.w.PostMessage(&types.DirectoryInfo{
-				Info: w.getDirectoryInfo(w.currentQueryName, w.query),
-			}, nil)
-			w.done(msg)
-		}
+	err := cmd.Run()
+	switch {
+	case ctx.Err() != nil:
+		w.err(msg, fmt.Errorf("(%s) checkmail: timed out", msg.Account()))
+	case err != nil:
+		w.err(msg, fmt.Errorf("(%s) checkmail: error running command: %w", msg.Account(), err))
+	default:
+		w.done(msg)
 	}
 }
 
@@ -737,12 +749,12 @@ func (w *worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.w.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.Remove(path); err != nil {
-			log.Errorf("could not remove message: %v", err)
+			w.w.Errorf("could not remove message: %v", err)
 			w.err(msg, err)
 			continue
 		}
@@ -773,11 +785,11 @@ func (w *worker) handleCopyMessages(msg *types.CopyMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.w.Errorf("could not get message: %v", err)
 			return err
 		}
 		if err := m.Copy(dest); err != nil {
-			log.Errorf("could not copy message: %v", err)
+			w.w.Errorf("could not copy message: %v", err)
 			return err
 		}
 	}
@@ -809,7 +821,7 @@ func (w *worker) handleMoveMessages(msg *types.MoveMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			log.Errorf("could not get message: %v", err)
+			w.w.Errorf("could not get message: %v", err)
 			break
 		}
 		filenames, err := m.db.MsgFilenames(m.key)
@@ -827,7 +839,7 @@ func (w *worker) handleMoveMessages(msg *types.MoveMessages) error {
 			return fmt.Errorf("failed to parse message filename: %s", filenames[0])
 		}
 		if err := m.Move(source, dest); err != nil {
-			log.Errorf("could not copy message: %v", err)
+			w.w.Errorf("could not copy message: %v", err)
 			break
 		}
 		moved = append(moved, uid)
@@ -856,7 +868,7 @@ func (w *worker) handleAppendMessage(msg *types.AppendMessage) error {
 	}
 	key, writer, err := dest.Create(lib.ToMaildirFlags(msg.Flags))
 	if err != nil {
-		log.Errorf("could not create message at %s: %v", msg.Destination, err)
+		w.w.Errorf("could not create message at %s: %v", msg.Destination, err)
 		return err
 	}
 	filename, err := dest.Filename(key)
@@ -865,15 +877,22 @@ func (w *worker) handleAppendMessage(msg *types.AppendMessage) error {
 		return err
 	}
 	if _, err := io.Copy(writer, msg.Reader); err != nil {
-		log.Errorf("could not write message to destination: %v", err)
+		w.w.Errorf("could not write message to destination: %v", err)
 		writer.Close()
 		os.Remove(filename)
 		return err
 	}
 	writer.Close()
-	if _, err := w.db.IndexFile(filename); err != nil {
+	id, err := w.db.IndexFile(filename)
+	if err != nil {
 		return err
 	}
+
+	err = w.addFlags(id, msg.Flags)
+	if err != nil {
+		return err
+	}
+
 	w.w.PostMessage(&types.DirectoryInfo{
 		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
 	}, nil)
@@ -888,7 +907,7 @@ func (w *worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 
 	dir := w.store.Dir(msg.Directory)
 	if err := dir.Init(); err != nil {
-		log.Errorf("could not create directory %s: %v",
+		w.w.Errorf("could not create directory %s: %v",
 			msg.Directory, err)
 		return err
 	}
@@ -897,13 +916,19 @@ func (w *worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 }
 
 func (w *worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
-	if w.store == nil {
+	_, inQueryMap := w.nameQueryMap[msg.Directory]
+	if inQueryMap {
 		return errUnsupported
 	}
 
+	if w.store == nil {
+		w.done(msg)
+		return nil
+	}
+
 	dir := w.store.Dir(msg.Directory)
 	if err := os.RemoveAll(string(dir)); err != nil {
-		log.Errorf("could not remove directory %s: %v",
+		w.w.Errorf("could not remove directory %s: %v",
 			msg.Directory, err)
 		return err
 	}
@@ -938,9 +963,27 @@ func (w *worker) processNewMaildirFiles(dir string) error {
 		// Force message to move from new/ to cur/
 		err = w.db.MsgModifyTags(key, nil, nil)
 		if err != nil {
-			log.Errorf("MsgModifyTags failed: %v", err)
+			w.w.Errorf("MsgModifyTags failed: %v", err)
 		}
 	}
 
 	return nil
 }
+
+func (w *worker) addFlags(id string, flags models.Flags) error {
+	addTags := []string{}
+	removeTags := []string{}
+	for flag, tag := range flagToTag {
+		if !flags.Has(flag) {
+			continue
+		}
+
+		if flagToInvert[flag] {
+			removeTags = append(removeTags, tag)
+		} else {
+			addTags = append(addTags, tag)
+		}
+	}
+
+	return w.db.MsgModifyTags(id, addTags, removeTags)
+}
