@@ -1,11 +1,13 @@
 package types
 
 import (
+	"context"
 	"io"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/models"
+	"github.com/emersion/go-message/mail"
 )
 
 type WorkerMessage interface {
@@ -59,6 +61,10 @@ type Error struct {
 	Error error
 }
 
+type Cancelled struct {
+	Message
+}
+
 type ConnError struct {
 	Message
 	Error error
@@ -93,24 +99,29 @@ type ListDirectories struct {
 
 type OpenDirectory struct {
 	Message
+	Context   context.Context
 	Directory string
 }
 
 type FetchDirectoryContents struct {
 	Message
-	SortCriteria   []*SortCriterion
-	FilterCriteria []string
+	Context      context.Context
+	SortCriteria []*SortCriterion
+	Filter       *SearchCriteria
 }
 
 type FetchDirectoryThreaded struct {
 	Message
-	SortCriteria   []*SortCriterion
-	FilterCriteria []string
+	Context       context.Context
+	SortCriteria  []*SortCriterion
+	Filter        *SearchCriteria
+	ThreadContext bool
 }
 
 type SearchDirectory struct {
 	Message
-	Argv []string
+	Context  context.Context
+	Criteria *SearchCriteria
 }
 
 type DirectoryThreaded struct {
@@ -132,7 +143,8 @@ type RemoveDirectory struct {
 
 type FetchMessageHeaders struct {
 	Message
-	Uids []uint32
+	Context context.Context
+	Uids    []uint32
 }
 
 type FetchFullMessages struct {
@@ -148,7 +160,8 @@ type FetchMessageBodyPart struct {
 
 type FetchMessageFlags struct {
 	Message
-	Uids []uint32
+	Context context.Context
+	Uids    []uint32
 }
 
 type DeleteMessages struct {
@@ -160,7 +173,7 @@ type DeleteMessages struct {
 type FlagMessages struct {
 	Message
 	Enable bool
-	Flag   models.Flag
+	Flags  models.Flags
 	Uids   []uint32
 }
 
@@ -185,7 +198,7 @@ type MoveMessages struct {
 type AppendMessage struct {
 	Message
 	Destination string
-	Flags       []models.Flag
+	Flags       models.Flags
 	Date        time.Time
 	Reader      io.Reader
 	Length      int
@@ -198,6 +211,11 @@ type CheckMail struct {
 	Timeout     time.Duration
 }
 
+type StartSendingMessage struct {
+	Message
+	Header *mail.Header
+}
+
 // Messages
 
 type Directory struct {
@@ -207,8 +225,8 @@ type Directory struct {
 
 type DirectoryInfo struct {
 	Message
-	Info     *models.DirectoryInfo
-	SkipSort bool
+	Info    *models.DirectoryInfo
+	Refetch bool
 }
 
 type DirectoryContents struct {
@@ -270,3 +288,8 @@ type CheckMailDirectories struct {
 	Message
 	Directories []string
 }
+
+type MessageWriter struct {
+	Message
+	Writer io.WriteCloser
+}
