@@ -5,40 +5,60 @@ import (
 	"sync"
 	"sync/atomic"
 
-	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 )
 
+type WorkerInteractor interface {
+	log.Logger
+	Actions() chan WorkerMessage
+	ProcessAction(WorkerMessage) WorkerMessage
+	PostAction(WorkerMessage, func(msg WorkerMessage))
+	PostMessage(WorkerMessage, func(msg WorkerMessage))
+	Unwrap() WorkerInteractor
+}
+
 var lastId int64 = 1 // access via atomic
 
 type Backend interface {
 	Run()
+	Capabilities() *models.Capabilities
+	PathSeparator() string
 }
 
 type Worker struct {
 	Backend Backend
-	Actions chan WorkerMessage
-	Name    string
 
+	actions          chan WorkerMessage
 	actionCallbacks  map[int64]func(msg WorkerMessage)
 	messageCallbacks map[int64]func(msg WorkerMessage)
 	actionQueue      *list.List
 	status           int32
+	name             string
 
 	sync.Mutex
+	log.Logger
 }
 
 func NewWorker(name string) *Worker {
 	return &Worker{
-		Actions:          make(chan WorkerMessage),
-		Name:             name,
+		Logger:           log.NewLogger(name, 2),
+		actions:          make(chan WorkerMessage),
 		actionCallbacks:  make(map[int64]func(msg WorkerMessage)),
 		messageCallbacks: make(map[int64]func(msg WorkerMessage)),
 		actionQueue:      list.New(),
+		name:             name,
 	}
 }
 
+func (worker *Worker) Unwrap() WorkerInteractor {
+	return nil
+}
+
+func (worker *Worker) Actions() chan WorkerMessage {
+	return worker.actions
+}
+
 func (worker *Worker) setId(msg WorkerMessage) {
 	id := atomic.AddInt64(&lastId, 1)
 	msg.setId(id)
@@ -61,9 +81,10 @@ func (worker *Worker) queue(msg WorkerMessage) {
 	}
 }
 
-// Start processing the action queue and write all messages to the Actions
+// Start processing the action queue and write all messages to the actions
 // channel, one by one. Stop when the action queue is empty.
 func (worker *Worker) processQueue() {
+	defer log.PanicHandler()
 	for {
 		worker.Lock()
 		e := worker.actionQueue.Front()
@@ -74,7 +95,7 @@ func (worker *Worker) processQueue() {
 		}
 		msg := worker.actionQueue.Remove(e).(WorkerMessage)
 		worker.Unlock()
-		worker.Actions <- msg
+		worker.actions <- msg
 	}
 }
 
@@ -82,13 +103,7 @@ func (worker *Worker) processQueue() {
 // from the same goroutine that the worker runs in or deadlocks may occur
 func (worker *Worker) PostAction(msg WorkerMessage, cb func(msg WorkerMessage)) {
 	worker.setId(msg)
-
-	if resp := msg.InResponseTo(); resp != nil {
-		log.Tracef("PostAction %T:%T", msg, resp)
-	} else {
-		log.Tracef("PostAction %T", msg)
-	}
-	// write to Actions channel without blocking
+	// write to actions channel without blocking
 	worker.queue(msg)
 
 	if cb != nil {
@@ -98,20 +113,17 @@ func (worker *Worker) PostAction(msg WorkerMessage, cb func(msg WorkerMessage))
 	}
 }
 
+var WorkerMessages = make(chan WorkerMessage, 50)
+
 // PostMessage posts an message to the UI. This method should not be called
 // from the same goroutine that the UI runs in or deadlocks may occur
 func (worker *Worker) PostMessage(msg WorkerMessage,
 	cb func(msg WorkerMessage),
 ) {
 	worker.setId(msg)
-	msg.setAccount(worker.Name)
+	msg.setAccount(worker.name)
 
-	if resp := msg.InResponseTo(); resp != nil {
-		log.Tracef("PostMessage %T:%T", msg, resp)
-	} else {
-		log.Tracef("PostMessage %T", msg)
-	}
-	ui.MsgChannel <- msg
+	WorkerMessages <- msg
 
 	if cb != nil {
 		worker.Lock()
@@ -121,18 +133,14 @@ func (worker *Worker) PostMessage(msg WorkerMessage,
 }
 
 func (worker *Worker) ProcessMessage(msg WorkerMessage) WorkerMessage {
-	if resp := msg.InResponseTo(); resp != nil {
-		log.Tracef("ProcessMessage %T(%d):%T(%d)", msg, msg.getId(), resp, resp.getId())
-	} else {
-		log.Tracef("ProcessMessage %T(%d)", msg, msg.getId())
-	}
 	if inResponseTo := msg.InResponseTo(); inResponseTo != nil {
 		worker.Lock()
 		f, ok := worker.actionCallbacks[inResponseTo.getId()]
 		worker.Unlock()
 		if ok {
 			f(msg)
-			if _, ok := msg.(*Done); ok {
+			switch msg.(type) {
+			case *Cancelled, *Done:
 				worker.Lock()
 				delete(worker.actionCallbacks, inResponseTo.getId())
 				worker.Unlock()
@@ -143,11 +151,6 @@ func (worker *Worker) ProcessMessage(msg WorkerMessage) WorkerMessage {
 }
 
 func (worker *Worker) ProcessAction(msg WorkerMessage) WorkerMessage {
-	if resp := msg.InResponseTo(); resp != nil {
-		log.Tracef("ProcessAction %T(%d):%T(%d)", msg, msg.getId(), resp, resp.getId())
-	} else {
-		log.Tracef("ProcessAction %T(%d)", msg, msg.getId())
-	}
 	if inResponseTo := msg.InResponseTo(); inResponseTo != nil {
 		worker.Lock()
 		f, ok := worker.messageCallbacks[inResponseTo.getId()]
@@ -164,16 +167,6 @@ func (worker *Worker) ProcessAction(msg WorkerMessage) WorkerMessage {
 	return msg
 }
 
-// PostMessageInfoError posts a MessageInfo message to the worker when an
-// error was encountered fetching the message header
-func (worker *Worker) PostMessageInfoError(msg WorkerMessage, uid uint32, err error) {
-	worker.PostMessage(&MessageInfo{
-		Info: &models.MessageInfo{
-			Envelope: &models.Envelope{},
-			Flags:    []models.Flag{models.SeenFlag},
-			Uid:      uid,
-			Error:    err,
-		},
-		Message: RespondTo(msg),
-	}, nil)
+func (worker *Worker) PathSeparator() string {
+	return worker.Backend.PathSeparator()
 }
